/*
 * LCD-Wecker mit grafischem Display (240x128 Pixel) und Controller UC1611s, Scott-Falk Hhn, "main.c", v1.02, 04.03.2016
 * AtmelStudio 7.x Editor-Einstellung: Tools / Options / Text Editor / GCC / Tabs / Smart, Tab size: 2, Indent size: 2, Insert spaces
 *
 * Controller: ATmega1284P
 * Taktfrequenz 10,0 MHz
 * Betriebsspannung: 3,3V
 *
 * Verwendung der I/O-Ports:
 * PA0 - Touchscreen-Anschluss BOTTOM
 * PA1 - Touchscreen-Anschluss LEFT
 * PA2 - Touchscreen-Anschluss TOP
 * PA3 - Touchscreen-Anschluss RIGHT
 * PA7 - Eingang vom Lichtsensor
 * PB0 - Ausgang zum LCD SPI-CLK
 * PB1 - Eingang vom Sensor/Taster mit Pull-up
 * PB2 - Ausgang fr externe Alarm-Signalisierung 1, High-aktiv (Dauer-aktiv whrend Alarm)
 * PB3 - Ausgang PWM fr die Steuerung der LCD-Beleuchtung
 * PB4 - Ausgang fr externe Alarm-Signalisierung 2, High-aktiv (1Hz-Impulse whrend Alarm)
 * PB5 - Anschluss ISP-MOSI
 * PB6 - Anschluss ISP-MISO
 * PB7 - Anschluss ISP-SCK
 * RES - Anschluss ISP-RESET
 * PC6 - Ausgang zum LCD SPI-CD
 * PC7 - Ausgang zum LCD SPI-CS0
 * PD1 - Ausgang zum LCD SPI-SI
 * PD2 - Eingang RXD vom USB-Modul
 * PD3 - Ausgang TXD zum USB-Modul
 * PD4 - Ausgang zur LED, Low-aktiv
 * PD5 - Ausgang Weck-Ton-Signal
 * PD6 - Eingang vom DCF77-Empfnger, wahlweise mit Pull-up
 * PD7 - Ausgang PWM fr die Steuerung der Lautstrke
 *
 * Verwendung der Timer:
 * Timer0A - PWM fr die Steuerung der LCD-Beleuchtung (2451Hz)
 * Timer1A - Erzeugung der Weck-Tne
 * Timer2A - PWM fr die Steuerung der Lautstrke (19608Hz)
 * Timer3A - Interrupt, Abfrage von Touchscreeens, Sensor/Taster, Lichtsensor und DCF77-Signal, Zeitzhlung (50Hz / 20ms)
 *
 * Verwendung der USART:
 * USART0 - TXD im SPI-Modus, Taktfrequenz 5MHz, Datenausgabe an das LCD
 * USART1 - RXD im Asynchron-Modus 8N1, Baudrate 125k, Empfang ber Interrupt, Erkennung einer Bootloader-Anfrage
 *
 * Verwendung des ADC:
 * Abfrage des Touchscreens (PA0-PA3) und des Lichtsensors (PA7), Vorteiler 64, ADC-Takt: 156,25kHz
 *
 * Einstellung der Fuse-Bits:
 * BODLEVEL =  2V7
 * OCDEN =     [ ]
 * JTAGEN =    [ ]
 * SPIEN =     [X]
 * WDTON =     [ ]
 * EESAVE =    [X]
 * BOOTSZ =    1024W_FC00
 * BOOTRST =   [X]
 * CKDIV8 =    [ ]
 * CKOUT =     [ ]
 * SUT_CKSEL = EXTXOSC_8MHZ_XX_16KCK_65MS
 * EXTENDED =  0xFD
 * HIGH =      0xD4
 * LOW =       0xFF
 */ 

// Definitionen und Bibliotheken

#define F_CPU 10000000UL          // Taktfrequenz 10MHz
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <string.h>

const char strd00[] PROGMEM = "v1.02, 04.03.17, Scott-Falk Hhn";

#include "charset.h"              // Zeichenstze fr Ziffern und Texte
#include "strings.h"              // Texte in Deutsch und Englisch
#include "tables.h"               // Tabellen fr Mens und Touchscreen-Bedienung
#include "sound.h"                // Frequenztabellen und Notenlisten fr Weck-Tne

#define UCPHA0 1                  // Definition fr Bit 1 in UCSR0C: SPI-Phase, fehlt in der "iom1284p.h"
#define SPI_BAUD 5000000UL        // SPI-Taktfrequenz 5MHz
#define RS_BAUD 125000UL          // RS-232 Baudrate fr Bootloader 125000
#define TIM3_CLK 50               // Timer3 Taktfrequenz 50Hz
#define YEARH 20                  // Jahrhundert
#define YEAR 16                   // Jahr Anfangswert und Minimalwert fr manuelle Zeiteinstellung
#define MONTH 6                   // Monat Anfangswert
#define HOUR 0                    // Stunde Anfangswert
#define ALMHOUR 6                 // Weck-Stunde Anfangswert
#define TS_THRH 30                // ADC-Schwellwert zur Erkennung einer Touchscreen-Berhrung
#define TS_LONG 35                // Anzahl der 20ms-Interrupt-Zyklen zur Erkennung einer langen Touchscreen-Berhrung (700ms)
#define TS_AUTO 6                 // Anzahl der 20ms-Interrupt-Zyklen fr die Auto-Repeat-Funktion (120ms)
#define LCDCONL 100               // LCD-Kontrast, unterer Endwert
#define LCDCONH 160               // LCD-Kontrast, oberer Endwert
#define LCDCOND 132               // LCD-Kontrast, Voreinstellung
#define BRIMIND 5                 // Minimale Helligkeit, Voreinstellung
#define BRIMAXD 5                 // Maximale Helligkeit, Voreinstellung
#define DCFTOUT 30                // Zeit in Minuten bis zur Wiedereinschaltung des Displays bei fester DCF77-Synchronisierungszeit (30min)
#define MSGTOUT 120               // Zeit in 20ms-Zyklen fr die Anzeige einer Meldung bei Sensor/Taster-Bettigung (2s)
#define MSGW 120                  // Breite der Meldung bei Sensor/Taster-Bettigung in Pixel, wird zentriert angezeigt
#define MSGH 50                   // Hhe der Meldung bei Sensor/Taster-Bettigung in Pixel, wird zentriert angezeigt
#define FLASHCNT 50               // Blink-Zeit in 20ms-Interrupt-Zyklen fr LED und Externen Alarmausgang 2 (1Hz)
#define SYNCTOUT 25               // Zeit in Stunden bis DCF77-Neusynchronisierung erforderlich ist
#define MENUTOUT 120              // Zeit in Sekunden bis zur automatischen Rckkehr in die normale Anzeige
#define ALMTOUT 60                // Alarm-Timeout in Sekunden (Zeitdauer eines Alarmsignals)
#define ALMDIS 15                 // Zeit in Minuten fr Weckzeit-Deaktivierung ber Sensor/Taster vor dem geplanten Weck- oder Schlummersignal
#define CNTVOL 30                 // Anzahl der 20ms-Interrupt-Zyklen fr Erhhung der Lautstrke im ansteigenden Modus
#define DCFHOU 16                 // Feste Synchronisierungszeit 16:00 Uhr

#define TST_DDR DDRA              // Port-Parameter fr Touchscreen-Anschluss TOP
#define TST_PRT PORTA
#define TST_SIG PINA2
#define TST_ADC 2
#define TSB_DDR DDRA              // Port-Parameter fr Touchscreen-Anschluss BOTTOM
#define TSB_PRT PORTA
#define TSB_SIG PINA0
#define TSL_DDR DDRA              // Port-Parameter fr Touchscreen-Anschluss LEFT
#define TSL_PRT PORTA
#define TSL_SIG PINA1
#define TSL_ADC 1
#define TSR_DDR DDRA              // Port-Parameter fr Touchscreen-Anschluss RIGHT
#define TSR_PRT PORTA
#define TSR_SIG PINA3
#define LIS_DDR DDRA              // Port-Parameter fr Lichtsensor (Eingang, Analog)
#define LIS_SIG PINA7
#define LIS_ADC 7
#define CLK_DDR DDRB              // Port-Parameter fr SPI-CLK (Ausgang)
#define CLK_SIG PINB0
#define SI_DDR DDRD               // Port-Parameter fr SPI-SI (Ausgang)
#define SI_PRT PORTD
#define SI_SIG PIND0
#define CD_DDR DDRC               // Port-Parameter fr SPI-CD (Ausgang)
#define CD_PRT PORTC
#define CD_SIG PINC6
#define CS0_DDR DDRC              // Port-Parameter fr SPI-CS0 (Ausgang)
#define CS0_PRT PORTC
#define CS0_SIG PINC7
#define OC0A_DDR DDRB             // Port-Parameter fr OC0A (PWM-Ausgang fr LCD-Beleuchtung)
#define OC0A_SIG PINB3
#define OC1A_DDR DDRD             // Port-Parameter fr OC1A (Weck-Ton-Ausgang)
#define OC1A_SIG PIND5
#define OC2A_DDR DDRD             // Port-Parameter fr OC2A (PWM-Ausgang fr Lautstrke)
#define OC2A_SIG PIND7
#define SEN_PRT PORTB             // Port-Parameter fr Sensor/Taster (Eingang mit Pull-up)
#define SEN_PIN PINB
#define SEN_SIG PINB1
#define DCF_PRT PORTD             // Port-Parameter fr DCF77-Eingang (Eingang, wahlweise mit Pull-up)
#define DCF_PIN PIND
#define DCF_SIG PIND6
#define LED_PRT PORTD             // Port-Parameter fr Alarm-LED-Ausgang
#define LED_DDR DDRD
#define LED_SIG PIND4
#define EX1_PRT PORTB             // Port-Parameter fr Externen Alarm-Ausgang 1
#define EX1_DDR DDRB
#define EX1_SIG PINB2
#define EX2_PRT PORTB             // Port-Parameter fr Externen Alarm-Ausgang 2
#define EX2_DDR DDRB
#define EX2_SIG PINB4

// EEPROM-Adressen (werden als Konstanten definiert, um eine feste Adresse zu erhalten)

const uint16_t e_lang   =   0;    // feste EEPROM-Adresse 000 - Lnge 001 Byte  - Sprache (siehe lang)
const uint16_t e_dcfsyn =   1;    // feste EEPROM-Adresse 001 - Lnge 001 Byte  - Feste DCF-Synchronisierungzeit (siehe dcfsyn)
const uint16_t e_dcfinv =   2;    // feste EEPROM-Adresse 002 - Lnge 001 Byte  - DCF-Invertierung (siehe dcfinv)
const uint16_t e_dcfpup =   3;    // feste EEPROM-Adresse 003 - Lnge 001 Byte  - DCF-Pull-Up (siehe dcfpup)
const uint16_t e_ledmod =   4;    // feste EEPROM-Adresse 004 - Lnge 001 Byte  - LED-Modus (siehe ledmode)
const uint16_t e_brimin =   5;    // feste EEPROM-Adresse 005 - Lnge 001 Byte  - Minimale Helligkeit (siehe brimin)
const uint16_t e_brimax =   6;    // feste EEPROM-Adresse 006 - Lnge 001 Byte  - Maximale Helligkeit (siehe brimax)
const uint16_t e_tsleft =   7;    // feste EEPROM-Adresse 007 - Lnge 001 Byte  - Touchscreen X-Wert am linken Rand (siehe tsleft)
const uint16_t e_tsrigh =   8;    // feste EEPROM-Adresse 008 - Lnge 001 Byte  - Touchscreen X-Wert am rechten Rand (siehe tsright)
const uint16_t e_tstop  =   9;    // feste EEPROM-Adresse 009 - Lnge 001 Byte  - Touchscreen Y-Wert am oberen Rand (siehe tstop)
const uint16_t e_tsbott =  10;    // feste EEPROM-Adresse 010 - Lnge 001 Byte  - Touchscreen Y-Wert am unteren Rand (siehe tsbottom)
const uint16_t e_aldate =  11;    // feste EEPROM-Adresse 011 - Lnge 001 Byte  - Anzeige Alarmdatum (siehe aldate)
const uint16_t e_lcdcon =  12;    // feste EEPROM-Adresse 012 - Lnge 001 Byte  - LCD-Kontrast (siehe lcdcon)
const uint16_t e_dcfhou =  13;    // feste EEPROM-Adresse 013 - Lnge 001 Byte  - DCF-Synchronisierungsstunde (siehe dcfhou)
const uint16_t e_soundn =  14;    // feste EEPROM-Adresse 014 - Lnge 001 Byte  - Weck-Ton-Nummer (siehe soundnr)
const uint16_t e_volume =  15;    // feste EEPROM-Adresse 015 - Lnge 001 Byte  - Lautstrke (siehe volume)
const uint16_t e_soundo =  16;    // feste EEPROM-Adresse 016 - Lnge 001 Byte  - Lautsprecher-Status (siehe soundon)
const uint16_t e_curpro =  17;    // feste EEPROM-Adresse 017 - Lnge 001 Byte  - Aktuelles Weck-Programm (siehe curprog)
const uint16_t e_almpro =  18;    // feste EEPROM-Adresse 018 - Lnge 228 Bytes - Weck-Programm-Tabelle (siehe almprog)
const uint16_t e_almpre = 246;    // feste EEPROM-Adresse 246 - Lnge 006 Bytes - Voreingestellte Weckzeiten (siehe almpre)
const uint16_t e_snooze = 252;    // feste EEPROM-Adresse 252 - Lnge 001 Byte  - Schlummerzeit (siehe snooze)
const uint16_t e_almpos = 253;    // feste EEPROM-Adresse 253 - Lnge 001 Byte  - Weck-Programm-Position (siehe almpos)
const uint16_t e_digset = 254;    // feste EEPROM-Adresse 254 - Lnge 001 Byte  - Groziffernsatz (siehe digset)
const uint16_t e_vomode = 255;    // feste EEPROM-Adresse 255 - Lnge 001 Byte  - Lautstrke-Modus (siehe vomode)
const uint16_t e_lday   = 256;    // feste EEPROM-Adresse 256 - Lnge 001 Byte  - Letzte Alarm-Reorganisierung Tag (siehe lday)
const uint16_t e_lmonth = 257;    // feste EEPROM-Adresse 257 - Lnge 001 Byte  - Letzte Alarm-Reorganisierung Monat (siehe lmonth)
const uint16_t e_lyear  = 258;    // feste EEPROM-Adresse 258 - Lnge 001 Byte  - Letzte Alarm-Reorganisierung Jahr (siehe lyear)
const uint16_t e_nxtalm = 259;    // feste EEPROM-Adresse 259 - Lnge 009 Bytes - Nchste Weckzeiten (siehe nxtalm)
const uint16_t e_mode   = 268;    // feste EEPROM-Adresse 268 - Lnge 001 Byte  - Wecker-Modus (siehe mode)
const uint16_t e_autost = 269;    // feste EEPROM-Adresse 269 - Lnge 001 Byte  - Sommerzeit-Automatik (siehe autosumt)

// Variablen

static uint8_t lcdbuf[3840];      // LCD-Puffer fr 240x128 Pixel, 1 Byte enthlt die Daten von 8 bereinander liegenden Pixeln:
                                  //   Bit0 - Pixel in Zeile n
                                  //   Bit1 - Pixel in Zeile n + 1
                                  //   Bit2 - Pixel in Zeile n + 2
                                  //   Bit3 - Pixel in Zeile n + 3
                                  //   Bit4 - Pixel in Zeile n + 4
                                  //   Bit5 - Pixel in Zeile n + 5
                                  //   Bit6 - Pixel in Zeile n + 6
                                  //   Bit7 - Pixel in Zeile n + 7
static char strbuf[64];           // Stringpuffer fr die Textausgabe
static char strbu1[32];           // Stringpuffer fr Teilstring 1 nach automatischem Umbruch
static char strbu2[32];           // Stringpuffer fr Teilstring 2 nach automatischem Umbruch
static uint8_t bright[16];        // Helligkeits-Puffer, enthlt die letzten 16 im Abstand von 20 ms gelesenen ADC-Werte vom Helligkeitssensor
static uint8_t briavg;            // Mittelwert aller Daten im Helligkeits-Puffer
static uint8_t brimin;            // Minimum fr die Helligkeitssteuerung (0-9)
static uint8_t brimax;            // Maximum fr die Helligkeitssteuerung (0-9)
static uint8_t posx;              // X-Koordinate fr die Textausgabe (0-239)
static uint8_t posy;              // Y-Koordinate fr die Textausgabe (0-127)
static uint8_t basex;             // Basis-X-Koordinate eines Bereiches (Links, 0-239)
static uint8_t endx;              // Ende-X-Koordinate eines Bereiches (Rechts, 0-239), wird fr rechtsbndige Ausgaben verwendet
static uint8_t centx;             // Mitte-X-Koordinate eines Bereiches (0-239), wird fr zentrierte Ausgaben verwendet
static uint8_t basey;             // Basis-Y-Koordinate eines Bereiches Oben (0-127)
static uint8_t tsrawx;            // Touchscreen X-Raw-Wert (0-255)
static uint8_t tsrawy;            // Touchscreen Y-Raw-Wert (0-255)
static uint8_t tsposx;            // berechnete X-Koordinate aus den Werten tsleft, tsright und tsrawx (0-239)
static uint8_t tsposy;            // berechnete Y-Koordinate aus den Werten tstop, tsbottom und tsrawy (0-127)
static uint8_t tsleft;            // Touchscreen X-Wert am linken Rand (0-255)  |
static uint8_t tsright;           // Touchscreen X-Wert am rechten Rand (0-255) |  werden durch Kalibrierung ermittelt und zur
static uint8_t tstop;             // Touchscreen Y-Wert am oberen Rand (0-255)  |  Berechnung der exakten Koordinaten bentigt
static uint8_t tsbottom;          // Touchscreen Y-Wert am unteren Rand (0-255) |
static uint8_t tsx1;              // Zwischenspeicher fr Touchscreen-Kalibrierwert X oben links
static uint8_t tsy1;              // Zwischenspeicher fr Touchscreen-Kalibrierwert Y oben links
static uint8_t tsx2;              // Zwischenspeicher fr Touchscreen-Kalibrierwert X oben rechts
static uint8_t tsy2;              // Zwischenspeicher fr Touchscreen-Kalibrierwert Y oben rechts
static uint8_t tsx3;              // Zwischenspeicher fr Touchscreen-Kalibrierwert X unten links
static uint8_t tsy3;              // Zwischenspeicher fr Touchscreen-Kalibrierwert Y unten links
static uint8_t tsx4;              // Zwischenspeicher fr Touchscreen-Kalibrierwert X unten rechts
static uint8_t tsy4;              // Zwischenspeicher fr Touchscreen-Kalibrierwert Y unten rechts
static uint8_t tscalibr;          // Touchscreen-Kalibrierung (0-1), 0 = gltige Kalibrierwerte aus EEPROM gelesen, 1 = keine gltigen Werte im EEPROM -> Kalibrierung erforderlich
static uint8_t tssold;            // Touchscreen-Status vom vorherigen Interrupt (0-1)
static uint8_t tsstat;            // Touchscreen-Berhrungsstatus (0-1), 0 = Losgelassen, 1 = Berhrung (aktive Berhrung an Hauptprogramm bermitteln)
static uint8_t tsflag;            // Touchscreen-Flag (0-1), 0 = Berhrung quittiert, 1 = Berhrung muss quittiert werden (zeigt Hauptprogramm, dass noch eine Berhrung bearbeitet werden muss)
static uint8_t tslong;            // Zhler fr lange Berhrung (0-TS_LONG), wird alle 20ms weitergezhlt und bei Erreichen des Wertes TS_LONG gestoppt, Erkennung einer langen Berhrung
static uint8_t tsquit;            // Touchscreen-Quittungsstatus (0-1), 1 = Touchscreen-Berhrung wurde quittiert
static uint8_t sesold;            // Sensor/Taster-Status vom vorherigen Interrupt (0-255)
static uint8_t sestat;            // Sensor/Taster-Status (0-1), 0 = Losgelassen, 1 = gedrckt
static uint8_t sequit;            // Sensor/Taster-Quittierungsstatus (0-1), 1 = Sensor/Taster-Bettigung wurde quittiert
static uint8_t lang;              // Sprache (0-1), 0 = Deutsch, 1 = Englisch
static uint8_t digset;            // Groziffernsatz (0-1), 0 = Groziffernsatz 1 (rund), 1 = Groziffernsatz 2 (LED)
static uint8_t mode;              // Wecker-Modus (0-1), 0 = Programm, 1 = Manuell
static uint8_t lcdcon;            // LCD-Kontrast (100-160)
static uint8_t menlevel;          // Men-Ebene (0-3), 0 = Hauptmen-Ebene, 1-3 untergeordnete Men-Ebenen, die zugehrige Men-Nummer steht in menstack[menlevel]
static uint8_t menstack[4];       // Men-Stack, Speicher fr Men-Nummern zur die Rckkehr in die hhere Men-Ebene
static uint8_t menu;              // aktuelle Men-Nummer (0-n), 0 = normale Anzeige
static uint8_t area;              // aktuelle Bereichsnummer (0-n), wird nach Touchscreen-Berhrung ermittelt
static uint8_t actpend;           // anstehende Aktion nach Touchscreen-Berhrung (0-255), 0 = keine Aktion, 1-255 = Aktionsnummer aus der Mentabelle
static uint8_t actshort;          // Aktion bei kurzer Touchscreen-Berhrung (0-255), wird aus Mentabelle gelesen
static uint8_t actlong;           // Aktion bei langer Touchscreen-Berhrung (0-255), wird aus Mentabelle gelesen
static uint8_t okflag;            // OK-Flag (0-1), 0 = OK-Bereich ist unsichtbar, 1 = OK-Bereich ist sichtbar
static uint8_t clkmode;           // Zeitmodus (0-1), 0 = normale Zeitanzeige, 1 = Anzeige der Sekunden
static uint8_t syncstat;          // Status der DCF77-Synchronsierung (0-4):
                                  //   0 = Uhr wurde noch nie synchronisert
                                  //   1 = Uhr wurde manuell gestellt
                                  //   2 = Uhr wurde mindestens 24h nicht synchronisiert
                                  //   3 = Uhr wurde innerhalb 24h synchronisiert
static uint8_t soundon;           // Lautsprecher-Status (0-1), 0 = Ton aus, 1 = Ton ein
static uint8_t volume;            // Lautstrke (0-9)
static uint8_t vomode;            // Lautstrke-Modus (0-1), 0 = feste Lautstrke, 1 = ansteigende Lautstrke
static uint8_t soundnr;           // Weck-Ton-Nummer (0-9)
static uint8_t snstat;            // Status der Weck-Ton-Erzeugung:
                                  //   0 = aus (Ruhezustand)
                                  //   1 = Start, wird vom Hauptprogramm gesetzt, die Interrupt-Routine liest die aktuellen Weck-Ton-Einstellungen und setzt den Status auf 2 (Aktiv)
                                  //   2 = Aktiv, Weck-Ton-Programm wird komplett abgespielt, anschlieend werden alle Weck-Ton-Einstellungen neu geladen und das Programm erneut abgespielt
                                  //   3 = Stopp,  wird vom Hauptprogramm gesetzt, die Weck-Ton-Erzeugung wird sofort gestoppt und setzt anschlieend den Status 0 (Ruhezustand)
static PGM_P sndptr;              // Zeiger auf ausgewhltes Weck-Ton-Programm
static uint8_t sndnot;            // Notenwert, 0-72 = Note, 255 = Ende
static uint8_t sndlen;            // Tonlnge in 20ms-Einheiten (0-255)
static uint8_t sndrel;            // Release-Wert in Schritten je 20ms (0-255)
static uint8_t curvol;            // aktueller Lautstrkewert (0-255)
static uint8_t maxvol;            // maximaler Lautstrkewert (0-255)
static uint8_t endvol;            // Lautstrke-Endwert bei ansteigendem Lautstrke-Modus (0-255)
static uint8_t cntvol;            // Zhler fr die Lautstrke-Erhhung bei ansteigendem Modus (0-CNTVOL), nach Erreichen des Endwertes wird die Lautstrke erhht und der Zhler zurckgesetzt
static uint8_t intcnt;            // Interrupt-Zhler (0-49), reduziert die Frequenz von Timer3: 50Hz -> 1Hz (20ms -> 1s)
static uint8_t autcnt;            // Auto-Repeat-Zhler (0-TS_AUTO), wird alle 20ms weitergezhlt, bei Erreichen des Wertes TS_AUTO wird der gerade editierte Wert automatisch erhht oder vermindert
static uint8_t ledmode;           // LED-Modus (0-1), 0 = aus, 1 = Alarmanzeige, 2 = DCF77-Status anzeigen
static uint8_t dcflstat;          // DCF77-Signal-Status beim vorherigen Interrupt (0-1)
static uint8_t dcfflag;           // DCF77-Flag, Signalisierung des Minutenwechsels (0-1)
static uint8_t dcfcnt;            // DCF77-Zhler zur Ermittlung der Impulslnge (0-n)
static uint8_t dcfimp;            // DCF77-Zhler zur Ermittlung der Impulsanzahl (0-n)
static uint8_t dcfsyn;            // Feste DCF77-Synchronisierungszeit (0-1), 0 = aus, 1 = feste Zeit mit Abschaltung des LCD und der Hintergrundbeleuchtung zur sicheren DCF-Synchronisierung
static uint8_t dcfinv;            // DCF77-Invertierung (0-1), 0  = DCF-Empfnger liefert Low-Impulse, 1 = DCF-Empfnger liefert High-Impulse
static uint8_t dcfpup;            // DCF77-Pull-up (0-1), 0 = Pull-up ausgeschaltet, 1 = Pull-up eingeschaltet
static uint8_t dcfhou;            // DCF77-Synchronisierungsstunde (0-23)
static uint8_t dcftab[42];        // DCF77-Tabelle, enthlt empfangene Bits, wobei jedes Bit ein Byte der Tabelle belegt. Eingehende Bits werden in das letzte Byte eingeschoben und alle weiteren
                                  // Bytes um eine Position weitergerckt; die Bytes enthalten nach einem kompletten Minutenzyklus folgende Daten:
                                  //   Byte  0:    Zeitzone, 0 = Winterzeit, 1 = Sommerzeit
                                  //   Byte  1:    Zeitzone, 1 = Winterzeit, 0 = Sommerzeit
                                  //   Byte  2:    Schaltsekunde (nicht ausgewertet)
                                  //   Byte  3:    Startbit (nicht ausgewertet)
                                  //   Byte  4- 7: Minute Einer
                                  //   Byte  8-10: Minute Zehner
                                  //   Byte 11:    Prfbit 1 (Minuten)
                                  //   Byte 12-15: Stunde Einer
                                  //   Byte 16-17: Stunde Zehner
                                  //   Byte 18:    Prfbit 2 (Stunden)
                                  //   Byte 19-22: Kalendertag Einer
                                  //   Byte 23-24: Kalendertag Zehner
                                  //   Byte 25-27: Wochentag
                                  //   Byte 28-31: Monat Einer
                                  //   Byte 32:    Monat Zehner
                                  //   Byte 33-36: Jahr Einer
                                  //   Byte 37-40: Jahr Zehner
                                  //   Byte 41:    Prfbit 3 (Datum)
static uint8_t dcfbuf1[7];        // DCF-Puffer 1, enthlt die aktuelle dekodierte Uhrzeit:
                                  //   Byte 0: Zeitzone (0-1), 0 = MEZ, 1 = MESZ
                                  //   Byte 1: Minute (0-59)
                                  //   Byte 2: Stunde (0-23)
                                  //   Byte 3: Kalendertag (1-31)
                                  //   Byte 4: Wochentag (1-7)
                                  //   Byte 5: Monat (1-12)
                                  //   Byte 6: Jahr (0-99)
static uint8_t dcfbuf2[7];        // DCF-Puffer 2, enthlt die dekodierte Uhrzeit eine Minute zuvor (korrigiert um eine Minute), gleicher Aufbau wie dcbuf1
static uint8_t dcfbuf3[7];        // DCF-Puffer 3, enthlt die dekodierte Uhrzeit eine Minute zuvor (korrigiert um eine Minute), gleicher Aufbau wie dcbuf1
static uint8_t autosumt;          // Sommerzeit-Automatik (0-1), 0 = aus, 1 = ein
static uint8_t timezone;          // Zeitzone (0-1), 0 = MEZ (Normalzeit), 1 = MESZ (Sommerzeit)
static uint8_t second;            // Sekunden-Zhler (0-59)
static uint8_t minute;            // Minuten-Zhler (0-59)
static uint8_t hour;              // Stunden-Zhler (0-23)
static uint8_t day;               // Tages-Zhler (1-31)
static uint8_t wday;              // Wochentags-Zhler (1-7)
static uint8_t month;             // Monatszhler (1-12)
static uint8_t year;              // Jahres-Zhler (0-99)
static uint8_t xminute;           // Minutenwert fr Zeiteinstellung (0-59)
static uint8_t xhour;             // Stundenwert fr Zeiteinstellung (0-23)
static uint8_t xday;              // Tageswert fr Zeiteinstellung (1-31)
static uint8_t xmonth;            // Monatswert fr Zeiteinstellung (1-12)
static uint8_t xyear;             // Jahreswert fr Zeiteinstellung (0-99)
static uint8_t xautost;           // Sommerzeit-Automatik fr Zeiteinstellung (0-1)
static uint8_t aldate;            // Weckzeit-Datum (0-1), 0 = Weckzeit-Anzeige mit Datum, 1 = Weckzeit-Anzeige alternativ "Heute" und "Morgen"
static uint8_t alarms[4][6];      // Weckzeit-Tabelle mit den nchsten 3 Ereignissen und Zeit der letzten DCF77-Synchronisierung:
                                  //   [n]... - Weckzeit-Nummer (0-2), letzte DCF77-Synchronisierung (3)
                                  //   ...[0] - Weckzeit-Status (0-1), 0 = Weckzeit inaktiv, 1 = Weckzeit aktiv, bei letzter DCF77-Synchronisierung: 0 = MEZ, 1 = MESZ
                                  //   ...[1] - Minute (0-59)
                                  //   ...[2] - Stunde (0-23)
                                  //   ...[3] - Tag (1-31)
                                  //   ...[4] - Monat (1-12)
                                  //   ...[5] - Jahr (0-99)
static uint8_t curprog;           // aktuelles Weck-Programm (0-3)
static uint8_t almprog[4][57];    // Weck-Programm-Tabelle:
                                  //   [n].... - Weck-Programm (0-3)
                                  //   ...[00] - Weckzeit Tag 1, Minute (0-59)
                                  //   ...[01] - Weckzeit Tag 1, Stunde (0-23, 24 = Weckzeit inaktiv)
                                  //   ...[54] - Weckzeit Tag 28, Minute (0-59)
                                  //   ...[55] - Weckzeit Tag 28, Stunde (0-23, 24 = Weckzeit inaktiv)
                                  //   ...[56] - Anzahl der Tage (1-28)
static uint8_t almpos;            // Weck-Programm-Position (0-27)
static uint8_t xalnum;            // Alarmnummer fr die Weckzeit-Einstellung (0-27, entspricht Feld 1-28)
static uint8_t xalmin;            // Minute fr die Weckzeit-Einstellung (0-59)
static uint8_t xalhou;            // Stunde fr die Weckzeit-Einstellung (0-23, 24 = Weckzeit inaktiv)
static uint8_t xalmod;            // Modus fr die Weckzeit-Einstellung (0-1), 0 = Einstellung der Weck-Programm-Tabelle, 1 = Einstellung einer direkten Weckzeit
static uint8_t almpre[3][2];      // Voreingestellte Weckzeiten:
                                  //   [n]... - Weckzeit (0-2)
                                  //   ...[0] - Weckzeit Minute (0-59)
                                  //   ...[1] - Weckzeit Stunde (0-23)
static uint8_t snooze;            // Schlummerzeit in Minuten (0-15)
static uint8_t tday;              // Temporrer Tag fr Ermittlung der Weckzeiten
static uint8_t tmonth;            // Temporrer Monat fr Ermittlung der Weckzeiten
static uint8_t tyear;             // Temporres Jahr fr Ermittlung der Weckzeiten
static uint8_t talmpos;           // Temporre Weck-Programm-Position
static uint8_t lday;              // Letzte Alarm-Reorganisierung Tag (1-31)   | Daten werden bei einer Alarm-Reorganisierung gespeichert und stndig mit dem aktuellen Datum verglichen, bei Ungleich-
static uint8_t lmonth;            // Letzte Alarm-Reorganisierung Monat (1-12) | heit wird eine Alarm-Reorganisierung ausgelst
static uint8_t lyear;             // Letzte Alarm-Reorganisierung Jahr (0-99)  |
static uint8_t almreorg;          // Alarm-Reorganisierungs-Flag (0-1), 1 = Alarm-Reorganisierung durchfhren
static uint8_t nxtalm[3][3];      // Nchste Weckzeiten:
                                  //   [n]... - Weckzeit (0-2), 0 = Heute, 1 = Morgen, 2 = bermorgen
                                  //   ...[0] - Weckzeit-Status (0-1), 0 = Weckzeit nach Programm, 1 = Weckzeit berschrieben mit nachfolgender Zeit
                                  //   ...[1] - Minute (0-59)
                                  //   ...[2] - Stunde (0-23, 24 = Weckzeit inaktiv)
static uint8_t curalm[7];         // Aktuelle Weckzeit:
                                  //   [0] - Weckzeit-Status (0-1), 0 = Weckzeit inaktiv, 1 = auf Schlummerzeit warten, 2 = Schlummerzeit aktiv, 3 = auf Weckzeit warten, 4 = Weckzeit aktiv, 5 = Ende
                                  //   [1] - Minute (0-59) Weckzeit
                                  //   [2] - Stunde (0-23) Weckzeit
                                  //   [3] - Minute (0-59) Schlummerzeit
                                  //   [4] - Stunde (0-23) Schlummerzeit
                                  //   [5] - Minute (0-59) Deaktivierungszeit
                                  //   [6] - Stunde (0-23) Deaktivierungszeit
static uint8_t lastrsrx[2];       // RS-232-Empfangspuffer fr 2 Zeichen
static uint8_t lcdoff;            // LCD-Abschaltung (0-1), 0 = Normalbetrieb, 1 = LCD-Kommunikation ausgeschaltet whrend DCF77-Synchronisierung
static uint8_t almflag;           // Alarm-Flag (0-1), 0 = Weck-Signal inaktiv, 1 = Weck-Signal aktiv
static uint8_t flashcnt;          // Blink-Zhler, wird fr das Blinken der LED und den Externen Alarm-Ausgang 2 bentigt
static uint8_t count1;            // freier Zhler 1 fr Timeouts (Vorteiler)
static uint8_t count2;            // freier Zhler 2 fr Timeouts (Sekunden)
static uint8_t count3;            // freier Zhler 3 fr Timeouts (Minuten)
static uint8_t msgnum;            // Meldungsnummer (0-4), wird zusammen mit einem Symbol zu einer Meldung generiert und dann in einem Fenster ber der normalen Anzeige eingeblendet:
                                  //   0 = Keine Funktion
                                  //   1 = Schlummern
                                  //   2 = Alarm aus
                                  //   3 = Weck-Ton aus
                                  //   4 = Weck-Ton ein
static uint8_t msgtout;           // Meldungs-Timeout-Zhler, wird bei Bettigung des Sensors/Tasters gestartet; whrend der Laufzeit erscheint eine Meldung ber dem normalen Bild; nach Ablauf der
                                  // Zeit wird der Zhler gestoppt und zur vorherigen Anzeige zurckgeschaltet
static uint8_t dcftout;           // DCF77-Timeout-Zhler, wird bei fester DCF77-Synchronisierungszeit (dcfsyn = 1) nach dem Systemstart und zur Synchronisierungsstunde gestartet; whrend der Lauf-
                                  // zeit wird das Display ausgeschaltet; nach erfolgreicher Synchronisierung oder nach Ablauf der Zeit wird der Zhler gestoppt und das Display wieder eingeschaltet
static uint8_t synctout;          // Synchronisierungs-Timeout-Zhler, nach Ablauf der Zeit wird der Synchronisierungs-Status auf 2 gesetzt; wird vom Hauptprogramm gesetzt und vom Timer-Interrupt im
                                  // Stundentakt auf 0 gezhlt, das Hauptprogramm setzt nach Bearbeitung den Wert auf 255 (Stopp)
static uint8_t menutout;          // Men-Timeout-Zhler, nach Ablauf der Zeit wird zur normalen Anzeige zurckgekehrt; wird vom Hauptprogramm gesetzt und vom Timer-Interrupt im Sekundentakt auf 0
                                  // gezhlt, das Hauptprogramm setzt nach Bearbeitung den Wert auf 255 (Stopp)
static uint8_t almtout;           // Alarm-Timeout-Zhler, Zeitdauer eines Alarmsignals, nach Ablauf der Zeit wird der Alarm beendet; wird vom Hauptprogramm gesetzt und vom Timer-Interrupt im Sekun-
                                  // dentakt auf 0 gezhlt, das Hauptprogramm setzt nach Bearbeitung den Wert auf 255 (Stopp)

#ifdef SCRSHOT                    // wenn Screenshot-Definition aktiv
static uint8_t usbflag;           // USB-Flag, wird bei aktiver Screenshot-Funktion vom Timer-Interrupt gesetzt und vom Hauptprogramm wieder gelscht
#endif

// Interrupt-Routinen ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

ISR(TIMER3_COMPA_vect)                                                // wird alle 20ms (50Hz) aufgerufen; Abfrage des Lichtsensors, des Touchscreens, des DCF77-Signals und Zeitzhlung
{
  int8_t i;                                                           // Schleifenzhler
  uint8_t bitval;                                                     // Wert eines empfangenen DCF77-Bits (0 oder 1)
  uint8_t dcfstat;                                                    // Zwischenspeicher fr DCF77-Status
  uint8_t dcftemp;                                                    // Zwischenspeicher fr die Bearbeitung der DCF77-Tabelle
  uint16_t brs;                                                       // Summenwert fr die Helligkeitssteuerung
  uint8_t bra;                                                        // Zwischenspeicher fr den Helligkeits-Mittelwert
  uint8_t brt;                                                        // Zwischenspeicher fr die Helligkeitsbegrenzung
  uint8_t tsx;                                                        // Zwischenspeicher fr Touchscreen-X-Wert
  uint8_t tsy;                                                        // Zwischenspeicher fr Touchscreen-Y-Wert
  uint8_t tsz;                                                        // Zwischenspeicher fr Touchscreen-Z-Wert
  uint8_t tst;                                                        // Zwischenspeicher fr Touchscreen-Berhrungsstatus
  uint8_t set;                                                        // Zwischenspeicher fr Sensor/Taster

                                                                      // --- DCF77 bearbeiten
  dcfcnt ++;                                                          // DCF77-Zhler erhhen
  dcfstat = (DCF_PIN & _BV(DCF_SIG));                                 // aktuellen DCF77-Status lesen
  if (dcfinv) dcfstat = !dcfstat;                                     // falls Invertierung aktiviert -> invertieren
  if (!dcflstat) {                                                    // wenn vorheriges DCF77-Signal Low
    if (dcfstat) {                                                    // wenn aktuelles DCF77-Signal High (entspricht Impuls-Start)
      if ((dcfcnt > 2) && (dcfcnt < 13)) {                            // wenn Impulslnge 60-260ms -> gltiger Impuls
        dcfimp ++;                                                    // Impulsanzahl erhhen
        if (dcfcnt > 7) bitval = 1;                                   // wenn Impulslnge > 140ms -> 1-Bit
        else bitval = 0;                                              // sonst -> 0-Bit
        for (i = 41; i >= 0; i --) {                                  // 42 Pufferbytes bearbeiten (von hinten beginnend)
          dcftemp = dcftab[i];                                        // Pufferbyte holen und merken
          dcftab[i] = bitval;                                         // neues Pufferbyte setzen
          bitval = dcftemp;                                           // bertrag fr nchstes Pufferbyte
        }
      }
    }
  }
  else {                                                              // wenn vorheriges DCF77-Signal High
    if (!dcfstat) {                                                   // wenn aktuelles DCF77-Signal Low (entspricht Impuls-Ende)
      if (dcfcnt > 75) dcfflag = 1;                                   // wenn Impulslnge > 1,5s -> neuer Minutenbeginn
      if (dcfcnt > 45) dcfcnt = 0;                                    // wenn Imuplslnge > 900ms -> Zhler auf 0 setzen
    }
  }
  dcflstat = dcfstat;                                                 // neuen DCF77-Status speichern
  if (ledmode == 2) {                                                 // wenn LED-Modus auf DCF77-Status gesetzt
    if (dcfstat) LED_PRT |= _BV(LED_SIG);                             // wenn DCF77-Status = 1 -> High an PortD4 ausgeben (LED ausschalten)
    else LED_PRT &= ~_BV(LED_SIG);                                    // sonst Low an PortD4 ausgeben (LED einschalten)
  }
                                                                      // --- Lichtsensor lesen
  ADCSRA |= _BV(ADSC);                                                // ADC starten, Multiplexer ist bereits auf Lichtsensor eingestellt
  brs = 0;                                                            // Summenwert fr die Helligkeitssteuerung lschen
  for (i = 0; i < 15; i ++) {                                         // 15 Puffereintrge bearbeiten
    bright[i] = bright[i + 1];                                        // Puffereintrag verschieben
    brs += bright[i];                                                 // Puffereintrag zum Summenwert addieren
  }
  while (ADCSRA & _BV(ADSC));                                         // warten bis ADC fertig ist
  bright[15] = ADCH;                                                  // ADC lesen und in Pufferende eintragen
  brs += bright[15];                                                  // aktuellen Wert ebenfalls zum Summenwert addieren
  bra = brs >> 4;                                                     // Helligkeits-Mittelwert zwischenspeichern
  briavg = bra;                                                       // Helligkeits-Mittelwert fr System-Informationen speichern
  brt = pgm_read_byte(&bmitab[brimin]);                               // Wert fr Minimale Helligkeit aus Tabelle holen
  if (bra < brt) bra = brt;                                           // wenn Mittelwert unter dem Minimalwert -> Mittelwert auf Minimalwert setzen
  brt = pgm_read_byte(&bmatab[brimax]);                               // Wert fr Maximale Helligkeit aus Tabelle holen
  if (bra > brt) bra = brt;                                           // wenn Mittelwert ber dem Maximalwert -> Mittelwert auf Maximalwert setzen
  if (dcftout == 255) OCR0A = bra;                                    // wenn keine DCF77-Synchronisierung aktiv -> ermittelten PWM-Wert anwenden
  else OCR0A = 0;                                                     // sonst PWM = 0 (Hintergrundbeleuchtung ausschalten)

                                                                      // --- Touchscreen-Anschluss Z-Wert lesen (LEFT = Low, RIGHT = Low, BOTTOM = Hi-Z, TOP = Pull-Up -> Spannung enthlt Z-Wert)
  TST_DDR &= ~_BV(TST_SIG);                                           // Touchscreen-Anschluss TOP auf Eingang (Hi-Z) setzen
  TSR_DDR |= _BV(TSR_SIG);                                            // Touchscreen-Anschluss RIGHT auf Ausgang setzen (ist bereits auf Low gesetzt)
  TST_PRT |= _BV(TST_SIG);                                            // Touchscreen-Anschluss TOP Pull-up einschalten
  ADMUX = (ADMUX & ~(_BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1) | _BV(MUX0))) | TST_ADC; // ADC-Multiplexer auf Touchscreen-Anschluss TOP setzen
  _delay_us(10);                                                      // 10s warten
  ADCSRA |= _BV(ADSC);                                                // ADC starten
  while (ADCSRA & _BV(ADSC));                                         // warten bis ADC fertig ist
  tsz = ADCH;                                                         // ermittelten Z-Wert zwischenspeichern
  tsx = 0;                                                            // Touchscreen-X-Wert zunchst lschen
  tsy = 0;                                                            // Touchscreen-Y-Wert zunchst lschen

  if (tsz < TS_THRH) {                                                // wenn eine Berhrung erkannt wurde
                                                                      // --- Touchscreen X-Wert lesen (LEFT = Low, RIGHT = High, BOTTOM = Hi-Z, TOP = Hi-Z -> Spannung enthlt X-Wert)
    TST_PRT &= ~_BV(TST_SIG);                                         // Touchscreen-Anschluss TOP Pull-up ausschalten (ist bereits auf Eingang gesetzt)
    TSR_PRT |= _BV(TSR_SIG);                                          // Touchscreen-Anschluss RIGHT auf High setzen (ist bereits auf Ausgang gesetzt)
    _delay_us(10);                                                    // ADC-Multiplexer ist noch auf Touchscreen-Anschluss TOP gesetzt, 10s warten
    ADCSRA |= _BV(ADSC);                                              // ADC starten
    while (ADCSRA & _BV(ADSC));                                       // warten bis ADC fertig ist
    tsx = ADCH;                                                       // ermittelten X-Wert zwischenspeichern

                                                                      // --- Touchscreen Y-Wert lesen (TOP = Low, BOTTOM = High, RIGHT = Hi-Z, LEFT = Hi-Z -> Spannung enthlt Y-Wert)
    TSL_DDR &= ~_BV(TSL_SIG);                                         // Touchscreen-Anschluss LEFT auf Eingang (Hi-Z) umschalten
    TSR_PRT &= ~_BV(TSR_PRT);                                         // Touchscreen-Anschluss RIGHT auf Low setzen
    TSR_DDR &= ~_BV(TSR_SIG);                                         // Touchscreen-Anschluss RIGHT auf Eingang  (Hi-Z) umschalten
    TST_PRT &= ~_BV(TST_SIG);                                         // Touchscreen-Anschluss TOP Pull-up ausschalten
    TST_DDR |= _BV(TST_SIG);                                          // Touchscreen-Anschluss TOP auf Ausgang setzen
    TSB_DDR |= _BV(TSB_SIG);                                          // Touchscreen-Anschluss BOTTOM auf Ausgang setzen
    TSB_PRT |= _BV(TSB_SIG);                                          // Touchscreen-Anschluss BOTTOM auf High setzen
    ADMUX = (ADMUX & ~(_BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1) | _BV(MUX0))) | TSL_ADC; // ADC-Multiplexer auf Touchscreen-Anschluss LEFT setzen
    _delay_us(10);                                                    // 10s warten
    ADCSRA |= _BV(ADSC);                                              // ADC starten
    while (ADCSRA & _BV(ADSC));                                       // warten bis ADC fertig ist
    tsy = ADCH;                                                       // ermittelten Y-Wert zwischenspeichern
  }
                                                                      // --- Touchscreen-Anschlsse wieder in Ruhezustand setzen
  TSB_PRT &= ~_BV(TSB_SIG);                                           // Touchscreen-Anschluss BOTTOM auf Low setzen
  TSB_DDR &= ~_BV(TSB_SIG);                                           // Touchscreen-Anschluss BOTTOM auf Eingang (Hi-Z) setzen
  TSR_PRT &= ~_BV(TSR_SIG);                                           // Touchscreen-Anschluss RIGHT auf Low setzen
  TSR_DDR &= ~_BV(TSR_SIG);                                           // Touchscreen-Anschluss RIGHT auf Eingang (Hi-Z) setzen
  TSL_PRT &= ~_BV(TSL_SIG);                                           // Touchscreen-Anschluss LEFT auf Low setzen
  TSL_DDR |= _BV(TSL_SIG);                                            // Touchscreen-Anschluss LEFT auf Ausgang setzen
  TST_PRT &= ~_BV(TST_SIG);                                           // Touchscreen-Anschluss TOP auf Low setzen
  TST_DDR |= _BV(TST_SIG);                                            // Touchscreen-Anschluss TOP auf Ausgang setzen
  ADMUX = (ADMUX & ~(_BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1) | _BV(MUX0))) | LIS_ADC; // ADC Multiplexer auf Lichtsensor-Eingang setzen

                                                                      // --- Touchscreen-Bedienung erkennen und auswerten
  tst = 0;                                                            // Berhrungsstatus zunchst lschen
  if (tsz < TS_THRH) tst = 1;                                         // wenn Z-Wert unterhalb der Erkennungsschwelle (Berhrung) -> Berhrungsstatus setzen
  if (tst == tssold) {                                                // wenn Berhrungsstatus identisch mit Status beim letzten Interrupt (Zustand stabil)
    if (tst) {                                                        // wenn Berhrung erkannt
      tsstat = 1;                                                     // Status auf "Berhrung" setzen
      tsflag = 1;                                                     // Berhrungs-Flag setzen
      tsrawx = tsx;                                                   // aktuellen X-Wert bergeben
      tsrawy = tsy;                                                   // aktuellen Y-Wert bergeben
      if (tslong < TS_LONG) tslong ++;                                // wenn Endwert fr lange Berhrung noch nicht erreicht -> Zhler erhhen
    }
    else {                                                            // wenn keine Berhrung mehr
      tsstat = 0;                                                     // Status auf "Losgelassen" setzen
      if (tsquit) {                                                   // wenn Status vom Hauptprogramm quittiert wurde
        tsquit = 0;                                                   // Quittierungs-Status lschen
        tsflag = 0;                                                   // Berhrungs-Flag lschen
        tslong = 0;                                                   // Zhler fr lange Berhrung lschen
      }        
    }
  }
  tssold = tst;                                                       // Berhrungsstatus fr nchsten Interrupt speichern

                                                                      // --- Sensor/Taster erkennen und auswerten
  set = ~SEN_PIN & _BV(SEN_SIG);                                      // Sensor/Tasterstatus lesen, invertieren und filtern
  if (set == sesold) {                                                // wenn aktueller Status identisch mit Status beim letzten Interrupt
    if (set) {                                                        // wenn Sensor/Taster gedrckt
      sestat = 1;                                                     // aktuellen Status auf "Gedrckt" setzen
    }
    else {                                                            // wenn Sensor/Taster losgelassen
      sestat = 0;                                                     // aktuellen Status auf "Losgelassen" setzen
      sequit = 0;                                                     // Quittierungs-Status lschen
    }
  }
  sesold = set;                                                       // Sensor/Tasterstatus fr nchsten Interrupt speichern

                                                                      // --- Weck-Ton-Erzeugung
  if (snstat == 1) {                                                  // wenn Weck-Ton-Erzeugung gestartet
    sndptr = (PGM_P) pgm_read_word(&sounds[soundnr]);                 // Pointer auf Weck-Ton-Sound setzen
    endvol = (volume + 21) * 5;                                       // Lautstrke-Endwert in PWM-Wert umrechnen (0-9 -> 105-150)
    if (vomode) maxvol = 110;                                         // wenn ansteigende Lautstrke -> maximale Lautstrke zunchst auf kleinen Wert setzen
    else maxvol = endvol;                                             // wenn feste Lautstrke -> maximale Lautstrke auf Endwert setzen
    snstat = 2;                                                       // Weck-Ton-Erzeugung auf aktiv setzen
    curvol = 0;                                                       // aktuelle Lautstrke auf 0 setzen
    sndnot = 0;                                                       // Notenwert = 0 setzen
    sndlen = 0;                                                       // Tonlnge = 0 setzen
    cntvol = 0;                                                       // Zhler fr Lautstrke-Erhhung zurcksetzen
    if (soundon) TCCR1B |= _BV(CS11);                                 // wenn Lautsprecher eingeschaltet -> Timer1 Vorteiler 8 aktivieren
    else maxvol = 0;                                                  // wenn Lautsprecher ausgeschaltet -> Lautstrke = 0 setzen
  }
  if (snstat == 2) {                                                  // wenn Weck-Ton-Erzeugung aktiv
    if (sndlen == 0) {                                                // wenn Tonlnge = 0 (Ton komplett abgespielt)
      sndnot = pgm_read_byte(sndptr ++);                              // nchsten Notenwert holen
      if (sndnot < 255) {                                             // wenn weitere Tne folgen
        sndlen = pgm_read_byte(sndptr ++);                            // nchste Tonlnge holen
        sndrel = pgm_read_byte(sndptr ++);                            // nchsten Release-Wert holen
        if (OCR1A != pgm_read_word(&freqdata_array[sndnot])) {        // wenn sich der Teilerwert ndert
          OCR1A = pgm_read_word(&freqdata_array[sndnot]);             // Timer1-OCR1A mit neuem Teilerwert laden
        }
        if (TCNT1 >= OCR1A) TCNT1 = OCR1A - 1;                        // wenn aktueller Zhlerwert von Timer1 zu gro -> Zhlerwert in zulssigen Bereich setzen (Tonaussetzer verhindern)
        curvol = maxvol;                                              // auf voreingestellte Lautstrke setzen (Ton einschalten)
      }
      else {                                                          // wenn keine weiteren Tne folgen -> Weck-Ton-Erzeugung neu starten
        sndptr = (PGM_P) pgm_read_word(&sounds[soundnr]);             // Pointer auf Weck-Ton-Sound setzen
        endvol = (volume + 21) * 5;                                   // Lautstrke-Endwert in PWM-Wert umrechnen (0-9 -> 105-150)
        if (!vomode) maxvol = endvol;                                 // wenn feste Lautstrke -> maximale Lautstrke auf Endwert setzen
        curvol = 0;                                                   // aktuelle Lautstrke auf 0 setzen
        sndnot = 0;                                                   // Notenwert = 0 setzen
        sndlen = 0;                                                   // Tonlnge = 0 setzen
        if (soundon) TCCR1B |= _BV(CS11);                             // wenn Lautsprecher eingeschaltet -> Timer1 Vorteiler 8 aktivieren
        else maxvol = 0;                                              // wenn Lautsprecher ausgeschaltet -> Lautstrke = 0 setzen
      }      
    }
  }
  if (snstat == 3) {                                                  // wenn Weck-Ton-Erzeugung beendet werden soll
    snstat = 0;                                                       // Weck-Ton-Erzeugung in Ruhezustand setzen
    curvol = 0;                                                       // aktuelle Lautstrke auf 0 setzen
    TCCR1B &= ~_BV(CS11);                                             // Timer1 stoppen (Ton ausschalten)
  }
  if (curvol >= sndrel) curvol -= sndrel;                             // Lautstrke vermindern, wenn Ergebnis im positiven Bereich
  else curvol = 0;                                                    // sonst Lautstrke = 0 setzen
  OCR2A  = curvol;                                                    // neuen Lautstrkewert an PWM-Register ausgeben
  if (sndlen > 0) sndlen --;                                          // wenn Tonlnge nicht komplett abgespielt -> 20ms abziehen
  cntvol ++;                                                          // Zhler fr Lautstrke-Erhhung erhhen
  if (cntvol >= CNTVOL) {                                             // wenn Endwert erreicht
    cntvol = 0;                                                       // Zhler zurcksetzen
    if (maxvol < endvol) maxvol ++;                                   // wenn Lautstrke-Endwert noch nicht erreicht -> Lautstrke erhhen
  }

  if (msgtout < 255) {                                                // wenn Meldungs-Timeout-Zhler aktiv
    if (msgtout > 0) msgtout --;                                      // wenn Meldungs-Timeout-Zhler noch nicht abgelaufen -> Timeout-Zhler vermindern
  }
                                                                      // --- Alarm-Signalalisierung
  if (almflag) {                                                      // wenn Weck-Signal aktiv
    EX1_PRT |= _BV(EX1_SIG);                                          // Externen Alarm-Ausgang 1 aktivieren
    if (flashcnt < FLASHCNT / 2) {                                    // wenn Blinkzhlerstand in der unteren Hlfte
      EX2_PRT |= _BV(EX2_SIG);                                        // Externen Alarm-Ausgang 2 aktivieren
      if (ledmode == 1) LED_PRT &= ~_BV(LED_SIG);                     // wenn LED-Modus auf Alarm-Anzeige -> LED einschalten
    }
    else {                                                            // wenn Blinkzhlerstand in der oberen Hlfte
      EX2_PRT &= ~_BV(EX2_SIG);                                       // Externen Alarm-Ausgang 2 deaktivieren
      if (ledmode == 1) LED_PRT |= _BV(LED_SIG);                      // wenn LED-Modus auf Alarm-Anzeige -> LED ausschalten
    }
    flashcnt ++;                                                      // Blinkzhler erhhen
    if (flashcnt >= FLASHCNT) flashcnt = 0;                           // wenn Ende erreicht -> Zhler zurcksetzen
  }
  else {                                                              // wenn Weck-Signal inaktiv
    EX1_PRT &= ~_BV(EX1_SIG);                                         // Externen Alarm-Ausgang 1 deaktivieren
    EX2_PRT &= ~_BV(EX2_SIG);                                         // Externen Alarm-Ausgang 2 deaktivieren
    if (ledmode == 1) LED_PRT |= _BV(LED_SIG);                        // wenn LED-Modus auf Alarm-Anzeige -> LED ausschalten
  }

  autcnt ++;                                                          // Auto-Repeat-Zhler erhhen
  intcnt ++;                                                          // Interrupt-Zhler erhhen

  #ifdef SCRSHOT                                                      // wenn Screenshot-Definition aktiv
  if ((intcnt == 4) && ((second % 10) == 0)) usbflag = 1;             // wenn Sekunde durch 10 teilbar und Interrupt-Zhler 4 -> USB-Flag setzen (Screenshot alle 10 Sekunden, Doppelpunkt an)
  #endif

  if (intcnt > 49) {                                                  // wenn Endwert berschritten
    intcnt = 0;                                                       // Interrupt-Zhler zurcksetzen

                                                                      // --- der folgende Abschnitt wird alle 1s (1Hz) ausgefhrt
    uint8_t mflag = 0;                                                // Merker fr Monatswechsel lschen
    second ++;                                                        // Sekunden erhhen
    if (second > 59) {                                                // wenn berlauf
      second = 0;                                                     // Sekunden zurcksetzen

                                                                      // --- der folgende Abschnitt wird alle 1min ausgefhrt
      minute ++;                                                      // Minuten erhhen
      if (minute > 59) {                                              // wenn berlauf
        minute = 0;                                                   // Minuten zurcksetzen
        hour ++;                                                      // Stunden erhhen
        if (autosumt) {                                               // wenn Sommerzeit-Automatik eingeschaltet
          if ((day > 24) && (wday == 7)) {                            // wenn Wechsel-Zeitraum und Sonntag
            if ((month == 3) && (hour == 2) && (timezone == 0)) {     // wenn Mrz, 2:00 Uhr und noch Normalzeit
              hour ++;                                                // eine Stunde vorstellen
              timezone = 1;                                           // Zeitzone auf MESZ setzen
            }            
            if ((month == 10) && (hour == 3) && (timezone == 1)) {    // wenn Oktober, 3:00 Uhr und noch Sommerzeit
              hour --;                                                // eine Stunde zurckstellen
              timezone = 0;                                           // Zeitzone auf MEZ setzen
            }            
          }          
        }
        if (hour > 23) {                                              // wenn berlauf
          hour = 0;                                                   // Stunden zurcksetzen
          wday ++;                                                    // Wochentag erhhen
          if (wday > 7) wday = 1;                                     // wenn Wochentagberlauf -> Wochentag zurcksetzen
          day ++;                                                     // Tag erhhen
          if ((year % 4) == 0) {                                      // wenn Schaltjahr
            if (day > pgm_read_byte_far(&daytab2[month - 1]))         // wenn Tag > Schaltjahr-Tabelle
              mflag = 1;                                              // Monatswechsel
          }
          else {                                                      // wenn kein Schaltjahr
            if (day > pgm_read_byte_far(&daytab1[month - 1]))         // wenn Tag > Normaljahr-Tabelle
              mflag = 1;                                              // Monatswechsel
          }
          if (mflag) {                                                // wenn Monatswechsel erforderlich
            day = 1;                                                  // Tag zurcksetzen
            month ++;                                                 // Monat erhhen
            if (month > 12) {                                         // wenn berlauf
              month = 1;                                              // Monat zurcksetzen
              year ++;                                                // Jahr erhhen
            }
          }
        }
      }      
    }
  }                                                                   // --- Zweite Zhlkette fr Timeout-Zhler (Sekunden, Minuten und Stunden)
  count1 ++;                                                          // freien Zhler 1 erhhen (Vorteiler)
  if (count1 > 49) {                                                  // wenn Endwert berschritten
    count1 = 0;                                                       // freien Zhler 1 zurcksetzen
                                                                      // --- der folgende Abschnitt wird alle 1s (1Hz) ausgefhrt
    if (menutout < 255) {                                             // wenn Men-Timeout-Zhler aktiv
      if (menutout > 0) menutout --;                                  // wenn Men-Timeout-Zhler noch nicht abgelaufen -> Timeout-Zhler vermindern
    }
    if (almtout < 255) {                                              // wenn Alarm-Timeout-Zhler aktiv
      if (almtout > 0) almtout --;                                    // wenn Alarm-Timeout-Zhler noch nicht abgelaufen -> Timeout-Zhler vermindern
    }
    count2 ++;                                                        // freien Zhler 2 erhhen (Sekunden)
    if (count2 > 59) {                                                // wenn Endwert berschritten
      count2 = 0;                                                     // freien Zhler 2 zurcksetzen
                                                                      // --- der folgende Abschnitt wird alle 1min ausgefhrt
      if (dcftout < 255) {                                            // wenn DCF-Synchronisierungs-Timeout-Zhler aktiv
        if (dcftout > 0) dcftout --;                                  // wenn DCF-Synchronisierungs-Timeout-Zhler noch nicht abgelaufen -> Timeout-Zhler vermindern
      }
      count3 ++;                                                      // freien Zhler 3 erhhen (Minuten)
      if (count3 > 59) {                                              // wenn Endwert berschritten
        count3 = 0;                                                   // freien Zhler 3 zurcksetzen
                                                                      // --- der folgende Abschnitt wird alle 1h ausgefhrt
        if (synctout < 255) {                                         // wenn Synchronisierungs-Timeout-Zhler aktiv
          if (synctout > 0) synctout--;                               // wenn Synchronisierungs-Timeout-Zhler noch nicht abgelaufen -> Timeout-Zhler vermindern
        }
      }
    }
  }
}

ISR(USART1_RX_vect)                                                   // Zeichen ber USART1 empfangen
{
  lastrsrx[0] = lastrsrx[1];                                          // vorheriges Zeichen im Puffer verschieben
  lastrsrx[1] = UDR1;                                                 // neues Zeichen im Puffer speichern
}

// Funktionen -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void lcd_start(void)                                                  // Datenbertragung zum LCD starten
{
  if (!lcdoff) {                                                      // wenn LCD im Normalbetrieb
    CS0_PRT &= ~_BV(CS0_SIG);                                         // Anschluss CS0 auf Low setzen
  }
}

void lcd_stop(void)                                                   // Datenbertragung zum LCD stoppen
{
  if (!lcdoff) {                                                      // wenn LCD im Normalbetrieb
    while (!(UCSR0A & _BV(TXC0)));                                    // warten solange noch Daten bertragen werden
    CS0_PRT |= _BV(CS0_SIG);                                          // Anschluss CS0 auf High setzen
  }  
}

void lcd_out(uint8_t cd, uint8_t byte)                                // Ein Byte an das LCD senden
{                                                                     // cd: 0 = Control, 1 = Data), byte: zu sendendes Byte
  if (!lcdoff) {                                                      // wenn LCD im Normalbetrieb
    while (!(UCSR0A & _BV(TXC0)));                                    // warten solange noch Daten bertragen werden
    UCSR0A |= _BV(TXC0);                                              // TXC0-Bit lschen
    if (cd) CD_PRT |= _BV(CD_SIG);                                    // wenn cd = 1 -> Anschluss CD auf High setzen
    else CD_PRT &= ~_BV(CD_SIG);                                      // sonst Anschluss CD auf Low setzen
    UDR0 = byte;                                                      // Byte senden
  }
}

void send_buf(void)                                                   // LCD-Puffer an LCD ausgeben
{
  int i;                                                              // Zhlervariable
  if (!lcdoff) {                                                      // wenn LCD im Normalbetrieb
    lcd_start();                                                      // Datenbertragung zum LCD starten
    lcd_out(0, 0x00);                                                 // Set Column Address LSB: auf 0 setzen
    lcd_out(0, 0x10);                                                 // Set Column Address MSB: auf 0 setzen
    lcd_out(0, 0x60);                                                 // Set Page Address LSB: auf 0 setzen
    lcd_out(0, 0x70);                                                 // Set Page Address MSB: auf 0 setzen
    for (i = 0; i < 3840; i ++)                                       // 3840 Bytes (240 x 16) ausgeben
    {
      lcd_out(1, lcdbuf[i]);                                          // Pixeldaten fr 8 bereinander liegende Pixel ausgeben
    }
    lcd_stop();                                                       // Datenbertragung zum LCD stoppen
  }
}

void set_lcdcon(void)                                                 // LCD-Kontrast einstellen
{
  if (!lcdoff) {                                                      // wenn LCD im Normalbetrieb
    lcd_start();                                                      // Datenbertragung zum LCD starten
    lcd_out(0, 0x81); lcd_out(0, lcdcon);                             // Set Potentiometer: LCD-Kontrast setzen
    lcd_stop();                                                       // Datenbertragung zum LCD stoppen
  }  
}

void lcd_off(void)                                                    // LCD ausschalten (Sleep-Modus) und Kommunikation ausschalten
{
  lcd_start();                                                        // Datenbertragung zum LCD starten
  lcd_out(0, 0xa8);                                                   // Set Display Enable: Sleep-Modus
  lcd_stop();                                                         // Datenbertragung zum LCD stoppen
  _delay_us(10);                                                      // Wartezeit nach Datenblatt
  lcdoff = 1;                                                         // LCD-Kommunikation auschalten
}

void lcd_on(void)                                                     // LCD-einschalten (Normal-Modus)
{
  lcdoff = 0;                                                         // LCD-Kommuniaktion einschalten
  lcd_start();                                                        // Datenbertragung zum LCD starten
  lcd_out(0, 0xa9);                                                   // Set Display Enable: Schwarz/Wei-Modus
  lcd_stop();                                                         // Datenbertragung zum LCD stoppen
  _delay_us(10);                                                      // Wartezeit nach Datenblatt
}

void clr_buf(void)                                                    // LCD-Puffer lschen
{
  int i;                                                              // Zhlervariable
  for (i = 0; i < 3840; i ++)                                         // Schleife fr 3840 Bytes
  {
    lcdbuf[i] = 0;                                                    // Pufferbyte lschen
  }
}

void set_pixel(uint8_t x, uint8_t y)                                  // Ein Pixel setzen
{                                                                     // x: X-Position (0-239), y: Y-Position (0-127)
  lcdbuf[(y / 8) * 240 + x] |= _BV(y % 8);                            // entsprechendes Pixel-Bit setzen
}

void clr_pixel(uint8_t x, uint8_t y)                                  // Ein Pixel lschen
{                                                                     // x: X-Position (0-239), y: Y-Position (0-127)
  lcdbuf[(y / 8) * 240 + x] &= ~_BV(y % 8);                           // entsprechendes Pixel-Bit lschen
}

void draw_xline(uint8_t x, uint8_t y, uint8_t x1)                     // Eine waagerechte Linie zeichnen
{                                                                     // x: X-Position Startpunkt (0-239), y: Y-Position Startpunkt (0-127), x1: X-Position Ziel
  uint8_t i;                                                          // Zhlervariable
  for (i = x; i <= x1; i ++) {                                        // Schleife von Start bis Ziel
    set_pixel(i, y);                                                  // Pixel setzen
  }
}

void draw_yline(uint8_t x, uint8_t y, uint8_t y1)                     // Eine senkrechte Linie zeichnen
{                                                                     // x: X-Position Startpunkt (0-239), y: Y-Position Startpunkt (0-127), y1: Y-Position Ziel
  uint8_t i;                                                          // Zhlervariable
  for (i = y; i <= y1; i ++) {                                        // Schleife von Start bis Ziel
    set_pixel(x, i);                                                  // Pixel setzen
  }
}

void draw_frame(uint8_t x, uint8_t y, uint8_t x1, uint8_t y1)         // Einen Rahmen mit abgerundeten Ecken zeichnen
{
  draw_xline(x + 2, y, x1 - 2);                                       // Oberen Rand zeichnen
  draw_xline(x + 2, y1, x1 - 2);                                      // Unteren Rand zeichnen
  draw_yline(x, y + 2, y1 - 2);                                       // linken Rand zeichnen
  draw_yline(x1, y + 2, y1 - 2);                                      // rechten Rand zeichnen
  set_pixel(x + 1, y + 1);                                            // Ecke links oben
  set_pixel(x1 - 1, y + 1);                                           // Ecke rechts oben
  set_pixel(x + 1, y1 - 1);                                           // Ecke links unten
  set_pixel(x1 - 1, y1 - 1);                                          // Ecke rechts unten
}

void lcd_digit1(uint8_t d)                                            // Eine Groziffer der Uhrzeit (34x54) in den LCD-Puffer schreiben
{                                                                     // d: auszugebende Ziffer (0-9)
  uint16_t offset;                                                    // Offset in der Zeichensatz-Tabelle
  uint8_t x;                                                          // Zhler fr die Zeichenausgabe, X-Position
  uint8_t y;                                                          // Zhler fr die Zeichenausgabe, Y-Position
  uint8_t z;                                                          // Byte-Zhler
  uint8_t b;                                                          // Zwischenspeicher fr Bytes aus der Zeichensatz-Tabelle
  offset = d * 5 * 54;                                                // Tabellen-Offset berechnen
  for (y = 0; y < 54; y ++) {                                         // 54 Zeilen ausgeben
    for (z = 0; z < 4; z ++) {                                        // Bytes 1-4 vollstndig ausgeben
      if (!digset) b = pgm_read_byte_far(&dig_array11[offset ++]);    // wenn Groziffernsatz 0 -> entsprechendes Zeilenbyte holen
      else b = pgm_read_byte_far(&dig_array12[offset ++]);            // wenn Groziffernsatz 1 -> entsprechendes Zeilenbyte holen
      for (x = 0; x < 8; x ++) {                                      // 8 Bits (Pixel) ausgeben
        if (b & _BV(7)) set_pixel(posx + (z * 8) + x, posy + y);      // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
        b <<= 1;                                                      // nchste Bit-Position an MSB
      }
    }    
    if (!digset) b = pgm_read_byte_far(&dig_array11[offset ++]);      // wenn Groziffernsatz 0 -> entsprechendes letztes Zeilenbyte holen, enthlt 2 Pixel
    else b = pgm_read_byte_far(&dig_array12[offset ++]);              // wenn Groziffernsatz 1 -> entsprechendes letztes Zeilenbyte holen, enthlt 2 Pixel
    for (x = 0; x < 2; x ++) {                                        // 2 Bits (Pixel) ausgeben
      if (b & _BV(7)) set_pixel(posx + 32 + x, posy + y);             // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
      b <<= 1;                                                        // nchste Bit-Position an MSB
    }
  }
}

void lcd_dot1(void)                                                   // Einen Doppelpunkt (9x54) in den LCD-Puffer schreiben
{
  uint16_t offset;                                                    // Offset in der Zeichensatz-Tabelle
  uint8_t x;                                                          // Zhler fr die Zeichenausgabe, X-Position
  uint8_t y;                                                          // Zhler fr die Zeichenausgabe, Y-Position
  uint8_t b;                                                          // Zwischenspeicher fr Bytes aus der Zeichensatz-Tabelle
  offset = 0;                                                         // Tabellen-Offset setzen
  for (y = 0; y < 54; y ++) {                                         // 54 Zeilen ausgeben
    if (!digset) b = pgm_read_byte_far(&dot_array11[offset ++]);      // wenn Groziffernsatz 0 -> entsprechendes Zeilenbyte holen
    else b = pgm_read_byte_far(&dot_array12[offset ++]);              // wenn Groziffernsatz 1 -> entsprechendes Zeilenbyte holen
    for (x = 0; x < 8; x ++) {                                        // 8 Bits (Pixel) ausgeben
      if (b & _BV(7)) set_pixel(posx + x, posy + y);                  // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
      b <<= 1;                                                        // nchste Bit-Position an MSB
    }
    if (!digset) b = pgm_read_byte_far(&dot_array11[offset ++]);      // wenn Groziffernsatz 0 -> entsprechendes letztes Zeilenbyte holen, enthlt 1 Pixel
    else b = pgm_read_byte_far(&dot_array12[offset ++]);              // wenn Groziffernsatz 1 -> entsprechendes letztes Zeilenbyte holen, enthlt 1 Pixel
    if (b & _BV(7)) set_pixel(posx + 8, posy + y);                    // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
  }
}

void lcd_digit2(uint8_t d)                                            // Eine Groziffer der Weckzeit (11x18) in den LCD-Puffer schreiben
{                                                                     // d: auszugebende Ziffer (0-9)
  uint16_t offset;                                                    // Offset in der Zeichensatz-Tabelle
  uint8_t x;                                                          // Zhler fr die Zeichenausgabe, X-Position
  uint8_t y;                                                          // Zhler fr die Zeichenausgabe, Y-Position
  uint8_t b;                                                          // Zwischenspeicher fr Bytes aus der Zeichensatz-Tabelle
  offset = d * 2 * 18;                                                // Tabellen-Offset berechnen
  for (y = 0; y < 18; y ++) {                                         // 18 Zeilen mit je 2 Bytes ausgeben
    if (!digset) b = pgm_read_byte_far(&dig_array21[offset ++]);      // wenn Groziffernsatz 0 -> entsprechendes Zeilenbyte 1 holen
    else b = pgm_read_byte_far(&dig_array22[offset ++]);              // wenn Groziffernsatz 1 -> entsprechendes Zeilenbyte 1 holen
    for (x = 0; x < 8; x ++) {                                        // 8 Bits (Pixel) ausgeben
      if (b & _BV(7)) set_pixel(posx + x, posy + y);                  // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
      b <<= 1;                                                        // nchste Bit-Position an MSB
    }
    if (!digset) b = pgm_read_byte_far(&dig_array21[offset ++]);      // wenn Groziffernsatz 0 -> entsprechendes Zeilenbyte 2 holen
    else b = pgm_read_byte_far(&dig_array22[offset ++]);              // wenn Groziffernsatz 1 -> entsprechendes Zeilenbyte 2 holen
    for (x = 0; x < 3; x ++) {                                        // 8 Bits (Pixel) ausgeben
      if (b & _BV(7)) set_pixel(posx + 8 + x, posy + y);              // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
      b <<= 1;                                                        // nchste Bit-Position an MSB
    }
  }
}

void lcd_symbol1(uint8_t d)                                           // Ein Grosymbol (24x24) in den LCD-Puffer schreiben
{                                                                     // d: auszugebende Symbolnummer (0-3)
  uint16_t offset;                                                    // Offset in der Zeichensatz-Tabelle
  uint8_t  x;                                                         // Zhler fr die Zeichenausgabe, X-Position
  uint8_t  y;                                                         // Zhler fr die Zeichenausgabe, Y-Position
  uint8_t  b;                                                         // Zwischenspeicher fr Bytes aus der Zeichensatz-Tabelle
  offset = d * 3 * 24;                                                // Tabellen-Offset berechnen
  for (y = 0; y < 24; y ++) {                                         // 24 Zeilen mit je 3 Bytes ausgeben
    b = pgm_read_byte_far(&symbol_array1[offset ++]);                 // Zeilenbyte 1 holen
    for (x = 0; x < 8; x ++) {                                        // 8 Bits (Pixel) ausgeben
      if (b & _BV(7)) set_pixel(posx + x, posy + y);                  // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
      b <<= 1;                                                        // nchste Bit-Position an MSB
    }
    b = pgm_read_byte_far(&symbol_array1[offset ++]);                 // Zeilenbyte 2 holen
    for (x = 0; x < 8; x ++) {                                        // 8 Bits (Pixel) ausgeben
      if (b & _BV(7)) set_pixel(posx + 8 + x, posy + y);              // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
      b <<= 1;                                                        // nchste Bit-Position an MSB
    }
    b = pgm_read_byte_far(&symbol_array1[offset ++]);                 // Zeilenbyte 3 holen
    for (x = 0; x < 8; x ++) {                                        // 8 Bits (Pixel) ausgeben
      if (b & _BV(7)) set_pixel(posx + 16 + x, posy + y);             // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
      b <<= 1;                                                        // nchste Bit-Position an MSB
    }
  }
}

void lcd_symbol2(uint8_t d)                                           // Ein Grosymbol (16x16) in den LCD-Puffer schreiben
{                                                                     // d: auszugebende Symbolnummer (0-4)
  uint16_t offset;                                                    // Offset in der Zeichensatz-Tabelle
  uint8_t  x;                                                         // Zhler fr die Zeichenausgabe, X-Position
  uint8_t  y;                                                         // Zhler fr die Zeichenausgabe, Y-Position
  uint8_t  b;                                                         // Zwischenspeicher fr Bytes aus der Zeichensatz-Tabelle
  offset = d * 2 * 16;                                                // Tabellen-Offset berechnen
  for (y = 0; y < 16; y ++) {                                         // 16 Zeilen mit je 2 Bytes ausgeben
    b = pgm_read_byte_far(&symbol_array2[offset ++]);                 // Zeilenbyte 1 holen
    for (x = 0; x < 8; x ++) {                                        // 8 Bits (Pixel) ausgeben
      if (b & _BV(7)) set_pixel(posx + x, posy + y);                  // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
      b <<= 1;                                                        // nchste Bit-Position an MSB
    }
    b = pgm_read_byte_far(&symbol_array2[offset ++]);                 // Zeilenbyte 2 holen
    for (x = 0; x < 8; x ++) {                                        // 8 Bits (Pixel) ausgeben
      if (b & _BV(7)) set_pixel(posx + 8 + x, posy + y);              // wenn Pixel gesetzt werden muss -> MSB als Pixel ausgeben
      b <<= 1;                                                        // nchste Bit-Position an MSB
    }
  }
}

void lcd_chr(char c)                                                  // Ein ASCII-Zeichen in den LCD-Puffer schreiben
{                                                                     // c: auszugebendes Zeichen (ASCII)
  uint16_t offset;                                                    // Offset in der Zeichensatz-Tabelle
  uint8_t cwidth;                                                     // Pixelbreite des Zeichens
  uint8_t x;                                                          // Zhler fr die Zeichenausgabe, X-Position
  uint8_t y;                                                          // Zhler fr die Zeichenausgabe, Y-Position
  uint8_t b;                                                          // Zwischenspeicher fr Bytes aus der Zeichensatz-Tabelle
  if (c >= 32) {                                                      // Ausgabe nur bei zulssigem Zeichencode
    offset = (c - 32) * 12;                                           // Tabellen-Offset berechnen
    cwidth = pgm_read_byte_far(&char_array[offset]);                  // Zeichenbreite holen
    if (cwidth > 8) cwidth = 8;                                       // Zeichenbreite auf 8 begrenzen
    for (y = 0; y < 11; y ++) {                                       // 11 Zeilenbytes ausgeben
      b = pgm_read_byte_far(&char_array[offset + y + 1]);             // Zeilenbyte holen
      for (x = cwidth; x > 0; x--) {                                  // Zeilenbyte bitweise ausgeben
        if (b & 1)                                                    // wenn Bit gesetzt
        set_pixel(posx + x - 1, posy + y);                            // Pixel setzen
        b >>= 1;                                                      // nchstes Bit an Ausgabe-Position
      }
    }
    posx += cwidth + 1;                                               // X-Position fr nchstes Zeichen
  }
}

void lcd_hex(uint8_t n)                                               // Ein Byte Hexadezimal 2-stellig ausgeben (fr Testzwecke, wird ansonsten nicht bentigt)
{                                                                     // n: auszugebender Wert
  uint8_t h, l;                                                       // h: Zwischenspeicher fr oberes Nibble, l: Zwischenspeicher fr unteres Nibble
  h = n / 16;                                                         // oberes Nibble ermitteln
  l = n % 16;                                                         // unteres Nibble ermitteln
  if (h > 9) lcd_chr(h + 55);                                         // wenn Wert ber 9 -> A-F ausgeben
  else lcd_chr(h + 48);                                               // sonst 0-9 ausgeben
  if (l > 9) lcd_chr(l + 55);                                         // wenn Wert ber 9 -> A-F ausgeben
  else lcd_chr(l + 48);                                               // sonst 0-9 ausgeben
}

void lcd_dez3(uint8_t n)                                              // Ein Byte dezimal 3-stellig ausgeben
{                                                                     // n: auszugebender Wert
  uint8_t h;                                                          // Zwischenspeicher fr Hunderter
  uint8_t l;                                                          // Zwischenspeicher
  h = n / 100;                                                        // Hunderter ermitteln
  l = n;                                                              // Wert zwischnspeichern
  if (n > 99) l = n - h * 100;                                        // wenn Wert grer 99 -> Zehner und Einer ermitteln
  lcd_chr(h + 48);                                                    // Hunderter ausgeben
  lcd_chr(l / 10 + 48);                                               // Zehner ausgeben
  lcd_chr(l % 10 + 48);                                               // Einer ausgeben
}

void lcd_dez(uint8_t n)                                               // Ein Byte dezimal ohne fhrende 0 ausgeben
{                                                                     // n: auszugebender Wert
  uint8_t h;                                                          // Zwischenspeicher fr Hunderter
  uint8_t l;                                                          // Zwischenspeicher
  h = n / 100;                                                        // Hunderter ermitteln
  l = n;                                                              // Wert zwischnspeichern
  if (n > 99) {                                                       // wenn Wert grer 99
    l = n - h * 100;                                                  // Zehner und Einer ermitteln
    lcd_chr(h + 48);                                                  // Hunderter ausgeben
  }
  if (n > 9) lcd_chr(l / 10 + 48);                                    // wenn Byte grer 9 -> Zehner ausgeben
  lcd_chr(l % 10 + 48);                                               // Einer ausgeben
}

void lcd_str(uint8_t strnr, uint8_t len)                              // Einen String aus dem Flash-Speicher in den LCD-Puffer schreiben
{                                                                     // strnr: String-Nummer im Flash-Speicher, len: Anzahl der auszugebenden Zeichen (255 = alle Zeichen ausgeben)
  uint8_t i = 0;                                                      // Zeichenzhler auf erstes Zeichen setzen
  PGM_P p;                                                            // Pointer auf Stringdaten im Flash
  if (lang) p = (PGM_P) pgm_read_word(&stre[strnr]);                  // String-Adresse ermitteln (wenn Englisch gewhlt)
  else p = (PGM_P) pgm_read_word(&strd[strnr]);                       // String-Adresse ermitteln (wenn Deutsch gewhlt)
  strcpy_P(strbuf, p);                                                // String in den RAM-Puffer kopieren
  while (strbuf[i] > 0) {                                             // Schleife bis zum Endezeichen (0)
    if (i < len) lcd_chr(strbuf[i]);                                  // wenn vorgegebene Zeichenanzahl noch nicht erreicht -> Zeichen ausgeben
    i ++;                                                             // nchstes Zeichen
  }
}

void lcd_str1(void)                                                   // Teilstring 1 in den LCD-Puffer schreiben
{
  uint8_t i = 0;                                                      // Zeichenzhler auf erstes Zeichen setzen
  while (strbu1[i] > 0) {                                             // Schleife bis zum Endezeichen (0)
    lcd_chr(strbu1[i]);                                               // Zeichen ausgeben
    i ++;                                                             // nchstes Zeichen
  }
}

void lcd_str2(void)                                                   // Teilstring 2 in den LCD-Puffer schreiben
{
  uint8_t i = 0;                                                      // Zeichenzhler auf erstes Zeichen setzen
  while (strbu2[i] > 0) {                                             // Schleife bis zum Endezeichen (0)
    lcd_chr(strbu2[i]);                                               // Zeichen ausgeben
    i ++;                                                             // nchstes Zeichen
  }
}

uint8_t split_str(uint8_t strnr, uint8_t split)                       // Einen String bei Leerzeichen oder Bindestrich in 2 Teile trennen, wird fr Men-Buttons bentigt
{                                                                     // strnr: String-Nummer im Flash-Speicher, split: Trennstellen-Position, Ergebnis: 0 = keine Trennung, 1 = Trennung ok
  uint8_t i = split;                                                  // Zeichenzhler auf Trennstellen-Position setzen
  uint8_t s = 0;                                                      // gefundene Trennstelle
  PGM_P p;                                                            // Pointer auf Stringdaten im Flash
  if (lang) p = (PGM_P) pgm_read_word(&stre[strnr]);                  // String-Adresse ermitteln (wenn Englisch gewhlt)
  else p = (PGM_P) pgm_read_word(&strd[strnr]);                       // String-Adresse ermitteln (wenn Deutsch gewhlt)
  strcpy_P(strbuf, p);                                                // String in den RAM-Puffer kopieren
  if (strlen(strbuf) > split) {                                       // wenn Stringlnge gre als Trennstellen-Position
    while (i > 0) {                                                   // solange Zeichenzhler grer als Stringanfang
      if ((strbuf[i] == ' ') || (strbuf[i] == '-')) {                 // wenn Leerzeichen oder Bindestrich gefunden
        s = i;                                                        // Trennstellenposition speichern
        break;                                                        // Schleife abbrechen
      }
      i --;                                                           // vorherige Position testen
    }
    if (s > 0) {                                                      // wenn Trennstelle gefunden
      for (i = 0; i < s; i ++) {                                      // Zeichen von Stringanfang bis vor Trennstelle in Teilstring 1 kopieren
        strbu1[i] = strbuf[i];                                        // Zeichen kopieren
        strbu1[s] = 0;                                                // Endezeichen setzen
      }
      if (strbuf[s] == '-') {                                         // wenn Bindestrich gefunden
        strbu1[s] = strbuf[s];                                        // Bindestrich ebenfalls kopieren
        strbu1[s + 1] = 0;                                            // Endezeichen hinter Bindestrich setzen
      }      
      for (i = 0; i < strlen(strbuf) - s; i ++) {                     // restliche Zeichen nach der Trennstelle bis Stringende und Endezeichen in Teilstring 2 kopieren
        strbu2[i] = strbuf[i + s + 1];                                // Zeichen kopieren
      }
      return 1;                                                       // Stringtrennung erfolgreich
    }
    else return 0;                                                    // wenn Trennstelle nicht gefunden -> Stringtrennung nicht erfolgreich
  }
  else return 0;                                                      // wenn String zu kurz -> Stringtrennung nicht erfolgreich
}

uint8_t chrw(uint8_t c)                                               // Die Pixelbreite eines Zeichens ermitteln
{                                                                     // c: Zeichen, Ergebnis: Zeichenbreite
  uint16_t offset;                                                    // Offset in der Zeichensatz-Tabelle
  uint8_t charw = 0;                                                  // Pixelbreite des Zeichens zunchst auf 0 setzen
  if (c == 0) c = 127;                                                // wenn Nullzeichen -> durch Endezeichen ersetzen
  if (c >= 32) {                                                      // wenn zulssiger Zeichencode
    offset = (c - 32) * 12;                                           // Tabellen-Offset berechnen
    charw = pgm_read_byte_far(&char_array[offset]);                   // Zeichenbreite holen
    if (charw > 8) charw = 8;                                         // Zeichenbreite auf 8 begrenzen
  }
  return charw;                                                       // Pixelbreite zurckgeben
}

uint8_t strw(uint8_t strnr, uint8_t len)                              // Die Pixelbreite eines Strings im Flash-Speicher ermitteln
{                                                                     // strnr: String-Nummer im Flash-Speicher, len: Anzahl der auszugebenden Zeichen (255 = alle Zeichen ermitteln)
                                                                      // Ergebnis: Stringlnge (Summe der Pixelbreite alle Zeichen plus einem Pixel zwischen jedem Zeichen)
  uint8_t i = 0;                                                      // Zeichenzhler
  PGM_P p;                                                            // Pointer auf Stringdaten im Flash
  uint8_t pw = 0;                                                     // Speicher fr Pixelbreite
  if (lang) p = (PGM_P) pgm_read_word(&stre[strnr]);                  // String-Adresse ermitteln (wenn Englisch gewhlt)
  else p = (PGM_P) pgm_read_word(&strd[strnr]);                       // String-Adresse ermitteln (wenn Deutsch gewhlt)
  strcpy_P(strbuf, p);                                                // String in den RAM-Puffer kopieren
  while (strbuf[i] > 0) {                                             // Schleife bis zum Endezeichen (0)
    if (i < len) pw += chrw(strbuf[i]) + 1;                           // wenn vorgegebene Zeichenanzahl noch nicht erreicht -> Zeichenlnge und Zwischenraum addieren
    i ++;                                                             // nchstes Zeichen
  }
  if (pw > 0) pw--;                                                   // wenn Pixelbreite > 0 -> korrigieren
  return pw;                                                          // Pixelbreite zurckgeben
}

uint8_t strw1(void)                                                   // Die Pixelbreite des Teilstrings 1 ermitteln
{                                                                     // Ergebnis: Stringlnge (Summe der Pixelbreite alle Zeichen plus einem Pixel zwischen jedem Zeichen)
  uint8_t i = 0;                                                      // Zeichenzhler
  uint8_t pw = 0;                                                     // Speicher fr Pixelbreite
  while (strbu1[i] > 0) {                                             // Schleife bis zum Endezeichen (0)
    pw += chrw(strbu1[i]) + 1;                                        // wenn vorgegebene Zeichenanzahl noch nicht erreicht -> Zeichenlnge und Zwischenraum addieren
    i ++;                                                             // nchstes Zeichen
  }
  if (pw > 0) pw--;                                                   // wenn Pixelbreite > 0 -> korrigieren
  return pw;                                                          // Pixelbreite zurckgeben
}

uint8_t strw2(void)                                                   // Die Pixelbreite des Teilstrings ermitteln
{                                                                     // Ergebnis: Stringlnge (Summe der Pixelbreite alle Zeichen plus einem Pixel zwischen jedem Zeichen)
  uint8_t i = 0;                                                      // Zeichenzhler
  uint8_t pw = 0;                                                     // Speicher fr Pixelbreite
  while (strbu2[i] > 0) {                                             // Schleife bis zum Endezeichen (0)
    pw += chrw(strbu2[i]) + 1;                                        // wenn vorgegebene Zeichenanzahl noch nicht erreicht -> Zeichenlnge und Zwischenraum addieren
    i ++;                                                             // nchstes Zeichen
  }
  if (pw > 0) pw--;                                                   // wenn Pixelbreite > 0 -> korrigieren
  return pw;                                                          // Pixelbreite zurckgeben
}

uint8_t calc_wday(uint8_t d, uint8_t m, uint8_t y)                    // Wochentags-Berechnung nach Gauss (nach http://www.paswiki.de/index.php/Ewiger_Kalender, existiert leider nicht mehr)
{                                                                     // d: Tag, m: Monat, y: Jahr, Ergebnis: Wochentag (1-7)
  int8_t w;                                                           // Wochtag
  if (m <= 2) {                                                       // wenn Monat Januar oder Februar
    m += 12;                                                          // Monat ans Jahresende verschieben
    y --;                                                             // Jahr vermindern
  }
  w = (d + (13 * (m + 1)) / 5 + y + y / 4 + YEARH / 4 - 2 * YEARH) % 7; // Wochtagsformel, kleinster Wert entspricht Samstag
  if (w < 0) w += 7;                                                  // falls Ergebnis negativ war
  w --;                                                               // Wochentag korrigieren (kleinster Wert ist Montag)
  if (w < 1) w += 7;                                                  // und in den Bereich 1-7 legen
  return w;                                                           // Wochentag zurckgeben
}

uint8_t chk_summtime(void)                                            // Prfen, ob aktuelles Datum in der Sommerzeit liegt
{                                                                     // Ergebnis: 0 = Normalzeit, 1 = Sommerzeit
  uint8_t stime = 0;                                                  // Anfangswert auf Normalzeit setzen
  if ((month > 3) && (month < 10)) stime = 1;                         // wenn Monat April bis September -> Sommerzeit setzen
  if ((month == 3) && (day > 24)) {                                   // wenn Datum im Zeitraum vom 25. bis 31. Mrz
    if (calc_wday(day, month, year) == 7) stime = 1;                  // wenn Sonntag -> Sommerzeit setzen (Uhrzeit wird hier nicht bercksichtigt)
    else if ((day - calc_wday(day, month, year)) > 24) stime = 1;     // wenn nicht Sonntag -> wenn Tag nach der Umstellung -> Sommerzeit setzen
  }
  if ((month == 10) && (day > 24)) {                                  // wenn Datum im Zeitraum vom 25. bis 31. Oktober
    if (calc_wday(day, month, year) == 7) stime = 0;                  // wenn Sonntag -> Normalzeit setzen (Uhrzeit wird hier nicht bercksichtigt)
    else if ((day - calc_wday(day, month, year)) < 25) stime = 1;     // wenn nicht Sonntag -> wenn Tag vor der Umstellung -> Sommerzeit setzen
  }
  return stime;                                                       // Sommerzeit-Status zurckgeben
}

void inc_date(void)                                                   // Datum um einen Tag erhhen, bertrge und Schaltjahr bercksichtigen
{
  uint8_t mflag = 0;                                                  // Merker fr Monatswechsel lschen
  tday ++;                                                            // Tag erhhen
  if ((tyear % 4) == 0) {                                             // wenn Schaltjahr
    if (tday > pgm_read_byte_far(&daytab2[tmonth - 1])) mflag = 1;    // wenn Tag grer als Eintrag in Schaltjahr-Tabelle -> Monatswechsel notwendig
  }
  else {                                                              // wenn kein Schaltjahr
    if (tday > pgm_read_byte_far(&daytab1[tmonth - 1])) mflag = 1;    // wenn Tag grer als Eintrag in Normaljahr-Tabelle -> Monatswechsel notwendig
  }
  if (mflag) {                                                        // wenn Monatswechsel notwendig
    tday = 1;                                                         // Tag auf 1 setzen
    tmonth ++;                                                        // Monat erhhen
    if (tmonth > 12) {                                                // wenn Jahreswechsel notwendig
      tmonth = 1;                                                     // Monat auf 1 setzen
      tyear ++;                                                       // Jahr erhhen
    }
  }
}

void read_area(uint8_t area)                                          // Bereich-Koordinaten aus Men-Tabelle lesen und Rahmen zeichnen
{                                                                     // area: Bereichs-Nummer
  PGM_P p;                                                            // Zeiger auf Tabellendaten im Flash
  p = (PGM_P) pgm_read_word(&menulist[menu]);                         // Zeiger auf Tabelle des aktuellen Mens holen
  basex = pgm_read_byte(&p[area * MTABSIZE]);                         // Basis-X-Koordinate holen
  basey = pgm_read_byte(&p[area * MTABSIZE + 1]);                     // Basis-Y-Koordinate holen
  endx = pgm_read_byte(&p[area * MTABSIZE + 2]);                      // Ende-X-Koordinate holen
  centx = (basex + endx + 1) / 2;                                     // Mitte-X-Koordinate berechnen
  draw_frame(basex, basey, endx, pgm_read_byte(&p[area * MTABSIZE + 3])); // restliche Koordinate lesen und Rahmen zeichnen
}

void find_area(void)                                                  // Berhrten Touchscreen-Bereich im aktuellen Men und zugehrige Zielmens ermitteln
{
  PGM_P p;                                                            // Pointer auf Tabellendaten im Flash
  uint8_t i;                                                          // Schleifenzhler
  uint8_t j;                                                          // Anzahl der Bereiche
  uint8_t x1, x2, y1, y2;                                             // gelesene Bereichskoordinaten
  p = (PGM_P) pgm_read_word(&menulist[menu]);                         // Zeiger auf gewhlte Men-Tabelle holen
  j = pgm_read_byte(&menusize[menu]);                                 // Anzahl der Bereiche lesen
  area = 255;                                                         // Bereichsnummer zunchst ungltig setzen
  for (i = 0; i < j; i ++) {                                          // Schleife ber alle Bereiche
    x1 = pgm_read_byte(&p[i * MTABSIZE]);                             // X-Koordinate links lesen
    y1 = pgm_read_byte(&p[i * MTABSIZE + 1]);                         // Y-Koordinate oben lesen
    x2 = pgm_read_byte(&p[i * MTABSIZE + 2]);                         // X-Koordinate rechts lesen
    y2 = pgm_read_byte(&p[i * MTABSIZE + 3]);                         // Y-Koordinate unten lesen
    if ((tsposx >= x1) && (tsposx <= x2) && (tsposy >= y1) && (tsposy <= y2)) { // wenn aktuelle Touchscreen-Koordinaten innerhalb der Bereichs-Koordinaten
      area = i;                                                       // Bereichsnummer speichern
      actshort = pgm_read_byte(&p[i * MTABSIZE + 4]);                 // Aktion bei kurzer Tochscreen-Berhrung speichern
      actlong = pgm_read_byte(&p[i * MTABSIZE + 5]);                  // Aktion bei langer Tochscreen-Berhrung speichern
    }    
  }
}

void lcd_title(uint8_t strnr)                                         // Men-Titel mit Datum und Uhrzeit ausgeben
{                                                                     // strnr: String-Nummer im Flash-Speicher
  read_area(0);                                                       // Basis-Koordinaten lesen und Rahmen zeichnen
  posx = centx - strw(strnr, 255) / 2;                                // X-Koordinate fr zentrierte Ausgabe der Men-berschrift
  posy = basey + 3;                                                   // Y-Koordinate
  lcd_str(strnr, 255);                                                // Men-berschrift ausgeben
  posx = basex + 4;                                                   // X-Koordinate fr linksbndiges Datum
  if (day > 9) lcd_chr(day / 10 + 48);                                // wenn Tag grer 9 -> Tag-Zehner ausgeben
  lcd_chr(day % 10 + 48);                                             // Tag-Einer ausgeben
  lcd_chr('.');                                                       // Punkt ausgeben
  lcd_chr(' ');                                                       // Leerzeichen ausgeben
  lcd_str(7 + month, 3);                                              // Monat mit 3 Zeichen ausgeben
  posx = endx - 3 - 3 * chrw('0') - chrw(':') - 3;                    // X-Koordinate fr rechtsbndige Zeit
  if (hour > 9) {                                                     // wenn Stunde grer 9
    posx = posx - chrw('0') - 1;                                      // X-Koordinate korrigieren
    lcd_chr(hour / 10 + 48);                                          // Stunden-Zehner ausgeben
  }
  lcd_chr(hour % 10 + 48);                                            // Stunden-Einer ausgeben
  if (!(second % 2)) lcd_chr(':');                                    // wenn Sekunde geradzahlig -> Doppelpunkt ausgeben
  else posx = posx + chrw(':') + 1;                                   // sonst X-Koordinatee auf Minuten setzen
  lcd_chr(minute / 10 + 48);                                          // Minuten-Zehner ausgeben
  lcd_chr(minute % 10 + 48);                                          // Minuten-Einer ausgeben
}

void calc_tspos(void)                                                 // Touchscreen-Berhrungs-Koordinaten X und Y berechnen
{
  uint8_t r;                                                          // Zwischenspeicher fr Touchscreen-Raw-Werte
  uint16_t f;                                                         // Zwischenspeicher fr Umrechnungsfaktor von Raw-Werten zu Koordinaten
  r = tsrawx;                                                         // X-Raw-Wert kopieren
  if (tsrawx < tsleft) r = tsleft;                                    // wenn Raw-Wert kleiner als Wert am linken Rand -> auf linken Rand setzen (vermeidet negative Koordinaten)
  if (tsrawx > tsright) r = tsright;                                  // wenn Raw-Wert grer als Wert am rechten Rand -> auf rechten Rand setzen (vermeidet Koordinaten ber 239)
  f = (uint16_t) 240 * 256 / (tsright - tsleft);                      // Umrechnungsfaktor von Raw-Werten zu Koordinaten ermitteln (mit 256 multipliziert fr hhere Interger-Genauigkeit)
  tsposx = ((r - tsleft) * f) >> 8;                                   // X-Raw-Wert in X-Koordinate umrechnen und durch 256 dividieren
  r = tsrawy;                                                         // Y-Raw-Wert kopieren
  if (tsrawy < tstop) r = tstop;                                      // wenn Raw-Wert kleiner als Wert am oberen Rand -> auf oberen Rand setzen (vermeidet negative Koordinaten)
  if (tsrawy > tsbottom) r = tsbottom;                                // wenn Raw-Wert grer als Wert am unteren Rand -> auf unteren Rand setzen (vermeidet Koordinaten ber 127)
  f = (uint16_t) 128 * 256 / (tsbottom - tstop);                      // Umrechnungsfaktor von Raw-Werten zu Koordinaten ermitteln (mit 256 multipliziert fr hhere Interger-Genauigkeit)
  tsposy = ((r - tstop) * f) >> 8;                                    // Y-Raw-Wert in Y-Koordinate umrechnen und durch 256 dividieren
}

void sub_menu(uint8_t submenu)                                        // Ein Untermen aufrufen (eine Men-Ebene nach unten)
{                                                                     // submenu: Untermen-Nummer
  if (menlevel < sizeof(menstack)) {                                  // wenn noch Platz im Men-Stack vorhanden ist
    menstack[menlevel] = menu;                                        // aktuelle Men-Nummer im Stack speichern
    menlevel ++;                                                      // Men-Ebene erhhen
    menu = submenu;                                                   // Wechsel in bergebene Untermen-Nummer
  }
}

void ret_menu(void)                                                   // Rckkehr aus einem Untermen (eine Men-Ebene nach oben in Richtung Hauptmen)
{
  if (menlevel > 0) {                                                 // wenn Hauptmen noch nicht erreicht
    menlevel --;                                                      // Men-Ebene vermindern auf bergeordnetes Men
    menu = menstack[menlevel];                                        // bergeordnete Men-Nummer aus Stack holen und einstellen
  }
  else menu = 0;                                                      // wenn Hauptmen bereits erreicht -> Hauptmen aufrufen
}

void top_menu(void)                                                   // direkte Rckkehr ins Hauptmen
{
  menlevel = 0;                                                       // Men-Ebene auf Hauptmen
  menu = menstack[menlevel];                                          // Men-Nummer aus Stack holen und einstellen
}

uint8_t eep_read_byte(uint16_t addr)                                  // Ein Byte von einer bestimmten EEPROM-Adresse lesen (aus AVR-GCC-Tutorial von http://www.mikrocontroller.net)
{                                                                     // addr: EEPROM-Adresse, Ergebnis: gelesenes Byte
  return eeprom_read_byte((uint8_t *) addr);                          // Byte ber die interne EEPROM-Routine lesen
}

void eep_write_byte(uint16_t addr, uint8_t val)                       // Ein Byte auf eine bestimmte EEPROM-Adresse schreiben (aus AVR-GCC-Tutorial von http://www.mikrocontroller.net)
{                                                                     // addr: EEPROM-Adresse, val: zu schreibendes Byte
  if (eeprom_read_byte((uint8_t *) addr) != val) {                    // wenn das Byte nicht mit dem EEPROM-Inhalt identisch ist
    eeprom_write_byte((uint8_t *) addr, val);                         // Byte ber die interne EEPROM-Routine schreiben
  }
}

void send_usb(uint8_t val)                                            // Ein Byte ber USART1 (USB) senden (wird nur fr Screenshot-Funktion bentigt)
{                                                                     // val: zu sendendes Byte
  while (!(UCSR1A & (1 << UDRE1)));                                   // warten solange noch Daten gesendet werden
  UDR1 = val;                                                         // Byten ber USART1 senden
}

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

int main(void)                                                        // Beginn Hauptprogramm-Initialisierung
{
  int i;                                                              // allgemeine Zhlervariable
  int j;                                                              // allgemeine Zhlervariable
  uint8_t l;                                                          // allgemeine 8-Bit Variable fr verschiedene Anwendungen

  CS0_PRT |= _BV(CS0_SIG);                                            // Anschluss CS0 auf High setzen
  CS0_DDR |= _BV(CS0_SIG);                                            // Anschluss CS0 auf Ausgang setzen
  CD_PRT |= _BV(CD_SIG);                                              // Anschluss CD auf High setzen
  CD_DDR |= _BV(CD_SIG);                                              // Anschluss CD auf Ausgang setzen
  CLK_DDR |= _BV(CLK_SIG);                                            // Anschluss CLK auf Ausgang setzen
  OC0A_DDR |= _BV(OC0A_SIG);                                          // Anschluss OC0A auf Ausgang setzen
  OC1A_DDR |= _BV(OC1A_SIG);                                          // Anschluss OC1A auf Ausgang setzen
  OC2A_DDR |= _BV(OC2A_SIG);                                          // Anschluss OC2A auf Ausgang setzen
  TSL_DDR |= _BV(TSL_SIG);                                            // Touchscreen-Anschluss LEFT auf Ausgang setzen
  TST_DDR |= _BV(TST_SIG);                                            // Touchscreen-Anschluss TOP auf Ausgang setzen
  SEN_PRT |= _BV(SEN_SIG);                                            // Sensor/Taster Pull-Up einschalten
  LED_DDR |= _BV(LED_SIG);                                            // LED-Anschluss auf Ausgang setzen
  LED_PRT |= _BV(LED_SIG);                                            // LED-Anschluss auf High setzen (LED aus)
  EX1_DDR |= _BV(EX1_SIG);                                            // Externen Alarm-Anschluss 1 auf Ausgang setzen
  EX2_DDR |= _BV(EX2_SIG);                                            // Externen Alarm-Anschluss 2 auf Ausgang setzen

  UBRR0 = 0;                                                          // USART0 Baudraten-Register zunchst lschen
  UCSR0C = _BV(UMSEL01) | _BV(UMSEL00) | _BV(UCPHA0) | _BV(UCPOL0);   // USART0 auf SPI-Modus 3 setzen (fallende Vorderflanke, steigende Rckflanke)
  UCSR0B = _BV(TXEN0);                                                // USART0 SPI-Sender aktivieren
  UBRR0 = F_CPU / 2 / SPI_BAUD - 1;                                   // USART0 SPI-Baudrate setzen
  UDR0 = 0;                                                           // Dummy-Ausgabe zum Setzen des TXC0-Bits
  UCSR1A = _BV(U2X1);                                                 // USART1 doppelte bertragungsgeschwindigkeit
  UCSR1B = _BV(RXCIE1) | _BV(RXEN1) | _BV(TXEN1);                     // USART1 Empfnger mit Interrupt und Sender aktivieren
  UCSR1C = _BV(UCSZ11) | _BV(UCSZ10);                                 // USART1 asynchron 8N1
  UBRR1 = F_CPU / 8 / RS_BAUD - 1;                                    // USART1 Baudrate setzen

  TCCR0A = _BV(COM0A1) | _BV(WGM00);                                  // Timer0 Phase Correct PWM setzen und Ausgang OC0A aktivieren
  TCCR0B = _BV(CS01);                                                 // Timer0 Vorteiler = 8, PWM-Frequenz = 2451Hz
  TCCR1A = _BV(COM1A0);                                               // Timer1 Ausgang OCRA in Toggle Mode
  TCCR1B = _BV(WGM12);                                                // Timer1 CTC-Mode, kein Takt
  TCCR2A = _BV(COM2A1) | _BV(WGM20);                                  // Timer2 Phase Correct PWM setzen und Ausgang OC2A aktivieren
  TCCR2B = _BV(CS20);                                                 // Timer2 Vorteiler = 1, PWM-Frequenz = 19608Hz
  TCCR3B = _BV(WGM32) | _BV(CS31) | _BV(CS30);                        // Timer3 CTC/OCR3A, Vorteiler = 64
  OCR3A = F_CPU / 64 / TIM3_CLK - 1;                                  // Timer3 Teilerwert setzen, Frequenz = 50Hz
  TIMSK3 = _BV(OCIE3A);                                               // Timer3 Interrupt fr Output Compare 3A aktivieren

  ADMUX = _BV(ADLAR) | _BV(REFS0) | LIS_ADC;                          // ADC Wert linksbndig speichern, AVCC als Referenz mit Kondensator an AREF, Multiplexer auf Lichtsensor setzen
  ADCSRA = _BV(ADEN) | _BV(ADSC) | _BV(ADPS2) | _BV(ADPS1);           // ADC aktivieren und starten, Vorteiler = 64
  DIDR0 = _BV(LIS_SIG) | _BV(TSR_SIG) | _BV(TSL_SIG) | _BV(TSB_SIG) | _BV(TST_SIG); // ADC Digital Input fr Touchscreen- und Lichtsensor-Anschlsse abschalten

  lang = eep_read_byte(e_lang);                                       // Sprache aus EEPROM lesen
  if (lang > 1) lang = 0;                                             // wenn Bereich berschritten -> Deutsch einstellen
  dcfsyn = eep_read_byte(e_dcfsyn);                                   // Feste DCF77-Synchronisierungszeit aus EEPROM lesen
  if (dcfsyn > 1) dcfsyn = 1;                                         // wenn Bereich berschritten -> einschalten
  dcfinv = eep_read_byte(e_dcfinv);                                   // DCF77-Invertierung aus EEPROM lesen
  if (dcfinv > 1) dcfinv = 1;                                         // wenn Bereich berschritten -> einschalten (Grundeinstellung fr Pollin-Empfnger)
  dcfpup = eep_read_byte(e_dcfpup);                                   // DCF77-Pull-up aus EEPROM lesen
  if (dcfpup > 1) dcfpup = 0;                                         // wenn Bereich berschritten -> ausschalten (Grundeinstellung fr Pollin-Empfnger)
  if (dcfpup) DCF_PRT |= _BV(DCF_SIG);                                // wenn Pull-up aktiviert -> beim DCF77-Eingang Pull-up einschalten
  ledmode = eep_read_byte(e_ledmod);                                  // LED-Modus aus EEPROM lesen
  if (ledmode > 2) ledmode = 1;                                       // wenn Bereich berschritten -> LED-Modus auf Alarmanzeige setzen
  brimin = eep_read_byte(e_brimin);                                   // minimale Helligkeit aus EEPROM lesen
  if (brimin > 9) brimin = BRIMIND;                                   // wenn Bereich berschritten -> minimale Helligkeit auf Voreinstellung setzen
  brimax = eep_read_byte(e_brimax);                                   // maximale Helligkeit aus EEPROM lesen
  if (brimax > 9) brimax = BRIMAXD;                                   // wenn Bereich berschritten -> maximale Helligkeit auf Voreinstellung setzen
  tsleft = eep_read_byte(e_tsleft);                                   // Touchscreen X-Wert links aus EEPROM lesen
  if ((tsleft < 5) || (tsleft > 50)) {                                // wenn Wert auerhalb des zulssigen Bereiches
    tsleft = 20;                                                      // Wert auf 20 setzen
    tscalibr = 1;                                                     // Touchscreen-Kalibrierung erforderlich
  }  
  tsright = eep_read_byte(e_tsrigh);                                  // Touchscreen X-Wert rechts aus EEPROM lesen
  if ((tsright < 205) || (tsright > 250)) {                           // wenn Wert auerhalb des zulssigen Bereiches
    tsright = 235;                                                    // Wert auf 235 setzen
    tscalibr = 1;                                                     // Touchscreen-Kalibrierung erforderlich
  }  
  tstop = eep_read_byte(e_tstop);                                     // Touchscreen Y-Wert oben aus EEPROM lesen
  if ((tstop < 5) || (tstop > 70)) {                                  // wenn Wert auerhalb des zulssigen Bereiches
    tstop = 40;                                                       // Wert auf 40 setzen
    tscalibr = 1;                                                     // Touchscreen-Kalibrierung erforderlich
  }
  tsbottom = eep_read_byte(e_tsbott);                                 // Touchscreen Y-Wert unten aus EEPROM lesen
  if ((tsbottom < 185) || (tsbottom > 250)) {                         // wenn Wert auerhalb des zulssigen Bereiches
    tsbottom = 215;                                                   // Wert auf 215 setzen
    tscalibr = 1;                                                     // Touchscreen-Kalibrierung erforderlich
  }
  aldate = eep_read_byte(e_aldate);                                   // Alarm-Datum aus EEPROM lesen
  if (aldate > 1) aldate = 0;                                         // wenn Bereich berschritten -> auf Normal schalten
  lcdcon = eep_read_byte(e_lcdcon);                                   // LCD-Kontrast aus EEPROM lesen
  if ((lcdcon < LCDCONL) || (lcdcon > LCDCONH)) lcdcon = LCDCOND;     // wenn Wert auerhalb des zulssigen Bereches -> auf Voreinstellung setzen
  dcfhou = eep_read_byte(e_dcfhou);                                   // DCF-Synchronisierungsstunde aus EEPROM lesen
  if (dcfhou > 23) dcfhou = DCFHOU;                                   // wenn Bereich berschritten -> DCF-Synchronisierungstunde auf Voreinstellung setzen
  soundnr = eep_read_byte(e_soundn);                                  // Weck-Ton-Nummer aus EEPROM lesen
  if (soundnr > 9) soundnr = 0;                                       // wenn Bereich berschritten -> Weck-Ton-Nummer auf 0 setzen
  volume = eep_read_byte(e_volume);                                   // Lautstrke aus EEPROM lesen
  if (volume > 9) volume = 3;                                         // wenn Bereich berschritten -> Lautstrke auf 3 setzen
  soundon = eep_read_byte(e_soundo);                                  // Lautsprecher-Status aus EEPROM lesen
  if (soundon > 1) soundon = 1;                                       // wenn Bereich berschritten -> Lautsprecher-Status auf ein setzen
  curprog = eep_read_byte(e_curpro);                                  // Aktuelles Weck-Programm aus dem EEPROM lesen
  if (curprog > 3) curprog = 0;                                       // wenn Bereich berschritten -> Aktuelles Weck-Programm auf 0 setzen
  for (i = 0; i < 4; i ++) {                                          // 4 Weck-Programme aus EEPROM lesen
    for (j = 0; j < 57; j ++) {                                       // jeweils 57 Bytes lesen (28 Werte-Paare mit Minute und Stunde und die Anzahl der Tage)
      almprog[i][j] = eep_read_byte(e_almpro + i * 57 + j);           // Byte aus EEPROM lesen
      if (j < 56) {                                                   // wenn Minuten-Stunden-Werte-Paar
        if ((j % 2) == 0) {                                           // wenn Minutenwert (gerader Zhlerwert)
          if (almprog[i][j] > 59) almprog[i][j] = 0;                  // wenn Minute grer 59 -> Minute auf 0 setzen
        }
        else {                                                        // wenn Stundenwert (ungerader Zhlerwert)
          if (almprog[i][j] > 24) almprog[i][j] = 24;                 // wenn Stunde grer 24 -> Stunde auf 24 (Weckzeit inaktiv) setzen
        }
      }
      else {                                                          // wenn Anzahl der Tage
        if ((almprog[i][j] < 1) || (almprog[i][j] > 28)) {            // wenn Tage kleiner 1 oder grer 28
          almprog[i][j] = 7;                                          // Anzahl der Tage auf 7 setzen
        }
      }      
    }
  }
  for (i = 0; i < 3; i ++) {                                          // 3 voreingestellte Weckzeiten aus EEPROM lesen
    for (j = 0; j < 2; j ++) {                                        // jeweils 2 Bytes lesen (Minute und Stunde)
      almpre[i][j] = eep_read_byte(e_almpre + i * 2 + j);             // Byte aus EEPROM lesen
      if ((j == 0) && (almpre[i][j] > 59)) almpre[i][j] = 0;          // wenn Minutenwert und Bereich berschritten -> Minute auf 0 setzen
      if ((j == 1) && (almpre[i][j] > 23)) almpre[i][j] = ALMHOUR;    // wenn Stundenwert und Bereich berschritten -> Stunde auf Voreinstellung setzen
    }
  }
  snooze = eep_read_byte(e_snooze);                                   // Schlummerzeit aus EEPROM lesen
  if (snooze > 15) snooze = 0;                                        // wenn Bereich berschritten -> Schlummerzeit auf 0 setzen
  almpos = eep_read_byte(e_almpos);                                   // Weck-Programm-Position aus EEPROM lesen
  if (almpos >= almprog[curprog][56]) almpos = 0;                     // wenn Bereich berschritten -> Weck-Programm-Position auf 0 setzen
  digset = eep_read_byte(e_digset);                                   // Groziffernsatz aus EEPROM lesen
  if (digset > 1) digset = 0;                                         // wenn Bereich berschritten -> Groziffernsatz auf 0 setzen
  vomode = eep_read_byte(e_vomode);                                   // Lautstrke-Modus aus EEPROM lesen
  if (vomode > 1) vomode = 0;                                         // wenn Bereich berschritten -> Lautstrke-Modus auf fest setzen
  lday = eep_read_byte(e_lday);                                       // Letzte Reorganisierung (Tag) aus EEPROM lesen
  if ((lday < 1) || (lday > 31)) lday = 1;                            // wenn auerhalb des Bereiches -> Tag auf 1 setzen
  lmonth = eep_read_byte(e_lmonth);                                   // Letzte Reorganisierung (Monat) aus EEPROM lesen
  if ((lmonth < 1) || (lmonth > 12)) lmonth = MONTH;                  // wenn auerhalb des Bereiches -> Monat auf Voreinstellung setzen
  lyear = eep_read_byte(e_lyear);                                     // Letzte Reorganisierung (Jahr) aus EEPROM lesen
  if ((lyear < YEAR) || (lyear > 99)) lyear = YEAR;                   // wenn auerhalb des Bereiches -> Jahr auf Voreinstellung setzen
  for (i = 0; i < 3; i ++) {                                          // Nchste Weckzeiten aus EEPROM lesen
    nxtalm[i][0] = eep_read_byte(e_nxtalm + i * 3);                   // Weckzeit-Status aus EEPROM lesen
    if (nxtalm[i][0] > 1) nxtalm[i][0] = 0;                           // wenn Bereich berschritten -> Status auf Programmzeit setzen
    nxtalm[i][1] = eep_read_byte(e_nxtalm + i * 3 + 1);               // Weckzeit-Minute aus EEPROM lesen
    if (nxtalm[i][1] > 59) nxtalm[i][1] = 0;                          // wenn Bereich berschritten -> Minute auf 0 setzen
    nxtalm[i][2] = eep_read_byte(e_nxtalm + i * 3 + 2);               // Weckzeit-Stunde aus EEPROM lesen
    if (nxtalm[i][2] > 24) nxtalm[i][2] = 0;                          // wenn Bereich berschritten -> Stunde auf 0 setzen
  }
  mode = eep_read_byte(e_mode);                                       // Wecker-Modus aus EEPROM lesen
  if (mode > 1) mode = 1;                                             // wenn Bereich berschritten -> Wecker-Modus auf Manuell setzen
  autosumt = eep_read_byte(e_autost);                                 // Sommerzeit-Automatik aus EEPROM lesen
  if (autosumt > 1) autosumt = 1;                                     // wenn Bereich berschritten -> Sommerzeit-Automatik einschalten

  _delay_ms(200);                                                     // warten auf LCD (mindestens 150ms)
  lcd_start();                                                        // Datenbertragung zum LCD starten
  lcd_out(0, 0xf1); lcd_out(0, 0x7f);                                 // Set COM End: letzte Pixelzeile auf 127 setzen
  lcd_out(0, 0xf2); lcd_out(0, 0);                                    // Set Partitial Display start: Startzeile auf 0 setzen
  lcd_out(0, 0xf3); lcd_out(0, 0x7f);                                 // Set Partitial Display end: Endzeile auf 127 setzen
  lcd_out(0, 0x81); lcd_out(0, lcdcon);                               // Set Potentiometer: LCD-Kontrast setzen
  lcd_out(0, 0xc0); lcd_out(0, 0x02);                                 // Set LCD Mapping Control: auf "Bottom-View" setzen
  lcd_out(0, 0xa3);                                                   // Set Line Rate: 9,4 kilo-lines/s
  lcd_out(0, 0x25);                                                   // Set Temperature Compensation: -0.1%/C
  lcd_out(0, 0xd1);                                                   // Set Display Pattern: 1 Bit/Pixel
  lcd_out(0, 0xa9);                                                   // Set Display Enable: Schwarz/Wei-Modus
  lcd_stop();                                                         // Datenbertragung zum LCD stoppen
  _delay_us(10);                                                      // Wartezeit nach Datenblatt

  hour = HOUR;                                                        // Stunde voreinstellen
  day = lday;                                                         // Tag voreinstellen
  month = lmonth;                                                     // Monat voreinstellen
  year = lyear;                                                       // Jahr voreinstellen
  wday = calc_wday(day, month, year);                                 // Wochentag voreinstellen
  if (autosumt) timezone = chk_summtime();                            // wenn Sommerzeit-Automatik eingeschaltet -> Zeitzone einstellen
  if (tscalibr) sub_menu(3);                                          // wenn Touchscreen-Kalibrierung erforderlich -> Men 3 (Touchscreen-Kalibrierung) aufrufen
  almreorg = 1;                                                       // Weckzeit-Reorganisierung durchfhren
  sei();                                                              // Interrupts aktivieren

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

  while (1)                                                           // Beginn Hauptprogrammschleife
  {
    clr_buf();                                                        // LCD-Puffer lschen
    switch (menu)                                                     // LCD-Anzeige entsprechend aktueller Men-Nummer
    {
      case 0:                                                         // LCD-Anzeige im Men 0 (normale Anzeige)
        read_area(0);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posy = basey + 7;                                             // Y-Koordinate fr Ziffern
        if (clkmode == 0) {                                           // wenn normale Zeitanzeige
          posx = basex + 25;                                          // X-Koordinate fr Ziffern bei dreistelliger Zeit
          if (hour > 9) {                                             // wenn Stunden zweistellig
            posx = basex + 6;                                         // X-Koordinate fr Ziffern bei vierstelliger Zeit
            lcd_digit1(hour / 10);                                    // Stunden-Zehner ausgeben
            posx += 38;                                               // X-Koordinate fr Stunden-Einer
          }        
          lcd_digit1(hour % 10);                                      // Stunden-Einer ausgeben
          l = posx + 37;                                              // X-Koordinate fr Doppelpunkt zwischenspeichern
          posx += 49;                                                 // X-Koordinate fr Minuten-Zehner
          lcd_digit1(minute / 10);                                    // Minuten-Zehner ausgeben
          posx += 38;                                                 // X-Koordinate fr Minuten-Einer
          lcd_digit1(minute % 10);                                    // Minuten-Einer ausgeben
        }
        else {                                                        // wenn Sekundenanzeige
          posx = basex + 93;                                          // X-Koordinate fr Sekunden-Zehner
          lcd_digit1(second / 10);                                    // Sekunden-Zehner ausgeben
          posx += 38;                                                 // X-Koordinate fr Sekunden-Einer
          lcd_digit1(second % 10);                                    // Sekunden-Einer ausgeben
          posx = basex + 35;                                          // X-Koordinate fr Zeitzonen-Symbol
          posy = basey + 6;                                           // Y-Koordinate
          lcd_symbol2(6 + timezone);                                  // Zeitzonen-Symbol ausgeben
          posx = basex + 41 - strw(wday, 255) / 2;                    // X-Koordinate fr zentrierten Wochentag
          posy = basey + 27;                                          // Y-Koordinate
          lcd_str(wday, 255);                                         // Wochentag ausgeben
          l = chrw('0');                                              // Lnge von einer Ziffer ermitteln (Tag-Zehner)
          if (day > 9) l += chrw('0') + 1;                            // wenn Tag zweistellig -> Lnge einer weiteren Ziffer mit Zwischenraum ergnzen
          l += chrw('.') + 1;                                         // Lnge eines Punktes mit Zwischenraum ergnzen
          l += chrw(' ') + 1;                                         // Lnge von eines Leerzeichens mit Zwischenrumen ergnzen
          l += strw(month + 7, 255);                                  // Lnge des Monats ergnzen
          posx = basex + 40 - l / 2;                                  // X-Koordinate fr zentrierte Ausgabe des Datums
          posy = basey + 40;                                          // Y-Koordinate
          if (day > 9) lcd_chr(day / 10 + 48);                        // wenn Tag zweistellig -> Tag-Zehner ausgeben
          lcd_chr(day % 10 + 48);                                     // Tag-Einer ausgeben
          lcd_chr('.');                                               // Punkt ausgeben
          lcd_chr(' ');                                               // Leerzeichen ausgeben
          lcd_str(month + 7, 255);                                    // Monat ausgeben
          posx = basex + 41 - (4 * chrw('0') + 3) / 2;                // X-Koordinate fr zentrierte Ausgabe des Jahrs (4 Ziffern und 3 Zwischenrume)
          posy = basey + 53;                                          // Y-Koordinate
          lcd_chr(YEARH / 10 + 48);                                   // Jahrhundert-Zehner ausgeben
          lcd_chr(YEARH % 10 + 48);                                   // Jahrhundert-Einer ausgeben
          lcd_chr(year / 10 + 48);                                    // Jahr-Zehner ausgeben
          lcd_chr(year % 10 + 48);                                    // Jahr-Einer ausgeben
          l = 81;                                                     // X-Koordinate fr Doppelpunkt zwischenspeichern
        }
        if (!(second % 2)) {                                          // wenn Sekunde geradzahlig
          posx = l;                                                   // zwischengespeicherte Position fr X-Koordinate Doppelpunkt
          posy = basey + 7;                                           // Y-Koordinate
          lcd_dot1();                                                 // Doppelpunkt ausgeben
        }
        j = 0;                                                        // Flag, noch keine Weckzeit-Umrahmung gesetzt
        for (i = 0; i < 3; i ++) {                                    // 3 Weckzeit-Bereiche ausgeben
          read_area(i + 1);                                           // Basis-Koordinaten lesen und Rahmen zeichnen (Bereich 1 - 3)
          posy = basey + 4;                                           // Y-Koordinate fr Datum-Zeile
          if ((i < 2) && aldate) {                                    // wenn Weckzeit-Bereich Heute oder Morgen und Alarm-Datum auf "Heute" gesetzt
            if (i == 0) {                                             // wenn Weckzeit-Bereich Heute
              posx = centx - strw(107, 255) / 2;                      // X-Koordinate fr zentrierte Ausgabe von "Heute"
              lcd_str(107, 255);                                      // String ausgeben
            }
            else {                                                    // wenn Weckzeit-Bereich Morgen
              posx = centx - strw(108, 255) / 2;                      // X-Koordinate fr zentrierte Ausgabe von "Morgen"
              lcd_str(108, 255);                                      // String ausgeben
            }
          }            
          else {                                                      // wenn Weckzeitbereich bermorgen und Alarm-Datum auf Normal gesetzt
            l = 3 * chrw('0') + 2;                                    // Lnge von 3 Ziffern und 2 Zwischenrumen ermitteln (Tag und Jahr)
            if (alarms[i][3] > 9) l += chrw('0') + 1;                 // wenn Tag zweistellig -> Lnge einer weiteren Ziffer mit Zwischenraum ergnzen
            l += chrw('.') + 1;                                       // Lnge eines Punktes mit Zwischenraum ergnzen
            l += 2 * chrw(' ') + 2;                                   // Lnge von 2 Leerzeichen mit Zwischenrumen ergnzen
            l += strw(alarms[i][4] + 7, 3) + 1;                       // Lnge des auf 3 Zeichen gekrzten Monats ergnzen
            posx = centx - l / 2;                                     // X-Koordinate fr zentrierte Ausgabe des Datums
            if (alarms[i][3] > 9) lcd_chr(alarms[i][3] / 10 + 48);    // wenn Tag zweistellig -> Tag-Zehner ausgeben
            lcd_chr(alarms[i][3] % 10 + 48);                          // Tag-Einer ausgeben
            lcd_chr('.');                                             // Punkt ausgeben
            lcd_chr(' ');                                             // Leerzeichen ausgeben
            lcd_str(alarms[i][4] + 7, 3);                             // erste 3 Zeichen des Monats ausgeben
            lcd_chr(' ');                                             // Leerzeichen ausgeben
            lcd_chr(alarms[i][5] / 10 + 48);                          // Jahr-Zehner ausgeben
            lcd_chr(alarms[i][5] % 10 + 48);                          // Jahr-Einer ausgeben
          }            
          l = calc_wday(alarms[i][3], alarms[i][4], alarms[i][5]);    // Wochentag aus Datum ermitteln
          posx = centx - strw(l, 255) / 2;                            // X-Koordinate fr zentrierte Ausgabe des Wochentages
          posy = basey + 17;                                          // Y-Koordinate
          lcd_str(l, 255);                                            // Wochentag ausgeben
          if (alarms[i][0]) {                                         // wenn Weckzeit aktiv
            posx = basex + 11;                                        // X-Koordinate der Weckzeit bei einstelliger Stunde
            posy = basey + 33;                                        // Y-Koordinate
            if (alarms[i][2] > 9) {                                   // wenn Stunde zweistellig
              posx = basex + 5;                                       // X-Koordinate der Weckzeit bei zweistelliger Stunde
              lcd_digit2(alarms[i][2] / 10);                          // Stunden-Zehner ausgeben
              posx += 13;                                             // X-Koordinate fr Stunden-Einer
            }            
            lcd_digit2(alarms[i][2] % 10);                            // Stunden-Einer ausgeben
            l = posx + 13;                                            // X-Koordinate fr Doppelpunkt zwischenspeichern
            posx += 19;                                               // X-Koordinate fr Minuten-Zehner
            lcd_digit2(alarms[i][1] / 10);                            // Minuten-Zehner ausgeben
            posx += 13;                                               // X-Koordinate fr Minuten-Einer
            lcd_digit2(alarms[i][1] % 10);                            // Minuten-Einer ausgeben
            posx = l;                                                 // X-Koordinate fr oberen Doppelpunkt
            posy = basey + 36;                                        // Y-Koordinate
            lcd_chr(137);                                             // oberen Doppelpunkt ausgeben
            posx = l;                                                 // X-Koordinate fr unteren Doppelpunkt
            posy = basey + 44;                                        // Y-Koordinate
            lcd_chr(137);                                             // unteren Doppelpunkt ausgeben
            if ((syncstat) && (j == 0)) {                             // wenn gltige Uhrzeit und noch keine Weckzeit-Umrahmung gesetzt
              if (i == 0) {                                           // wenn Weckzeitbereich Heute
                if ((curalm[0] > 0) && (curalm[0] < 5)) {             // wenn Wecksignal aktiv
                  draw_frame(basex + 2, basey + 30, endx - 2, basey + 53); // Rahmen um Weckzeit zeichnen
                  j = 1;                                              // Weckzeit-Umrahmung ist jetzt gesetzt
                }
              }
              else {                                                  // wenn Weckzeitbereich Morgen oder bermorgen
                if (alarms[i][0]) {                                   // wenn Alarm aktiv ist
                  draw_frame(basex + 2, basey + 30, endx - 2, basey + 53); // Rahmen um Weckzeit zeichnen
                  j = 1;                                              // Weckzeit-Umrahmung ist jetzt gesetzt
                }
              }
            }
          }
          else {                                                      // wenn Weckzeit inaktiv
            draw_xline(basex + 12, basey + 41, endx - 12);            // 2 waagerechte Linien anstatt Weckzeit ausgeben
            draw_xline(basex + 12, basey + 42, endx - 12);
          }
        }
        read_area(4);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(20, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Einstellung"
        posy = 7;                                                     // Y-Koordinate
        lcd_str(20, 255);                                             // Text "Einstellung" ausgeben
        read_area(5);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr Symbol
        posy = basey + 6;                                             // Y-Koordinate
        lcd_chr(136 - mode);                                          // Punkt-Symbol entsprechend Modus ausgeben
        posx += 2;                                                    // X-Koordinate fr Text "Programm"
        lcd_str(21, 255);                                             // Text "Programm" ausgeben
        posx = basex + 5;                                             // X-Koordinate fr Symbol
        posy = basey + 23;                                            // Y-Koordinate
        lcd_chr(135 + mode);                                          // Punkt-Symbol entsprechend Modus ausgeben
        posx += 2;                                                    // X-Koordinate fr Text "Manuell"
        lcd_str(22, 255);                                             // Text "Manuell" ausgeben
        read_area(6);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 7;                                             // X-Koordinate fr DCF77-Symbol
        posy = basey + 5;                                             // Y-Koordinate
        lcd_symbol2(syncstat);                                        // passendes DCF77-Symbol ausgeben
        read_area(7);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 7;                                             // X-Koordinate fr Lautsprecher-Symbol
        posy = basey + 5;                                             // Y-Koordinate
        lcd_symbol2(4 + soundon);                                     // passendes Lautsprecher-Symbol ausgeben
        break;

      case 1:                                                         // LCD-Anzeige im Men 1 (Datum und Uhrzeit)
        lcd_title(34);                                                // Mentitel ausgeben
        if (okflag) {                                                 // wenn OK-Bereich sichtbar
          read_area(1);                                               // Basis-Koordinaten und Rahmen fr OK-Bereich
          posx = centx - 12;                                          // X-Koordinate fr OK-Symbol
          posy = basey + 4;                                           // Y-Koordinate
          lcd_symbol1(4);                                             // OK-Symbol ausgeben
        }
        else {                                                        // wenn OK-Bereich nicht sichtbar
          if (dcfsyn) {                                               // wenn feste DCF77-Synchronisierungszeit
            read_area(1);                                             // Basis-Koordinaten und Rahmen fr "Synchr. starten" (gleiche Stelle wie OK)
            if (split_str(25, (endx - basex) / 6)) {                  // wenn Stringtrennung von "Synchr. starten" erfolgreich
              posx = centx - strw1() / 2;                             // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
              posy = basey + 4;                                       // Y-Koordinate
              lcd_str1();                                             // Teilstring 1 ausgeben
              posx = centx - strw2() / 2;                             // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
              posy = basey + 16;                                      // Y-Koordinate
              lcd_str2();                                             // Teilstring 2 ausgeben
            }
            else {                                                    // wenn Stringtrennung nicht erfolgreich
              posx = centx - strw(25, 255) / 2;                       // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
              posy = basey + 10;                                      // Y-Koordinate
              lcd_str(25, 255);                                       // String ausgeben
            }
          }
        }
        read_area(2);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(111, 255) / 2;                            // X-Koordinate fr zentrierte Ausgabe von "Abbruch"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(111, 255);                                            // String ausgeben

        read_area(3);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 6;                                             // X-Koordinate fr Tag-Zehner
        posy = basey + 7;                                             // Y-Koordinate
        lcd_digit2(xday / 10);                                        // Tag-Zehner ausgeben
        posx = basex + 19;                                            // X-Koordinate fr Tag-Einer
        lcd_digit2(xday % 10);                                        // Tag-Einer ausgeben
        posx = basex + 38;                                            // X-Koordinate fr Monat-Zehner
        lcd_digit2(xmonth / 10);                                      // Monat-Zehner ausgeben
        posx = basex + 51;                                            // X-Koordinate fr Monat-Einer
        lcd_digit2(xmonth % 10);                                      // Monat-Einer ausgeben
        posx = basex + 70;                                            // X-Koordinate fr Jahr-Zehner
        lcd_digit2(xyear / 10);                                       // Jahr-Zehner ausgeben
        posx = basex + 83;                                            // X-Koordinate fr Jahr-Einer
        lcd_digit2(xyear % 10);                                       // Jahr-Einer ausgeben
        posx = basex + 32;                                            // X-Koordinate fr Punkt zwischen Tag und Monat
        posy = basey + 21;                                            // Y-Koordinate
        lcd_chr(137);                                                 // Punkt ausgeben
        posx = basex + 64;                                            // X-Koordinate fr Punkt zwischen Monat und Jahr
        posy = basey + 21;                                            // Y-Koordinate
        lcd_chr(137);                                                 // Punkt ausgeben
        read_area(4);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 6;                                             // X-Koordinate fr Stunden-Zehner
        posy = basey + 7;                                             // Y-Koordinate
        lcd_digit2(xhour / 10);                                       // Stunden-Zehner ausgeben
        posx = basex + 19;                                            // X-Koordinate fr Stunden-Einer
        lcd_digit2(xhour % 10);                                       // Stunden-Einer ausgeben
        posx = basex + 38;                                            // X-Koordinate fr Minuten-Zehner
        lcd_digit2(xminute / 10);                                     // Minuten-Zehner ausgeben
        posx = basex + 51;                                            // X-Koordinate fr Minuten-Einer
        lcd_digit2(xminute % 10);                                     // Minuten-Einer ausgeben
        posx = basex + 32;                                            // X-Koordinate fr oberen Doppelpunkt zwischen Stunde und Minute
        posy = basey + 10;                                            // Y-Koordinate
        lcd_chr(137);                                                 // oberen Doppelpunkt ausgeben
        posx = basex + 32;                                            // X-Koordinate fr unteren Doppelpunkt zwischen Stunde und Minute
        posy = basey + 18;                                            // Y-Koordinate
        lcd_chr(137);                                                 // unteren Doppelpunkt ausgeben

        read_area(5);                                                 // Basis-Koordinaten und Rahmen fr "Tag erhhen"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(0);                                               // Pfeilsymbol nach oben ausgeben
        read_area(6);                                                 // Basis-Koordinaten und Rahmen fr "Tag vermindern"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(1);                                               // Pfeilsymbol nach unten ausgeben
        read_area(7);                                                 // Basis-Koordinaten und Rahmen fr "Monat erhhen"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(0);                                               // Pfeilsymbol nach oben ausgeben
        read_area(8);                                                 // Basis-Koordinaten und Rahmen fr "Monat vermindern"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(1);                                               // Pfeilsymbol nach unten ausgeben
        read_area(9);                                                 // Basis-Koordinaten und Rahmen fr "Jahr erhhen"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(0);                                               // Pfeilsymbol nach oben ausgeben
        read_area(10);                                                // Basis-Koordinaten und Rahmen fr "Jahr vermindern"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(1);                                               // Pfeilsymbol nach unten ausgeben
        read_area(11);                                                // Basis-Koordinaten und Rahmen fr "Stunde erhhen"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(0);                                               // Pfeilsymbol nach oben ausgeben
        read_area(12);                                                // Basis-Koordinaten und Rahmen fr "Stunden vermindern"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(1);                                               // Pfeilsymbol nach unten ausgeben
        read_area(13);                                                // Basis-Koordinaten und Rahmen fr "Minute erhhen"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(0);                                               // Pfeilsymbol nach oben ausgeben
        read_area(14);                                                // Basis-Koordinaten und Rahmen fr "Minute vermindern"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(1);                                               // Pfeilsymbol nach unten ausgeben
        read_area(15);                                                // Basis-Koordinaten und Rahmen fr "Sommerzeit-Automatik"
        posx = basex + 5;                                             // X-Koordinate fr Sommerzeit-Automatik
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(43, 255);                                             // String ausgeben
        posx = endx - 5 - strw(57 + xautost, 255);                    // X-Koordinate fr Aus/Ein
        posy = basey + 16;                                            // Y-Koordinate
        lcd_str(57 + xautost, 255);                                   // String ausgeben
        break;

      case 2:                                                         // LCD-Anzeige im Men 2 (Einstellung)
        lcd_title(20);                                                // Mentitel ausgeben
        read_area(1);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(27, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Zurck"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(27, 255);                                             // String ausgeben
        read_area(2);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        if (split_str(28, (endx - basex) / 6)) {                      // wenn Stringtrennung von "Allgemeine Einstellungen" erfolgreich
          posx = centx - strw1() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
          posy = basey + 4;                                           // Y-Koordinate
          lcd_str1();                                                 // Teilstring 1 ausgeben
          posx = centx - strw2() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str2();                                                 // Teilstring 2 ausgeben
        }
        else {                                                        // wenn Stringtrennung nicht erfolgreich
          posx = centx - strw(28, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
          posy = basey + 10;                                          // Y-Koordinate
          lcd_str(28, 255);                                           // String ausgeben
        }
        read_area(3);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        if (split_str(29, (endx - basex) / 6)) {                      // wenn Stringtrennung von "Anzeige-Einstellungen" erfolgreich
          posx = centx - strw1() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
          posy = basey + 4;                                           // Y-Koordinate
          lcd_str1();                                                 // Teilstring 1 ausgeben
          posx = centx - strw2() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str2();                                                 // Teilstring 2 ausgeben
        }
        else {                                                        // wenn Stringtrennung nicht erfolgreich
          posx = centx - strw(29, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
          posy = basey + 10;                                          // Y-Koordinate
          lcd_str(29, 255);                                           // String ausgeben
        }
        read_area(4);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        if (split_str(30, (endx - basex) / 6)) {                      // wenn Stringtrennung von "DCF77-Einstellungen" erfolgreich
          posx = centx - strw1() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
          posy = basey + 4;                                           // Y-Koordinate
          lcd_str1();                                                 // Teilstring 1 ausgeben
          posx = centx - strw2() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str2();                                                 // Teilstring 2 ausgeben
        }
        else {                                                        // wenn Stringtrennung nicht erfolgreich
          posx = centx - strw(30, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
          posy = basey + 10;                                          // Y-Koordinate
          lcd_str(30, 255);                                           // String ausgeben
        }
        read_area(5);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        if (split_str(31, (endx - basex) / 6)) {                      // wenn Stringtrennung von "Audio-Einstellungen" erfolgreich
          posx = centx - strw1() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
          posy = basey + 4;                                           // Y-Koordinate
          lcd_str1();                                                 // Teilstring 1 ausgeben
          posx = centx - strw2() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str2();                                                 // Teilstring 2 ausgeben
        }
        else {                                                        // wenn Stringtrennung nicht erfolgreich
          posx = centx - strw(31, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
          posy = basey + 10;                                          // Y-Koordinate
          lcd_str(31, 255);                                           // String ausgeben
        }
        read_area(6);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        if (split_str(32, (endx - basex) / 6)) {                      // wenn Stringtrennung von "Touchscreen kalibrieren" erfolgreich
          posx = centx - strw1() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
          posy = basey + 4;                                           // Y-Koordinate
          lcd_str1();                                                 // Teilstring 1 ausgeben
          posx = centx - strw2() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str2();                                                 // Teilstring 2 ausgeben
        }
        else {                                                        // wenn Stringtrennung nicht erfolgreich
          posx = centx - strw(32, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
          posy = basey + 10;                                          // Y-Koordinate
          lcd_str(32, 255);                                           // String ausgeben
        }
        read_area(7);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        if (split_str(33, (endx - basex) / 6)) {                      // wenn Stringtrennung von "System-Informationen" erfolgreich
          posx = centx - strw1() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
          posy = basey + 4;                                           // Y-Koordinate
          lcd_str1();                                                 // Teilstring 1 ausgeben
          posx = centx - strw2() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str2();                                                 // Teilstring 2 ausgeben
        }
        else {                                                        // wenn Stringtrennung nicht erfolgreich
          posx = centx - strw(33, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
          posy = basey + 10;                                          // Y-Koordinate
          lcd_str(33, 255);                                           // String ausgeben
        }
        read_area(8);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        if (split_str(34, (endx - basex) / 6)) {                      // wenn Stringtrennung von "Datum und Uhrzeit" erfolgreich
          posx = centx - strw1() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
          posy = basey + 4;                                           // Y-Koordinate
          lcd_str1();                                                 // Teilstring 1 ausgeben
          posx = centx - strw2() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str2();                                                 // Teilstring 2 ausgeben
        }
        else {                                                        // wenn Stringtrennung nicht erfolgreich
          posx = centx - strw(34, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
          posy = basey + 10;                                          // Y-Koordinate
          lcd_str(34, 255);                                           // String ausgeben
        }
        read_area(9);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        if (split_str(35, (endx - basex) / 6)) {                      // wenn Stringtrennung von "Weck-Programme" erfolgreich
          posx = centx - strw1() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
          posy = basey + 4;                                           // Y-Koordinate
          lcd_str1();                                                 // Teilstring 1 ausgeben
          posx = centx - strw2() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str2();                                                 // Teilstring 2 ausgeben
        }
        else {                                                        // wenn Stringtrennung nicht erfolgreich
          posx = centx - strw(35, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
          posy = basey + 10;                                          // Y-Koordinate
          lcd_str(35, 255);                                           // String ausgeben
        }
        break;

      case 3:                                                         // LCD-Anzeige im Men 3 (Touchscreen kalibrieren)
        read_area(0);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(32, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Touchscreen kalibrieren"
        posy = basey + 3;                                             // Y-Koordinate
        lcd_str(32, 255);                                             // String ausgeben
        read_area(1);                                                 // Basis-Koordinaten und Rahmen fr Hinweistext
        posx = centx - strw(40, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe der ersten Zeile des Hinweistextes
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(40, 255);                                             // String ausgeben
        posx = centx - strw(41, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe der zweiten Zeile des Hinweistextes
        posy = basey + 16;                                            // Y-Koordinate
        lcd_str(41, 255);                                             // String ausgeben
        read_area(2);                                                 // Basis-Koordinaten und Rahmen fr Zurck-Bereich
        posx = centx - strw(111, 255) / 2;                            // X-Koordinate fr zentrierte Ausgabe von "Abbruch"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(111, 255);                                            // String ausgeben
        if (okflag) {                                                 // wenn OK-Bereich sichtbar
          read_area(3);                                               // Basis-Koordinaten und Rahmen fr OK-Bereich
          posx = centx - 12;                                          // X-Koordinate fr OK-Symbol
          posy = basey + 4;                                           // Y-Koordinate
          lcd_symbol1(4);                                             // OK-Symbol ausgeben
        }        
        draw_xline(0, 0, 20);                                         // Markierung in der linken oberen Ecke zeichnen
        draw_yline(0, 0, 20);
        set_pixel(1, 1);
        draw_xline(219, 0, 239);                                      // Markierung in der rechten oberen Ecke zeichnen
        draw_yline(239, 0, 20);
        set_pixel(238, 1);
        draw_xline(0, 127, 20);                                       // Markierung in der linken unteren Ecke zeichnen
        draw_yline(0, 107, 127);
        set_pixel(1, 126);
        draw_xline(219, 127, 239);                                    // Markierung in der rechten unteren Ecke zeichnen
        draw_yline(239, 107, 127);
        set_pixel(238, 126);
        if (tsx1) {                                                   // wenn Kalibrierungspunkt links oben gesetzt ist
          posx = 5;                                                   // X-Koordinate fr Text "OK"
          posy = 5;                                                   // Y-Koordinate
          lcd_str(26, 255);                                           // String ausgeben
        }
        if (tsx2) {                                                   // wenn Kalibrierungspunkt rechts oben gesetzt ist
          posx = 234 - strw(26, 255);                                 // X-Koordinate fr Text "OK"
          posy = 5;                                                   // Y-Koordinate
          lcd_str(26, 255);                                           // String ausgeben
        }
        if (tsx3) {                                                   // wenn Kalibrierungspunkt links unten gesetzt ist
          posx = 5;                                                   // X-Koordinate fr Text "OK"
          posy = 111;                                                 // Y-Koordinate
          lcd_str(26, 255);                                           // String ausgeben
        }
        if (tsx4) {                                                   // wenn Kalibrierungspunkt rechts unten gesetzt ist
          posx = 234 - strw(26, 255);                                 // X-Koordinate fr Text "OK"
          posy = 111;                                                 // Y-Koordinate
          lcd_str(26, 255);                                           // String ausgeben
        }
        break;

      case 4:                                                         // LCD-Anzeige im Men 4 (Allgemeine Einstellungen)
        lcd_title(28);                                                // Mentitel ausgeben
        read_area(1);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(27, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Zurck"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(27, 255);                                             // String ausgeben
        read_area(2);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(2);                                               // Pfeilsymbol nach links ausgeben
        read_area(3);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr Sprache
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(44, 255);                                             // String ausgeben
        posx = endx - 4 - strw(45 + lang, 255) - chrw('0') - chrw(' ') - chrw('-') - chrw(' ') - 4; // X-Koordinate fr rechtsbndige Ausgabe der Spracheinstellung
        posy = basey + 16;                                            // Y-Koordinate
        lcd_chr(lang + 49);                                           // Optionsnummer (1-2) ausgeben
        lcd_chr(' ');                                                 // Leerzeichen ausgeben
        lcd_chr('-');                                                 // Minuszeichen ausgeben
        lcd_chr(' ');                                                 // Leerzeichen ausgeben
        lcd_str(45 + lang, 255);                                      // String ausgeben
        read_area(4);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(3);                                               // Pfeilsymbol nach rechts ausgeben
        read_area(5);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(2);                                               // Pfeilsymbol nach links ausgeben
        read_area(6);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr LED-Modus
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(47, 255);                                             // String ausgeben
        posx = endx - 4 - strw(48 + ledmode, 255) - chrw('0') - chrw(' ') - chrw('-') - chrw(' ') - 4; // X-Koordinate fr rechtsbndige Ausgabe der LED-Modus-Einstellung
        posy = basey + 16;                                            // Y-Koordinate
        lcd_chr(ledmode + 49);                                        // Optionsnummer (1-3) ausgeben
        lcd_chr(' ');                                                 // Leerzeichen ausgeben
        lcd_chr('-');                                                 // Minuszeichen ausgeben
        lcd_chr(' ');                                                 // Leerzeichen ausgeben
        lcd_str(48 + ledmode, 255);                                   // String ausgeben
        read_area(7);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(3);                                               // Pfeilsymbol nach rechts ausgeben
        read_area(8);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(2);                                               // Pfeilsymbol nach links ausgeben
        read_area(9);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr Schlummerzeit
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(98, 255);                                             // String ausgeben
        if (snooze) {                                                 // wenn Schlummerzeit grer 0
          posx = endx - 4 - chrw('0');                                // X-Koordinate fr rechtsbndige Ausgabe der Schlummerzeit
          posy = basey + 16;                                          // Y-Koordinate
          if (snooze > 9) {                                           // wenn Schlummerzeit grer 9
            posx = posx - chrw('0') - 1;                              // X-Koordinate um eine Ziffer und einen Zwischenraum vermindern
            lcd_chr(snooze / 10 + 48);                                // Schlummerzeit-Zehner ausgeben
          }
          lcd_chr(snooze % 10 + 48);                                  // Schlummerzeit-Einer ausgeben
        }
        else {                                                        // wenn Schlummerzeit 0
          posx = endx - 4 - strw(57, 255);                            // X-Koordinate fr rechtsbndige Ausgabe von "Aus"
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str(57, 255);                                           // String ausgeben
        }
        read_area(10);                                                // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(3);                                               // Pfeilsymbol nach rechts ausgeben
        break;

      case 5:                                                         // LCD-Anzeige im Men 5 (Anzeige-Einstellungen)
        lcd_title(29);                                                // Mentitel ausgeben
        read_area(1);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(27, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Zurck"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(27, 255);                                             // String ausgeben
        read_area(2);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(2);                                               // Pfeilsymbol nach links ausgeben
        read_area(3);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr Minimale Helligkeit
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(36, 255);                                             // String ausgeben
        posx = endx - 4 - chrw('0');                                  // X-Koordinate fr rechtsbndige Ausgabe der Minimalen Helligkeit
        posy = basey + 16;                                            // Y-Koordinate
        if (brimin > 8) {                                             // wenn Minimale Helligkeit grer 8 (Wertebereich 0-9 wird als 1-10 angezeigt)
          posx = posx - chrw('0') - 1;                                // zustzlich eine Ziffer und einen Zwischenraum bercksichtigen
          lcd_chr((brimin + 1) / 10 + 48);                            // Zehner ausgeben
        }
        lcd_chr((brimin + 1) % 10 + 48);                              // Einer ausgeben
        read_area(4);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(3);                                               // Pfeilsymbol nach rechts ausgeben
        read_area(5);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(2);                                               // Pfeilsymbol nach links ausgeben
        read_area(6);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr Maximale Helligkeit
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(37, 255);                                             // String ausgeben
        posx = endx - 4 - chrw('0');                                  // X-Koordinate fr rechtsbndige Ausgabe der Maximalen Helligkeit
        posy = basey + 16;                                            // Y-Koordinate
        if (brimax > 8) {                                             // wenn Minimale Helligkeit grer 8 (Wertebereich 0-9 wird als 1-10 angezeigt)
          posx = posx - chrw('0') - 1;                                // zustzlich eine Ziffer und einen Zwischenraum bercksichtigen
          lcd_chr((brimax + 1) / 10 + 48);                            // Zehner ausgeben
        }
        lcd_chr((brimax + 1) % 10 + 48);                              // Einer ausgeben
        read_area(7);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(3);                                               // Pfeilsymbol nach rechts ausgeben
        read_area(8);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(2);                                               // Pfeilsymbol nach links ausgeben
        read_area(9);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr Kontrast
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(38, 255);                                             // String ausgeben
        posx = endx - 4 - chrw('0') * 3 - 2;                          // X-Koordinate fr rechtsbndige Ausgabe des Kontrastes
        posy = basey + 16;                                            // Y-Koordinate
        lcd_dez3(lcdcon);                                             // Zahl ausgeben
        read_area(10);                                                // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(3);                                               // Pfeilsymbol nach rechts ausgeben
        read_area(11);                                                // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(39, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Weckdatum"
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(39, 255);                                             // String ausgeben
        posx = centx - strw(109 + aldate, 255) / 2;                   // X-Koordinate fr zentrierte Ausgabe der Einstellung
        posy = basey + 16;                                            // Y-Koordinate
        lcd_str(109 + aldate, 255);                                   // String ausgeben
        read_area(12);                                                // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 6;                                             // X-Koordinate fr Groziffer
        posy = basey + 7;                                             // Y-Koordinate
        l = digset;                                                   // aktuellen Groziffernsatz zwischenspeichern
        digset = 0;                                                   // vorbergehend auf Groziffernsatz 1 umschalten
        lcd_digit2(0);                                                // Ziffer 0 ausgeben
        posx = endx - 11 - 6;                                         // X-Koordinate fr rechtsbndige Groziffer
        posy = basey + 7;                                             // Y-Koordinate
        digset = 1;                                                   // vorbergehend auf Groziffernsatz 2 umschalten
        lcd_digit2(0);                                                // Ziffer 0 ausgeben
        digset = l;                                                   // ursprnglichen Groziffernsatz wieder herstellen
        posx = centx - 8;                                             // X-Koordinate fr Pfeilsymbole
        posy = basey + 11;                                            // Y-Koordinate
        if (!digset) {                                                // wenn Groziffernsatz 0
          lcd_chr(139);                                               // Pfeil nach links ausgeben
          posx --;                                                    // Zwischenraum entfernen
          lcd_chr(141);                                               // Pfeillinie ausgeben
        }
        else {                                                        // wenn Groziffernsatz 1
          lcd_chr(141);                                               // Pfeillinie ausgeben
          posx --;                                                    // Zwischenraum entfernen
          lcd_chr(140);                                               // Pfeil nach rechts ausgeben
        }
        break;

      case 6:                                                         // LCD-Anzeige im Men 6 (DCF77-Einstellungen)
        lcd_title(30);                                                // Mentitel ausgeben
        read_area(1);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(27, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Zurck"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(27, 255);                                             // String ausgeben
        read_area(2);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr DCF-Invertierung
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(51, 255);                                             // String ausgeben
        posx = endx - 4 - strw(52 + dcfinv, 255);                     // X-Koordinate fr rechtsbndige Ausgabe der DCF-Invertierung
        posy = basey + 16;                                            // Y-Koordinate
        lcd_str(52 + dcfinv, 255);                                    // DCF-Invertierung ausgeben
        read_area(3);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr DCF-Pull-up
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(54, 255);                                             // String ausgeben
        posx = endx - 4 - strw(55 + dcfpup, 255);                     // X-Koordinate fr rechtsbndige Ausgabe des DCF-Pull-up
        posy = basey + 16;                                            // Y-Koordinate
        lcd_str(55 + dcfpup, 255);                                    // DCF-Pull-up ausgeben
        read_area(4);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr DCF-Synchronisierung
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(42, 255);                                             // String ausgeben
        posx = endx - 4 - strw(57 + dcfsyn, 255);                     // X-Koordinate fr rechtsbndige Ausgabe der festen DCF-Synchronierungszeit
        posy = basey + 16;                                            // Y-Koordinate
        lcd_str(57 + dcfsyn, 255);                                    // Feste DCF-Synchronisierungszeit ausgeben
        if (dcfsyn) {                                                 // wenn Feste DCF-Synchronisierungszeit aktiviert
          draw_xline(100, 111, 116);                                  // Verbindungslinie 1 zur Synchronisierungszeit-Einstellung
          draw_yline(116, 75, 111);                                   // Verbindungslinie 2 zur Synchronisierungszeit-Einstellung
          draw_xline(116, 75, 131);                                   // Verbindungslinie 3 zur Synchronisierungszeit-Einstellung
          read_area(5);                                               // Basis-Koordinaten und Rahmen zeichnen
          posx = basex + 2;                                           // X-Koordinate fr Pfeilsymbol
          posy = basey + 4;                                           // Y-Koordinate
          lcd_symbol1(0);                                             // Pfeilsymbol nach oben ausgeben
          read_area(6);                                               // Basis-Koordinaten und Rahmen zeichnen
          posx = basex + 6;                                           // X-Koordinate fr Stunden-Zehner
          posy = basey + 7;                                           // Y-Koordinate
          lcd_digit2(dcfhou / 10);                                    // Stunden-Zehner ausgeben
          posx = basex + 19;                                          // X-Koordinate fr Stunden-Einer
          lcd_digit2(dcfhou % 10);                                    // Stunden-Einer ausgeben
          posx = basex + 38;                                          // X-Koordinate fr Minuten-Zehner
          lcd_digit2(0);                                              // Minuten-Zehner ausgeben (ist immer 0)
          posx = basex + 51;                                          // X-Koordinate fr Minuten-Einer
          lcd_digit2(0);                                              // Minuten-Einer ausgeben (ist immer 0)
          posx = basex + 32;                                          // X-Koordinate fr oberen Doppelpunkt zwischen Stunde und Minute
          posy = basey + 10;                                          // Y-Koordinate
          lcd_chr(137);                                               // oberen Doppelpunkt ausgeben
          posx = basex + 32;                                          // X-Koordinate fr unteren Doppelpunkt zwischen Stunde und Minute
          posy = basey + 18;                                          // Y-Koordinate
          lcd_chr(137);                                               // unteren Doppelpunkt ausgeben
          read_area(7);                                               // Basis-Koordinaten und Rahmen zeichnen
          posx = basex + 2;                                           // X-Koordinate fr Pfeilsymbol
          posy = basey + 4;                                           // Y-Koordinate
          lcd_symbol1(1);                                             // Pfeilsymbol nach unten ausgeben
        }        
        break;

      case 7:                                                         // LCD-Anzeige im Men 7 (Audio-Einstellungen)
        lcd_title(31);                                                // Mentitel ausgeben
        read_area(1);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(27, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Zurck"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(27, 255);                                             // String ausgeben
        read_area(2);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(2);                                               // Pfeilsymbol nach links ausgeben
        read_area(3);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr Weck-Ton-Nummer
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(59, 255);                                             // String ausgeben
        posx = endx - 4 - strw(60 + soundnr, 255) - chrw('0') - chrw(' ') - chrw('-') - chrw(' ') - 4; // X-Koordinate fr rechtsbndige Ausgabe der Weck-Ton-Nummer und des Namens
        posy = basey + 16;                                            // Y-Koordinate
        if ((soundnr + 1) > 9) {                                      // wenn Weck-Ton-Nummer grer 9 (bezogen auf Wertebereich 1-10)
          posx = posx - chrw('0') - 1;                                // zustzlich eine Ziffer und einen Zwischenraum bercksichtigen
          lcd_chr((soundnr + 1) / 10 + 48);                           // Zehnerstelle ausgeben
        }
        lcd_chr((soundnr + 1) % 10 + 48);                             // Einerstelle ausgeben
        lcd_chr(' ');                                                 // Leerzeichen ausgeben
        lcd_chr('-');                                                 // Minuszeichen ausgeben
        lcd_chr(' ');                                                 // Leerzeichen ausgeben
        lcd_str(60 + soundnr, 255);                                   // String ausgeben
        read_area(4);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(3);                                               // Pfeilsymbol nach rechts ausgeben
        read_area(5);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(2);                                               // Pfeilsymbol nach links ausgeben
        read_area(6);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr Lautstrke
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(70, 255);                                             // String ausgeben
        posx = endx - 4 - chrw('0');                                  // X-Koordinate fr rechtsbndige Ausgabe der Lautstrke
        posy = basey + 16;                                            // Y-Koordinate
        if (volume > 8) {                                             // wenn Lautstrke grer 8 (Wertebereich 0-9 wird als 1-10 angezeigt)
          posx = posx - chrw('0') - 1;                                // X-Koordinate korrigieren
          lcd_chr((volume + 1) / 10 + 48);                            // Lautstrke Zehner ausgeben
        }        
        lcd_chr((volume + 1) % 10 + 48);                              // Lautstrke Einer ausgeben
        read_area(7);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(3);                                               // Pfeilsymbol nach rechts ausgeben
        read_area(8);                                                 // Basis-Koordinaten und Rahmen zeichnen
        posx = basex + 6;                                             // X-Koordinate fr Lautsprecher-Symbol
        posy = basey + 8;                                             // Y-Koordinate
        lcd_symbol2(4 + soundon);                                     // passendes Lautsprecher-Symbol ausgeben
        read_area(9);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posy = basey + 10;                                            // Y-Koordinate fr "Weck-Ton testen" oder "Stopp"
        if (almflag) {                                                // wenn Weck-Signal aktiv
          posx = centx - strw(78, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe von "Stopp"
          lcd_str(78, 255);                                           // String ausgeben
        }
        else {                                                        // wenn Weck-Signal inaktiv
          posx = centx - strw(71, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe von "Weck-Ton testen"
          lcd_str(71, 255);                                           // String ausgeben
        }
        read_area(10);
        if (split_str(23 + vomode, (endx - basex) / 6)) {             // wenn Stringtrennung von Lautstrke-Modus erfolgreich
          posx = centx - strw1() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
          posy = basey + 4;                                           // Y-Koordinate
          lcd_str1();                                                 // Teilstring 1 ausgeben
          posx = centx - strw2() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str2();                                                 // Teilstring 2 ausgeben
        }
        else {                                                        // wenn Stringtrennung nicht erfolgreich
          posx = centx - strw(23 + vomode, 255) / 2;                  // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
          posy = basey + 10;                                          // Y-Koordinate
          lcd_str(23 + vomode, 255);                                  // String ausgeben
        }
        break;

      case 8:                                                         // LCD-Anzeige im Men 8 (System-Informationen)
        lcd_title(33);                                                // Mentitel ausgeben
        read_area(1);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(27, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Zurck"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(27, 255);                                             // String ausgeben
        basex = 0;                                                    // Basis-X-Koordinate fr Informationstexte
        basey = 25;                                                   // Basis-Y-Koordinate
        posx = basex;                                                 // X-Koordinate fr Textzeile 1
        posy = basey;                                                 // Y-Koordinate
        lcd_str(72, 255);                                             // Text "LCD-Wecker" ausgeben
        lcd_str(0, 255);                                              // Versionsinformationen ausgeben
        posx = 0;                                                     // X-Koordinate fr Textzeile 2
        posy = basey + 13;                                            // Y-Koordinate
        lcd_str(73, 255);                                             // Text "DCF77-Status" ausgeben
        lcd_str(74 + syncstat, 255);                                  // DCF77-Status ausgeben
        posx = 0;                                                     // X-Koordinate fr Textzeile 3
        posy = basey + 26;                                            // Y-Koordinate
        lcd_str(79, 255);                                             // Text "Letzte Sync." ausgeben
        if (alarms[3][3]) {                                           // wenn eine gltiger Tag empfangen wurde
          lcd_str(80 + alarms[3][0], 255);                            // Zeitzone ausgeben
          lcd_str(calc_wday(alarms[3][3], alarms[3][4], alarms[3][5]), 255); // Wochentag ausgeben
          lcd_chr(',');                                               // Komma ausgeben
          lcd_chr(' ');                                               // Leerzeichen ausgeben
          lcd_chr(alarms[3][3] / 10 + 48);                            // Tag-Zehner ausgeben
          lcd_chr(alarms[3][3] % 10 + 48);                            // Tag-Einer ausgeben
          lcd_chr('.');                                               // Punkt ausgeben
          lcd_chr(alarms[3][4] / 10 + 48);                            // Monat-Zehner ausgeben
          lcd_chr(alarms[3][4] % 10 + 48);                            // Monat-Einer ausgeben
          lcd_chr('.');                                               // Punkt ausgeben
          lcd_chr(alarms[3][5] / 10 + 48);                            // Jahr-Zehner ausgeben
          lcd_chr(alarms[3][5] % 10 + 48);                            // Jahr-Einer ausgeben
          lcd_chr(',');                                               // Komma ausgeben
          lcd_chr(' ');                                               // Leerzeichen ausgeben
          lcd_chr(alarms[3][2] / 10 + 48);                            // Stunden-Zehner ausgeben
          lcd_chr(alarms[3][2] % 10 + 48);                            // Stunden-Einer ausgeben
          lcd_chr(':');                                               // Doppelpunkt ausgeben
          lcd_chr(alarms[3][1] / 10 + 48);                            // Minuten-Zehner ausgeben
          lcd_chr(alarms[3][1] % 10 + 48);                            // Minuten-Einer ausgeben
        }
        else lcd_str(82, 255);                                        // wenn kein gltiger Tag ampfangen wurde -> "keine Daten" ausgeben
        posx = 0;                                                     // X-Koordinate fr Textzeile 4
        posy = basey + 39;                                            // Y-Koordinate
        lcd_str(83, 255);                                             // Text "Touchscreen-Kalibrierdaten" ausgeben
        posx = 0;                                                     // X-Koordinate fr Textzeile 5
        posy = basey + 52;                                            // Y-Koordinate
        lcd_str(84, 255);                                             // Text "Links" ausgeben
        lcd_dez3(tsleft);                                             // X-Wert Links ausgeben
        lcd_str(85, 255);                                             // Text "Rechts" ausgeben
        lcd_dez3(tsright);                                            // X-Wert Rechts ausgeben
        lcd_str(86, 255);                                             // Text "Oben" ausgeben
        lcd_dez3(tstop);                                              // Y-Wert Oben ausgeben
        lcd_str(87, 255);                                             // Text "Unten" ausgeben
        lcd_dez3(tsbottom);                                           // Y-Wert Unten ausgeben
        posx = 0;                                                     // X-Koordinate fr Textzeile 6
        posy = basey + 65;                                            // Y-Koordinate
        lcd_str(88, 255);                                             // Text "Touchscreen X" ausgeben
        lcd_dez3(tsposx);                                             // aktuelle Touchscreen X-Koordinate ausgeben
        lcd_str(89, 255);                                             // Text "Y" ausgeben
        lcd_dez3(tsposy);                                             // aktuelle Touchscreen Y-Koordinate ausgeben
        posx = 0;                                                     // X-Koordinate fr Textzeile 7
        posy = basey + 78;                                            // Y-Koordinate
        lcd_str(90, 255);                                             // Text "Helligkeitssensor" ausgeben
        lcd_dez3(briavg);                                             // Helligkeits-Mittelwert ausgeben
        posx = 0;                                                     // X-Koordinate fr Textzeile 8
        posy = basey + 91;                                            // Y-Koordinate
        lcd_str(91, 255);                                             // Text "Beleuchtung PWM-Wert" ausgeben
        lcd_dez3(OCR0A);                                              // PWM-Wert ausgeben
        break;

      case 9:                                                         // LCD-Anzeige im Men 9 (Weck-Programme)
        lcd_title(35);                                                // Mentitel ausgeben
        read_area(1);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(27, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Zurck"
        posy = basey + 6;                                             // Y-Koordinate
        lcd_str(27, 255);                                             // String ausgeben
        basex = 0;                                                    // Basis-X-Koordinate fr Wochentag-Zeile
        basey = 20;                                                   // Basis-Y-Koordinate
        for (i = 0; i < 7; i ++) {                                    // 7 Wochentage oder Nummern ausgeben
          posy = basey;                                               // Y-Koordinate
          if ((almprog[curprog][56] % 7) == 0) {                      // wenn Lnge des aktuellen Weck-Programms 7, 14, 21 oder 28 Tage
            posx = basex + 5 + i * 34;                                // X-Koordinate fr Wochentag
            lcd_str(i + 1, 3);                                        // Wochentag ausgeben (3 Zeichen)
          }
          else {                                                      // wenn Lnge des aktuellen Weck-Programms nicht im Wochenraster
            posx = basex + 6 + i * 34;                                // X-Koordinate fr Nummern
            lcd_chr(i + 49);                                          // Nummer 1-7 ausgeben
            lcd_str(94, 255);                                         // Texte "...2" ausgeben
            lcd_chr(i + 50);                                          // Nummer 2-8 ausgeben (ergibt 22-28)
          }
        }
        for (i = 0; i < 28; i ++) {                                   // 28 Zeitbereiche ausgeben
          read_area(i + 2);                                           // Basis-Koordinaten lesen und Rahmen zeichnen
          posx = basex + 5;                                           // X-Koordinate fr Weckzeit
          posy = basey + 4;                                           // Y-Koordinate
          if (i < almprog[curprog][56]) {                             // wenn Zhler kleiner als Anzahl der Tage -> Weck-Zeit ausgeben
            if (almprog[curprog][i * 2 + 1] < 24) {                   // wenn Stunde kleiner 24 (Weckzeit aktiv)
              lcd_chr(almprog[curprog][i * 2 + 1] / 10 + 48);         // Stunden-Zehner ausgeben
              lcd_chr(almprog[curprog][i * 2 + 1] % 10 + 48);         // Stunden-Einer ausgeben
              lcd_chr(':');                                           // Doppelpunkt ausgeben
              lcd_chr(almprog[curprog][i * 2] / 10 + 48);             // Minuten-Zehner ausgeben
              lcd_chr(almprog[curprog][i * 2] % 10 + 48);             // Minuten-Einer ausgeben
            }
            else lcd_str(93, 255);                                    // wenn Weckzeit grer 24 (Weckzeit inaktiv) -> Minuszeichen ausgeben
            if (i == almpos) {                                        // wenn aktuelle Weck-Programm-Position
              draw_xline(basex + 3, basey + 3, endx - 3);             // Markierungslinie oben zeichnen
              draw_xline(basex + 3, basey + 14, endx - 3);            // Markierungslinie unten zeichnen
            }
          }
        }
        for (i = 0; i < 4; i ++) {                                    // 4 Programm-Bereiche ausgeben
          read_area(i + 30);                                          // Basis-Koordinaten lesen und Rahmen zeichnen
          posx = centx - (chrw('P') + chrw('0')) / 2;                 // X-Koordinate fr zentrierte Ausgabe von "P" und Nummer
          posy = basey + 6;                                           // Y-Koordinate
          lcd_chr('P');                                               // "P" ausgeben
          lcd_chr(i + 49);                                            // Nummer ausgeben
          if (i == curprog) {                                         // wenn aktuelle Weck-Programm-Nummer
            draw_xline(basex + 3, basey + 3, endx - 3);               // Markierungslinie oben zeichnen
            draw_xline(basex + 3, basey + 19, endx - 3);              // Markierungslinie unten zeichnen
          }
        }
        read_area(34);                                                // Basis-Koordinaten lesen und Rahmen zeichnen
        l = chrw('0') + chrw(' ') + strw(92, 255) + 2;                // Gesamtlnge einer Ziffer, eines Leerzeichens, dem Text "Tage" und 2 Zwischenrumen ermitteln
        if (almprog[curprog][56] > 9) l = l + chrw('0') + 1;          // wenn Lnge des aktuellen Weck-Programms grer 9 -> Lnge um eine Ziffer und einen Zwischenraum erhhen
        posx = centx - l / 2;                                         // X-Koordinate fr zentrierte Ausgabe der Weck-Programm-Lnge
        posy = basey + 6;                                             // Y-Koordinate
        if (almprog[curprog][56] > 9) {                               // wenn Lnge des aktuellen Weck-Programms grer 9
          lcd_chr(almprog[curprog][56] / 10 + 48);                    // Zehnerstelle ausgeben
        }        
        lcd_chr(almprog[curprog][56] % 10 + 48);                      // Einerstelle ausgeben
        lcd_chr(' ');                                                 // Leerzeichen ausgeben
        lcd_str(92, 255);                                             // Text "Tage" ausgeben
        break;

      case 10:                                                        // LCD-Anzeige im Men 10 (Weckzeit und Programme, alternativ)
        lcd_title(112);                                               // Mentitel ausgeben
        read_area(1);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(27, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Zurck"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(27, 255);                                             // String ausgeben
        read_area(2);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(95, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe der ersten Zeile des Hinweistextes
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(95, 255);                                             // String ausgeben
        posx = centx - strw(96, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe der ersten Zeile des Hinweistextes
        posy = basey + 18;                                            // Y-Koordinate
        lcd_str(96, 255);                                             // String ausgeben
        break;

      case 11:                                                        // LCD-Anzeige im Men 11 (Weckzeit einstellen)
        lcd_title(97);                                                // Mentitel ausgeben
        read_area(1);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(27, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Zurck"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(27, 255);                                             // String ausgeben
        for (i = 0; i < 3; i ++) {                                    // 3 voreingestellte Weckzeiten ausgeben
          read_area(i + 2);                                           // Basis-Koordinaten lesen und Rahmen zeichnen
          posx = basex + 6;                                           // X-Koordinate fr Stunden-Zehner
          posy = basey + 7;                                           // Y-Koordinate
          lcd_digit2(almpre[i][1] / 10);                              // Stunden-Zehner ausgeben
          posx = basex + 19;                                          // X-Koordinate fr Stunden-Einer
          lcd_digit2(almpre[i][1] % 10);                              // Stunden-Einer ausgeben
          posx = basex + 38;                                          // X-Koordinate fr Minuten-Zehner
          lcd_digit2(almpre[i][0] / 10);                              // Minuten-Zehner ausgeben (ist immer 0)
          posx = basex + 51;                                          // X-Koordinate fr Minuten-Einer
          lcd_digit2(almpre[i][0] % 10);                              // Minuten-Einer ausgeben (ist immer 0)
          posx = basex + 32;                                          // X-Koordinate fr oberen Doppelpunkt zwischen Stunde und Minute
          posy = basey + 10;                                          // Y-Koordinate
          lcd_chr(137);                                               // oberen Doppelpunkt ausgeben
          posx = basex + 32;                                          // X-Koordinate fr unteren Doppelpunkt zwischen Stunde und Minute
          posy = basey + 18;                                          // Y-Koordinate
          lcd_chr(137);                                               // unteren Doppelpunkt ausgeben
          posx = endx - 30;                                           // X-Koordinate fr rechtsbndiges OK-Symbol
          posy = basey + 4;                                           // Y-Koordinate
          lcd_symbol1(4);                                             // OK-Symbol ausgeben
        }
        read_area(5);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 6;                                             // X-Koordinate fr Stunden-Zehner
        posy = basey + 7;                                             // Y-Koordinate
        lcd_digit2(xalhou / 10);                                      // Stunden-Zehner ausgeben
        posx = basex + 19;                                            // X-Koordinate fr Stunden-Einer
        lcd_digit2(xalhou % 10);                                      // Stunden-Einer ausgeben
        posx = basex + 38;                                            // X-Koordinate fr Minuten-Zehner
        lcd_digit2(xalmin / 10);                                      // Minuten-Zehner ausgeben
        posx = basex + 51;                                            // X-Koordinate fr Minuten-Einer
        lcd_digit2(xalmin % 10);                                      // Minuten-Einer ausgeben
        posx = basex + 32;                                            // X-Koordinate fr oberen Doppelpunkt zwischen Stunde und Minute
        posy = basey + 10;                                            // Y-Koordinate
        lcd_chr(137);                                                 // oberen Doppelpunkt ausgeben
        posx = basex + 32;                                            // X-Koordinate fr unteren Doppelpunkt zwischen Stunde und Minute
        posy = basey + 18;                                            // Y-Koordinate
        lcd_chr(137);                                                 // unteren Doppelpunkt ausgeben
        posx = endx - 30;                                             // X-Koordinate fr rechtsbndiges OK-Symbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(4);                                               // OK-Symbol ausgeben

        read_area(6);                                                 // Basis-Koordinaten und Rahmen fr "Stunde erhhen"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(0);                                               // Pfeilsymbol nach oben ausgeben
        read_area(7);                                                 // Basis-Koordinaten und Rahmen fr "Stunden vermindern"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(1);                                               // Pfeilsymbol nach unten ausgeben
        read_area(8);                                                 // Basis-Koordinaten und Rahmen fr "Minute erhhen"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(0);                                               // Pfeilsymbol nach oben ausgeben
        read_area(9);                                                 // Basis-Koordinaten und Rahmen fr "Minute vermindern"
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(1);                                               // Pfeilsymbol nach unten ausgeben
        read_area(10);                                                // Basis-Koordinaten lesen und Rahmen zeichnen
        if (split_str(99, (endx - basex) / 6)) {                      // wenn Stringtrennung von "Zeit lschen" erfolgreich
          posx = centx - strw1() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
          posy = basey + 4;                                           // Y-Koordinate
          lcd_str1();                                                 // Teilstring 1 ausgeben
          posx = centx - strw2() / 2;                                 // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
          posy = basey + 16;                                          // Y-Koordinate
          lcd_str2();                                                 // Teilstring 2 ausgeben
        }
        else {                                                        // wenn Stringtrennung nicht erfolgreich
          posx = centx - strw(99, 255) / 2;                           // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
          posy = basey + 8;                                           // Y-Koordinate
          lcd_str(99, 255);                                           // String ausgeben
        }
        break;

      case 12:                                                        // LCD-Anzeige im Men 12 (Anzahl der Tage einstellen)
        lcd_title(100);                                               // Mentitel ausgeben
        read_area(1);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = centx - strw(27, 255) / 2;                             // X-Koordinate fr zentrierte Ausgabe von "Zurck"
        posy = basey + 10;                                            // Y-Koordinate
        lcd_str(27, 255);                                             // String ausgeben
        read_area(2);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(2);                                               // Pfeilsymbol nach links ausgeben
        read_area(3);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 5;                                             // X-Koordinate fr Anzahl der Tage
        posy = basey + 4;                                             // Y-Koordinate
        lcd_str(101, 255);                                            // String ausgeben
        posx = endx - 4 - chrw('0');                                  // X-Koordinate fr rechtsbndige Ausgabe der Anzahl der Tage
        posy = basey + 16;                                            // Y-Koordinate
        if (almprog[curprog][56] > 9) {                               // wenn Anzahl der Tage grer 9
          posx = posx - chrw('0') - 1;                                // X-Koordinate korrigieren
          lcd_chr(almprog[curprog][56] / 10 + 48);                    // Anzahl-Zehner ausgeben
        }
        lcd_chr(almprog[curprog][56] % 10 + 48);                      // Anzahl-Einer ausgeben
        read_area(4);                                                 // Basis-Koordinaten lesen und Rahmen zeichnen
        posx = basex + 2;                                             // X-Koordinate fr Pfeilsymbol
        posy = basey + 4;                                             // Y-Koordinate
        lcd_symbol1(3);                                               // Pfeilsymbol nach rechts ausgeben
        break;
    }
                                                                      // --- Meldung bei Sensor/Taster-Bettigung ausgeben
    if (msgtout < 255) {                                              // wenn Meldungs-Timeout-Zhler aktiv
      for (i = 120 - MSGW / 2; i < 120 + MSGW / 2; i ++) {            // Schleife fr X-Position
        for (j = 64 - MSGH / 2; j < 64 + MSGH / 2; j ++) {            // Schleife fr Y-Position
          clr_pixel(i, j);                                            // alle Pixel im Meldungsbereich lschen
        }
      }
      draw_frame(121 - MSGW / 2, 65 - MSGH / 2, 118 + MSGW / 2, 62 + MSGH / 2); // Rahmen zeichnen
      posx = 120 - MSGW / 2 + 12;                                     // X-Koordinate fr linksbndiges Symbol
      posy = 64 - 12;                                                 // Y-Koordinate
      lcd_symbol1(5 + msgnum);                                        // Symbol entsprechend der Meldungsnummer ausgeben
      if (split_str(102 + msgnum, (MSGW - 48) / 6)) {                 // wenn Stringtrennung des Meldungstextes erfolgreich
        posx = 120 + 18 - strw1() / 2;                                // X-Koordinate fr zentrierte Ausgabe von Teilstring 1
        posy = 64 - 11;                                               // Y-Koordinate
        lcd_str1();                                                   // Teilstring 1 ausgeben
        posx = 120 + 18 - strw2() / 2;                                // X-Koordinate fr zentrierte Ausgabe von Teilstring 2
        posy = 64 + 1;                                                // Y-Koordinate
        lcd_str2();                                                   // Teilstring 2 ausgeben
      }
      else {                                                          // wenn Stringtrennung nicht erfolgreich
        posx = 120 + 18 - strw(102 + msgnum, 255) / 2;                // X-Koordinate fr zentrierte Ausgabe des gesamten Strings
        posy = 64 - 5;                                                // Y-Koordinate
        lcd_str(102 + msgnum, 255);                                   // String ausgeben
      }
    }

    send_buf();                                                       // Bildschirm-Puffer an LCD bertragen

    #ifdef SCRSHOT                                                    // wenn Screenshot-Definition aktiv
    uint8_t hi;                                                       // Zwischenspeicher fr Hunderter-Wert
    uint8_t k;                                                        // Zhlervariable
    if (usbflag) {                                                    // wenn USB-Flag gesetzt (neuen Screenshot senden)
      usbflag = 0;                                                    // USB-Flag wieder lschen
      send_usb(13);                                                   // CR senden
      send_usb(10);                                                   // LF senden
      i = 0;                                                          // Zhler fr LCD-Puffer-Position zurcksetzen
      for (j = 0; j < 128; j ++) {                                    // 128 Byteblcke bearbeiten
        for (k = 0; k < 30; k ++) {                                   // 30 Bytes bearbeiten
          l = lcdbuf[i];                                              // Byte aus LCD-Puffer holen
          hi = l /100;                                                // Hunderter-Stelle ermitteln
          send_usb(hi + '0');                                         // und als ASCII senden
          l -= hi * 100;                                              // Restwert des Bytes ermitteln
          send_usb(l / 10 + '0');                                     // Zehner ermitteln und als ASCII senden
          send_usb(l % 10 + '0');                                     // Einer ermitteln und als ASCII senden
          send_usb(',');                                              // Komma senden
          i ++;                                                       // LCD-Puffer-Position erhhen
        }
        send_usb(13);                                                 // CR senden
        send_usb(10);                                                 // LF senden (neue Zeile)
      }
      send_usb(13);                                                   // CR senden
      send_usb(10);                                                   // LF senden (Leerzeile am Ende)
    }
    #endif
                                                                      // --- Touchscreen-Aktionen bearbeiten
    if (tsflag && !tsquit) {                                          // wenn Touchscreen berhrt und noch nicht quittiert wurde, kurze oder lange Berhrung auswerten
      calc_tspos();                                                   // Touchscreen-Koordinaten berechnen
      find_area();                                                    // berhrten Touchscreen-Bereich ermitteln
      if (dcftout < 255) {                                            // wenn DCF77-Synchronisierung aktiv
        dcftout = 255;                                                // DCF77-Synchronisierungs-Timeout-Zhler stoppen
        lcd_on();                                                     // LCD einschalten
        area = 255;                                                   // Touchscreen-Bereich verwerfen, keine Aktion ausfhren
        tsquit = 1;                                                   // Touchscreen-Bedienung quittieren
        menutout = MENUTOUT;                                          // Men-Timeout-Zhler neu starten
      }
      if (area < 255) {                                               // wenn ein Touchscreen-Bereich gefunden wurde
        actpend = 0;                                                  // zunchst steht keine Aktion nach Touchscreen-Bedienung an
        if (tslong == TS_LONG) {                                      // wenn die Zeit fr lange Touchscreen-Berhrung erreicht ist
          tsquit = 1;                                                 // Touchscreen-Bedienung quittieren
          menutout = MENUTOUT;                                        // Men-Timeout-Zhler neu starten
          if (actlong) actpend = actlong;                             // wenn Aktion bei langer Touchscreen-Bedienung definiert -> zur Ausfhrung festlegen
        }
        else {                                                        // wenn die Zeit fr lange Touchscreen-Berhrung noch nicht erreicht ist
          if (!tsstat) {                                              // wenn Touchscreen bereits losgelassen wurde
            tsquit = 1;                                               // Touchscreen-Bedienung quittieren
            menutout = MENUTOUT;                                      // Men-Timeout-Zhler neu starten
            if (actshort) actpend = actshort;                         // wenn Aktion bei kurzer Touchscreen-Bedienung definiert -> zur Ausfhrung festlegen
          }
          else {                                                      // wenn Touchscreen noch berhrt wird
            if (actshort == actlong) {                                // wenn gleiche Aktion bei kurzer und langer Touchscreen-Bedienung definiert ist
              tsquit = 1;                                             // Touchscreen-Bedienung quittieren
              menutout = MENUTOUT;                                    // Men-Timeout-Zhler neu starten
              if (actshort) actpend = actshort;                       // wenn Aktion bei kurzer Touchscreen-Bedienung definiert -> zur Ausfhrung festlegen
            }
          }
        }
        
        autcnt = 0;                                                   // Auto-Repeat-Zhler zurcksetzen
        switch (actpend) {                                            // Aktion bei Touchscreen-Bedienung entsprechend Nummer ausfhren
          case 1:                                                     // Aktion "Menwechsel -> Datum und Uhrzeit"
            sub_menu(actpend);                                        // in neue Men-Ebene wechseln
            okflag = 0;                                               // OK-Bereich unsichtbar machen
            break;
          case 2:                                                     // Aktion "Menwechsel -> Einstellung"
            sub_menu(actpend);                                        // in neue Men-Ebene wechseln
            break;
          case 3:                                                     // Aktion "Menwechsel -> Touchscreen kalibrieren"
            sub_menu(actpend);                                        // in neue Men-Ebene wechseln
            okflag = 0;                                               // OK-Bereich unsichtbar machen
            tsx1 = 0;                                                 // Zwischenspeicher fr Kalibrierwerte zurcksetzen
            tsy1 = 0;
            tsx2 = 0;
            tsy2 = 0;
            tsx3 = 0;
            tsy3 = 0;
            tsx4 = 0;
            tsy4 = 0;
            break;
          case 4 ... 8:                                               // Aktion "Menwechsel"
            sub_menu(actpend);                                        // in neue Men-Ebene wechseln
            break;
          case 9:                                                     // Aktion "Menwechsel -> Weck-Programme"
            if (syncstat) sub_menu(actpend);                          // wenn gltige Uhrzeit -> in neue Men-Ebene wechseln
            else sub_menu(10);                                        // wenn keine gltige Uhrzeit -> in alternatives Men-Wechseln
            break;
          case 12:                                                    // Aktion "Menwechsel -> Anzahl der Tage einstellen"
            sub_menu(actpend);                                        // in neue Men-Ebene wechseln
            break;
          case 99:                                                    // Aktion "Zurck zum vorherigen Men
            ret_menu();                                               // in bergeordnete Men-Ebene wechseln
            break;
          case 100:                                                   // Aktion "Sekundenanzeige"
            clkmode ++;                                               // Zeitmodus erhhen
            if (clkmode > 1) clkmode = 0;                             // wenn Wertebereich berschritten -> zurcksetzen
            break;
          case 101:                                                   // Aktion "Zeit speichern und zurck zum vorherigen Men"
            if (okflag) {                                             // wenn OK-Bereich sichtbar
              cli();                                                  // Interrupts sperren
              intcnt = 0;                                             // Interrupt-Zhler zurcksetzen
              second = 0;                                             // Sekunde zurcksetzen
              timezone = 0;                                           // Zeitzone auf MEZ setzen
              minute = xminute;                                       // Minute bernehmen
              hour = xhour;                                           // Stunde bernehmen
              if ((xyear % 4) == 0) {                                 // wenn Schaltjahr
                if (xday > pgm_read_byte_far(&daytab2[xmonth - 1]))   // wenn Tag > Schaltjahr-Tabelle
                  xday =  pgm_read_byte_far(&daytab2[xmonth - 1]);    // auf letzten Tag im Monat setzen
              }
              else {                                                  // wenn kein Schaltjahr
                if (xday > pgm_read_byte_far(&daytab1[xmonth - 1]))   // wenn Tag > Normaljahr-Tabelle
                  xday =  pgm_read_byte_far(&daytab2[xmonth - 1]);    // auf letzten Tag im Monat setzen
              }
              day = xday;                                             // Tage bernehmen
              month = xmonth;                                         // Monat bernehmen
              year = xyear;                                           // Jahr bernehmen
              wday = calc_wday(day, month, year);                     // Wochentag berechnen
              autosumt = xautost;                                     // Sommerzeit-Automatik bernehmen
              sei();                                                  // Interrupts wieder freigeben
              almreorg = 1;                                           // Weckzeit-Reorganisierung durchfhren
              syncstat = 1;                                           // Uhr wurde manuell gestellt
              if (autosumt) timezone = chk_summtime();                // wenn Sommerzeit-Automatik eingeschaltet -> Zeitzone einstellen
              else timezone = 0;                                      // sonst Zeitzone auf MEZ setzen
              eep_write_byte(e_autost, autosumt);                     // Sommerzeit-Automatik im EEPROM speichern
              ret_menu();                                             // in bergeordnete Men-Ebene wechseln
            }
            else {                                                    // wenn OK-Bereich nicht sichtbar
              if (dcfsyn) {                                           // wenn feste DCF77-Synchronisierungszeit
                dcftout = DCFTOUT;                                    // DCF77-Synchronisierungs-Timeout setzen
                lcd_off();                                            // LCD ausschalten
                top_menu();                                           // in normale Anzeige wechseln
              }
            }
            break;
          case 102:                                                   // Aktion "Tag erhhen"
            xday ++;                                                  // Tag erhhen
            if (xday > 31) xday = 1;                                  // wenn Endwert berschritten -> Tag auf 1 setzen
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 103:                                                   // Aktion "Tag vermindern"
            xday --;                                                  // Tag vermindern
            if (xday < 1) xday = 31;                                  // wenn Endwert unterschritten -> Tag auf 31 setzen
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 104:                                                   // Aktion "Monat erhhen"
            xmonth ++;                                                // Monat erhhen
            if (xmonth > 12) xmonth = 1;                              // wenn Endwert berschritten -> Monat auf 1 setzen
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 105:                                                   // Aktion "Monat vermindern"
            xmonth --;                                                // Monat vermindern
            if (xmonth < 1) xmonth = 12;                              // wenn Endwert unterschritten -> Monat auf 12 setzen
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 106:                                                   // Aktion "Jahr erhhen"
            if (xyear < 99) xyear ++;                                 // wenn Jahr kleiner Maximalert -> Jahr erhhen
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 107:                                                   // Aktion "Jahr vermindern"
            if (xyear > YEAR) xyear --;                               // wenn Jahr ber dem Minimalwert -> Jahr vermindern
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 108:                                                   // Aktion "Stunde erhhen"
            xhour ++;                                                 // Stunde erhhen
            if (xhour > 23) xhour = 0;                                // wenn Endwert berschritten -> Stunde auf 0 setzen
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 109:                                                   // Aktion "Stunde vermindern"
            if (xhour > 0) xhour --;                                  // wenn Stunde grer 0 -> Stunde vermindern
            else xhour = 23;                                          // sonst Stunde auf 23 setzen
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 110:                                                   // Aktion "Minute erhhen"
            xminute ++;                                               // Minute erhhen
            if (xminute > 59) xminute = 0;                            // wenn Endwert berschritten -> Minute auf 0 setzen
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 111:                                                   // Aktion "Minute vermindern"
            if (xminute > 0) xminute --;                              // wenn Minute grer 0 -> Minute vermindern
            else xminute = 59;                                        // sonst Minute auf 59 setzen
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 112:                                                   // Aktion Sommerzeit-Automatik umschalten
            xautost ++;                                               // Sommerzeit-Automatik erhhen
            if (xautost > 1) xautost = 0;                             // wenn Wertebereich berschritten -> zurcksetzen
            okflag = 1;                                               // OK-Bereich sichtbar machen
            break;
          case 113:                                                   // Aktion "Kalibrierwerte speichern und zurck zum vorherigen Men"
            if (okflag) {                                             // wenn OK-Bereich sichtbar
              ret_menu();                                             // in bergeordnete Men-Ebene wechseln
              tsleft = (tsx1 + tsx3) / 2;                             // Touchscreen X-Wert am linken Rand (Mittelwert)
              tsright = (tsx2 + tsx4) / 2;                            // Touchscreen X-Wert am rechten Rand (Mittelwert)
              tstop = (tsy1 + tsy2) / 2;                              // Touchscreen X-Wert am oberen Rand (Mittelwert)
              tsbottom = (tsy3 + tsy4) / 2;                           // Touchscreen X-Wert am unteren Rand (Mittelwert)
              eep_write_byte(e_tsleft, tsleft);                       // Touchscreen X-Wert am linken Rand im EEPROM speichern
              eep_write_byte(e_tsrigh, tsright);                      // Touchscreen X-Wert am rechten Rand im EEPROM speichern
              eep_write_byte(e_tstop, tstop);                         // Touchscreen X-Wert am oberen Rand im EEPROM speichern
              eep_write_byte(e_tsbott, tsbottom);                     // Touchscreen X-Wert am unteren Rand im EEPROM speichern
            }
            break;
          case 114:                                                   // Aktion "Sprache vermindern"
            if (lang > 0) lang --;                                    // wenn Sprache grer 0 -> Sprache vermindern
            eep_write_byte(e_lang, lang);                             // Sprache im EEPROM speichern
            break;
          case 115:                                                   // Aktion "Sprache erhhen"
            if (lang < 1) lang ++;                                    // wenn Sprache kleiner 1 -> Sprache erhhen
            eep_write_byte(e_lang, lang);                             // Sprache im EEPROM speichern
            break;
          case 116:                                                   // Aktion "LED-Modus vermindern"
            if (ledmode > 0) ledmode --;                              // wenn LED-Modus grer 0 -> LED-Modus vermindern
            LED_PRT |= _BV(LED_SIG);                                  // LED ausschalten
            eep_write_byte(e_ledmod, ledmode);                        // LED-Modus im EEPROM speichern
            break;
          case 117:                                                   // Aktion "LED-Modus erhhen"
            if (ledmode < 2) ledmode ++;                              // wenn LED-Modus kleiner 2 -> LED-Modus erhhen
            LED_PRT |= _BV(LED_SIG);                                  // LED ausschalten
            eep_write_byte(e_ledmod, ledmode);                        // LED-Modus im EEPROM speichern
            break;
          case 118:                                                   // Aktion "Minimale Helligkeit vermindern"
            if (brimin > 0) brimin --;                                // wenn Minimale Helligkeit grer 0 -> Minimale Helligkeit vermindern
            eep_write_byte(e_brimin, brimin);                         // Minimale Helligkeit im EEPROM speichern
            break;
          case 119:                                                   // Aktion "Minimale Helligkeit erhhen"
            if (brimin < 9) brimin ++;                                // wenn Minimale Helligkeit kleiner 9 -> Minimale Helligkeit erhhen
            eep_write_byte(e_brimin, brimin);                         // Minimale Helligkeit im EEPROM speichern
            break;
          case 120:                                                   // Aktion "Maximale Helligkeit vermindern"
            if (brimax > 0) brimax --;                                // wenn Maximale Helligkeit grer 0 -> Maximale Helligkeit vermindern
            eep_write_byte(e_brimax, brimax);                         // Maximale Helligkeit im EEPROM speichern
            break;
          case 121:                                                   // Aktion "Maximale Helligkeit erhhen"
            if (brimax < 9) brimax ++;                                // wenn Maximale Helligkeit kleiner 9 -> Maximale Helligkeit erhhen
            eep_write_byte(e_brimax, brimax);                         // Maximale Helligkeit im EEPROM speichern
            break;
          case 122:                                                   // Aktion "Kontrast vermindern
            if (lcdcon > LCDCONL) lcdcon --;                          // wenn Kontrast ber dem unteren Endwert -> Kontrast verminden
            set_lcdcon();                                             // Kontrast-Wert an LCD senden
            eep_write_byte(e_lcdcon, lcdcon);                         // Kontrast im EEPROM speichern
            break;
          case 123:                                                   // Aktion "Kontrast erhhen"
            if (lcdcon < LCDCONH) lcdcon ++;                          // wenn Kontrast unter dem oberen Endwert -> Kontrast erhhen
            set_lcdcon();                                             // Kontrast-Wert an LCD senden
            eep_write_byte(e_lcdcon, lcdcon);                         // Kontrast im EEPROM speichern
            break;
          case 124:                                                   // Aktion "Alarm-Datum umschalten"
            aldate ++;                                                // Alarm-Datum erhhen
            if (aldate > 1) aldate = 0;                               // wenn Endwert berschritten -> Alarm-Datum auf normal setzen
            eep_write_byte(e_aldate, aldate);                         // Alarm-Datum im EEPROM speichern
            break;
          case 125:                                                   // Aktion "DCF-Invertierung umschalten"
            dcfinv ++;                                                // DCF-Invertierung erhhen
            if (dcfinv > 1) dcfinv = 0;                               // wenn Endwert berschritten -> DCF-Invertierung auf 0 setzen
            eep_write_byte(e_dcfinv, dcfinv);                         // DCF-Invertierung im EEPROM speichern
            break;
          case 126:                                                   // Aktion "DCF-Pull-up umschalten"
            dcfpup ++;                                                // DCF-Pull-up erhhen
            if (dcfpup > 1) dcfpup = 0;                               // wenn Endwert berschritten -> DCF-Pull-up auf 0 setzen
            eep_write_byte(e_dcfpup, dcfpup);                         // DCF-Pull-up im EEPROM speichern
            if (dcfpup) DCF_PRT |= _BV(DCF_SIG);                      // wenn DCF-Pull-up ein -> Pull-up-Widerstand einschalten
            else DCF_PRT &= ~_BV(DCF_SIG);                            // wenn DCF-Pull-up aus -> Pull-up-Widerstand ausschalten
            break;
          case 127:                                                   // Aktion "Feste DCF-Synchronisierungzeit umschalten"
            dcfsyn ++;                                                // Feste DCF-Synchronisierungszeit erhhen
            if (dcfsyn > 1) dcfsyn = 0;                               // wenn Endwert berschritten -> Feste DCF-Synchronisierungszeit ausschalten
            eep_write_byte(e_dcfsyn, dcfsyn);                         // Feste DCF-Synchronisierungzeit im EEPROM speichern
            break;
          case 128:                                                   // Aktion "DCF-Synchronisierungsstunde erhhen"
            if (dcfsyn) {                                             // wenn feste DCF-Synchronisierungszeit
              dcfhou ++;                                              // DCF-Synchronisierungsstunde erhhen
              if (dcfhou > 23) dcfhou = 0;                            // wenn Endwert berschritten -> Stunde auf 0 setzen
              eep_write_byte(e_dcfhou, dcfhou);                       // DCF-Synchronisierungsstunde im EEPROM speichern
            }
            break;
          case 129:                                                   // Aktion "DCF-Synchronisierungsstunde vermindern"
            if (dcfsyn) {                                             // wenn feste DCF-Synchronisierungszeit
              if (dcfhou > 0) dcfhou --;                              // wenn Stunde grer 0 -> Stunde vermindern
              else dcfhou = 23;                                       // sonst Stunde auf 23 setzen
              eep_write_byte(e_dcfhou, dcfhou);                       // DCF-Synchronisierungsstunde im EEPROM speichern
            }
            break;
          case 130:                                                   // Aktion "Weck-Ton-Nummer vermindern"
            if (soundnr > 0) soundnr --;                              // wenn Weck-Ton-Nummer grer 0 -> Weck-Ton-Nummer vermindern
            eep_write_byte(e_soundn, soundnr);                        // Weck-Ton-Nummer im EEPROM speichern
            break;
          case 131:                                                   // Aktion "Weck-Ton-Nummer erhhen"
            if (soundnr < 9) soundnr ++;                              // wenn Weck-Ton-Nummer kleiner 9 -> Weck-Ton-Nummer erhhen
            eep_write_byte(e_soundn, soundnr);                        // Weck-Ton-Nummer im EEPROM speichern
            break;
          case 132:                                                   // Aktion "Lautstrke vermindern"
            if (volume > 0) volume --;                                // wenn Laustrke grer 0 -> Lautstrke vermindern
            eep_write_byte(e_volume, volume);                         // Lautstrke im EEPROM speichern
            break;
          case 133:                                                   // Aktion "Lautstrke erhhen"
            if (volume < 9) volume ++;                                // wenn Lautstrke kleiner 9 -> Lautstrke erhhen
            eep_write_byte(e_volume, volume);                         // Lautstrke im EEPROM speichern
            break;
          case 134:                                                   // Aktion "Lautsprecher-Status umschalten"
            soundon ++;                                               // Lautsprecher-Status erhhen
            if (soundon > 1) soundon = 0;                             // wenn Bereich berschritten -> Lautsprecher-Status auf aus setzen
            eep_write_byte(e_soundo, soundon);                        // Lautsprecher-Status im EEPROM speichern
            msgtout = MSGTOUT;                                        // zustzlich Meldungs-Timeout-Zhler starten
            msgnum = 3 + soundon;                                     // Meldung auf "Weck-Ton aus" oder "Weck-Ton ein" setzen
            break;
          case 135:                                                   // Aktion "Weck-Ton testen"
            if (almflag == 0) {                                       // wenn Weck-Signal inaktiv
              flashcnt = 0;                                           // Blinkzhler zurcksetzen
              almflag = 1;                                            // Weck-Signal aktivieren
              almtout = ALMTOUT;                                      // Alarm-Timeout starten
            }
            else almflag = 0;                                         // wenn Weck-Signal aktiv -> Weck-Signal deaktivieren
            break;
          case 136:                                                   // Aktion "Groziffern umschalten"
            digset ++;                                                // Groziffernsatz erhhen
            if (digset > 1) digset = 0;                               // wenn Bereich berschritten -> Groziffernsatz auf 0 setzen
            eep_write_byte(e_digset, digset);                         // Groziffernsatz im EEPROM speichern
            break;
          case 137:                                                   // Aktion "Lautstrke-Modus umschalten"
            vomode ++;                                                // Lautstrke-Modus erhhen
            if (vomode > 1) vomode = 0;                               // wenn Bereich berschritten -> Lautstrke-Modus auf fest setzen
            eep_write_byte(e_vomode, vomode);                         // Lautstrke-Modus im EEPROM speichern
            break;
          case 140 ... 167:                                           // Aktion "Weckzeit einstellen"
            xalnum = actpend - 140;                                   // aktuelles Alarmfeld ermitteln
            if (xalnum < almprog[curprog][56]) {                      // wenn Alarmfeldnummer kleiner als Anzahl der Tage
              xalmin = almprog[curprog][xalnum * 2];                  // Weckzeit-Minute des aktuellen Alarmfeldes kopieren
              xalhou = almprog[curprog][xalnum * 2 + 1];              // Weckzeit-Stunde des aktuellen Alarmfeldes kopieren
              if (xalhou == 24) {                                     // wenn Weckzeit inaktiv
                xalmin = 0;                                           // Minute auf 0 setzen
                xalhou = ALMHOUR;                                     // Stunde auf Voreinstellung setzen
              }
              xalmod = 0;                                             // Weck-Programm-Tabellen-Modus setzen
              sub_menu(11);                                           // Men-Wechsel -> Weckzeit einstellen
            }
            break;
          case 168 ... 171:                                           // Aktion "Weck-Programm umschalten"
            if (curprog != (actpend - 168)) {                         // wenn neues Weck-Programm ausgewhlt wurde
              curprog = actpend - 168;                                // neues Weck-Programm setzen
              eep_write_byte(e_curpro, curprog);                      // Weck-Programm im EEPROM speichern
              almreorg = 1;                                           // Weckzeit-Reorganisierung durchfhren
            }
            break;
          case 172:                                                   // Aktion "Wecker-Modus" umschalten
            mode ++;                                                  // Wecker-Modus erhhen
            if (mode > 1) mode = 0;                                   // wenn Wertebereich berschritten -> zurcksetzen
            eep_write_byte(e_mode, mode);                             // Wecker-Modus im EEPROM speichern
            almreorg = 1;                                             // Weckzeit-Reorganisierung durchfhren
            break;
          case 173:                                                   // Aktion "Stunde erhhen"
            xalhou ++;                                                // Stunde erhhen
            if (xalhou > 23) xalhou = 0;                              // wenn Endwert berschritten -> Stunde auf 0 setzen
            break;
          case 174:                                                   // Aktion "Stunde vermindern"
            if (xalhou > 0) xalhou --;                                // wenn Stunde grer 0 -> Stunde vermindern
            else xalhou = 23;                                         // sonst Stunde auf 23 setzen
            break;
          case 175:                                                   // Aktion "Minute erhhen"
            xalmin ++;                                                // Minute erhhen
            if (xalmin > 59) xalmin = 0;                              // wenn Endwert berschritten -> Minute auf 0 setzen
            break;
          case 176:                                                   // Aktion "Minute vermindern"
            if (xalmin > 0) xalmin --;                                // wenn Minute grer 0 -> Minute vermindern
            else xalmin = 59;                                         // sonst Minute auf 59 setzen
            break;
          case 177 ... 179:                                           // Aktion "Voreingestellte Weckzeit nutzen"
            if (xalmod == 0) {                                        // wenn Weck-Programm-Tabellen-Modus
              almprog[curprog][xalnum * 2] = almpre[actpend - 177][0];                            // Minute von voreingestellter Weckzeit 1 kopieren
              almprog[curprog][xalnum * 2 + 1] = almpre[actpend - 177][1];                        // Stunde von voreingestellter Weckzeit 1 kopieren
              eep_write_byte(e_almpro + curprog * 57 + xalnum * 2, almpre[actpend - 177][0]);     // Minute von voreingestellter Weckzeit 1 im EEPROM speichern
              eep_write_byte(e_almpro + curprog * 57 + xalnum * 2 + 1, almpre[actpend - 177][1]); // Stunde von voreingestellter Weckzeit 1 im EEPROM speichern
            }
            else {                                                    // wenn Direkter Weckzeit-Modus
              nxtalm[xalnum][0] = 1;                                  // Programm-Weckzeit wird durch manuelle Eingabe ersetzt
              nxtalm[xalnum][1] = almpre[actpend - 177][0];           // Minute von voreingestellter Weckzeit 1 kopieren
              nxtalm[xalnum][2] = almpre[actpend - 177][1];           // Stunde von voreingestellter Weckzeit 1 kopieren
            }            
            almreorg = 1;                                             // Weckzeit-Reorganisierung durchfhren
            ret_menu();                                               // in bergeordnete Men-Ebene wechseln
            break;
          case 180:                                                   // Aktion "Einstellbare Weckzeit nutzen"
            if (xalmod == 0) {                                        // wenn Weck-Programm-Tabellen-Modus
              almprog[curprog][xalnum * 2] = xalmin;                  // Minute von einstellbarer Weckzeit kopieren
              almprog[curprog][xalnum * 2 + 1] = xalhou;              // Stunde von einstellbarer Weckzeit kopieren
              eep_write_byte(e_almpro + curprog * 57 + xalnum * 2, xalmin);     // Minute von einstellbarer Weckzeit im EEPROM speichern
              eep_write_byte(e_almpro + curprog * 57 + xalnum * 2 + 1, xalhou); // Stunde von einstellbarer Weckzeit im EEPROM speichern
            }
            else {
              nxtalm[xalnum][0] = 1;                                  // Programm-Weckzeit wird durch manuelle Eingabe ersetzt
              nxtalm[xalnum][1] = xalmin;                             // Minute von einstellbarer Weckzeit kopieren
              nxtalm[xalnum][2] = xalhou;                             // Stunde von einstellbarer Weckzeit kopieren
            }
            almreorg = 1;                                             // Weckzeit-Reorganisierung durchfhren
            ret_menu();                                               // in bergeordnete Men-Ebene wechseln
            break;
          case 181 ... 183:                                           // Aktion "Voreingestellte Weckzeit setzen"
            almpre[actpend - 181][0] = xalmin;                        // Minute von einstellbarer Weckzeit kopieren
            almpre[actpend - 181][1] = xalhou;                        // Stunde von einstellbarer Weckzeit kopieren
            eep_write_byte(e_almpre + (actpend - 181) * 2, xalmin);     // Minute von einstellbarer Weckzeit im EEPROM speichern
            eep_write_byte(e_almpre + (actpend - 181) * 2 + 1, xalhou); // Stunde von einstellbarer Weckzeit im EEPROM speichern
            break;
          case 184:                                                   // Aktion "Weckzeit lschen"
            if (xalmod == 0) {                                        // wenn Weck-Programm-Tabellen-Modus
              almprog[curprog][xalnum * 2] = 0;                       // Minute lschen
              almprog[curprog][xalnum * 2 + 1] = 24;                  // Stunde auf 24 setzen (Weckzeit inaktiv)
              eep_write_byte(e_almpro + curprog * 57 + xalnum * 2, 0);      // Minute im EEPROM speichern
              eep_write_byte(e_almpro + curprog * 57 + xalnum * 2 + 1, 24); // Stunde im EEPROM speichern
            }
            else {
              nxtalm[xalnum][0] = 1;                                  // Programm-Weckzeit wird durch manuelle Eingabe ersetzt
              nxtalm[xalnum][1] = 0;                                  // Minute lschen
              nxtalm[xalnum][2] = 24;                                 // Stunde auf 24 setzen (Weckzeit inaktiv)
            }
            almreorg = 1;                                             // Weckzeit-Reorganisierung durchfhren
            ret_menu();                                               // in bergeordnete Men-Ebene wechseln
            break;
          case 185:                                                   // Aktion "Anzahl der Tage vermindern"
            if (almprog[curprog][56] > 1) almprog[curprog][56] --;    // wenn Anzahl der Tage grer 1 -> Anzahl vermindern
            eep_write_byte(e_almpro + curprog * 57 + 56, almprog[curprog][56]); // Anzahl der Tage im EEPROM speichern
            if (almpos >= almprog[curprog][56]) {                     // wenn Weck-Programm-Position grer als Anzahl der Tage
              almpos = 0;                                             // Weck-Programm-Position auf 0 setzen
              eep_write_byte(e_almpos, almpos);                       // Weck-Programm-Position im EEPROM speichern
            }
            almreorg = 1;                                             // Weckzeit-Reorganisierung durchfhren
            break;
          case 186:                                                   // Aktion "Anzahl der Tage erhhen"
            if (almprog[curprog][56] < 28) almprog[curprog][56] ++;   // wenn Anzahl der Tage kleiner 28 -> Anzahl erhhen
            eep_write_byte(e_almpro + curprog * 57 + 56, almprog[curprog][56]); // Anzahl der Tage im EEPROM speichern
            almreorg = 1;                                             // Weckzeit-Reorganisierung durchfhren
            break;
          case 187 ... 214:                                           // Aktion "Weck-Programm-Position setzen"
            if ((actpend - 187) < almprog[curprog][56]) {             // wenn Alarmfeldnummer kleiner als Anzahl der Tage
              almpos = actpend - 187;                                 // neue Weck-Programm-Position setzen
              if ((almprog[curprog][56] % 7) == 0) {                  // wenn Lnge des aktuellen Weck-Programms 7, 14, 21 oder 28 Tage
                almpos = (almpos / 7) * 7 + wday - 1;                 // Weck-Position auf aktuellen Wochentag der gewhlten Woche setzen
              }
              eep_write_byte(e_almpos, almpos);                       // Weck-Programm-Position im EEPROM speichern
              almreorg = 1;                                           // Weckzeit-Reorganisierung durchfhren
            }            
            break;
          case 215:                                                   // Aktion "Schlummerzeit vermindern"
            if (snooze > 0) snooze --;                                // wenn Schlummerzeit grer 0 -> Schlummerzeit vermindern
            eep_write_byte(e_snooze, snooze);                         // Schlummerzeit im EEPROM speichern
            almreorg = 1;                                             // Weckzeit-Reorganisierung durchfhren
            break;
          case 216:                                                   // Aktion "Schlummerzeit erhhen"
            if (snooze < 15) snooze ++;                               // wenn Schlummerzeit kleiner 15 -> Schlummerzeit erhhen
            eep_write_byte(e_snooze, snooze);                         // Schlummerzeit im EEPROM speichern
            almreorg = 1;                                             // Weckzeit-Reorganisierung durchfhren
            break;
          case 220 ... 222:                                           // Aktion "Weckzeit direkt einstellen"
            xalnum = actpend - 220;                                   // aktuelles Weckzeit-Feld ermitteln
            if (syncstat) {                                           // wenn gltige Uhrzeit
              xalmin = alarms[xalnum][1];                             // Minute des aktuellen Weckzeit-Feldes kopieren
              xalhou = alarms[xalnum][2];                             // Stunde des aktuellen Weckzeit-Feldes kopieren
              if (alarms[xalnum][0] == 0) {                           // wenn Weckzeit inaktiv
                xalmin = 0;                                           // Minute auf 0 setzen
                xalhou = ALMHOUR;                                     // Stunde auf Voreinstellung setzen
              }
              xalmod = 1;                                             // Direkten Weckzeit-Eingabe-Modus setzen
              sub_menu(11);                                           // Men-Wechsel -> Weckzeit einstellen
            }
            else sub_menu(10);                                        // wenn keine gltige Uhrzeit -> in alternatives Men-Wechseln
            break;
        }
      }
      else {                                                          // wenn kein Touchscreen-Bereich gefunden wurde
        tsquit = 1;                                                   // Touchscreen-Bedienung quittieren
        if (menu == 3) {                                              // wenn Touchscreen-Kalibrierung aktiv
          menutout = MENUTOUT;                                        // Men-Timeout-Zhler neu starten
          if ((tsrawx > 5) && (tsrawx < 50) && (tsrawy > 5) && (tsrawy < 70)) {       // wenn Raw-Werte im Bereich der linken oberen Ecke
            tsx1 = tsrawx; tsy1 = tsrawy;                                             // Werte zwischenspeichern
          }
          if ((tsrawx > 205) && (tsrawx < 250) && (tsrawy > 5) && (tsrawy < 70)) {    // wenn Raw-Werte im Bereich der rechten oberen Ecke
            tsx2 = tsrawx; tsy2 = tsrawy;                                             // Werte zwischenspeichern
          }
          if ((tsrawx > 5) && (tsrawx < 50) && (tsrawy > 185) && (tsrawy < 250)) {    // wenn Raw-Werte im Bereich der linken unteren Ecke
            tsx3 = tsrawx; tsy3 = tsrawy;                                             // Werte zwischenspeichern
          }
          if ((tsrawx > 205) && (tsrawx < 250) && (tsrawy > 185) && (tsrawy < 250)) { // wenn Raw-Werte im Bereich der rechten unteren Ecke
            tsx4 = tsrawx; tsy4 = tsrawy;                                             // Werte zwischenspeichern
          }
          if (tsx1 && tsx2 && tsx3 && tsx4) okflag = 1;               // wenn alle Kalibrierungspunkte gesetzt sind -> OK-Bereich sichtbar machen
        }
      }
    }
                                                                      // --- Touchscreen Auto-Repeat-Aktionen bearbeiten
    if (tsflag && tsquit && actpend && (tslong == TS_LONG) && (autcnt >= TS_AUTO)) { // wenn Touchscreen lange berhrt und quittiert wurde, eine Aktion ansteht und Auto-Repeat-Zhler abgelaufen ist
      autcnt = 0;                                                     // Auto-Repeat-Zhler zurcksetzen
      menutout = MENUTOUT;                                            // Men-Timeout-Zhler neu starten
      switch (actpend) {                                              // Aktion bei Touchscreen-Bedienung entsprechend Nummer ausfhren
        case 102:                                                     // Aktion "Tag erhhen"
          xday ++;                                                    // Tag erhhen
          if (xday > 31) xday = 1;                                    // wenn Endwert berschritten -> Tag auf 1 setzen
          okflag = 1;                                                 // OK-Bereich sichtbar machen
          break;
        case 103:                                                     // Aktion "Tag vermindern"
          xday --;                                                    // Tag vermindern
          if (xday < 1) xday = 31;                                    // wenn Endwert unterschritten -> Tag auf 31 setzen
          okflag = 1;                                                 // OK-Bereich sichtbar machen
          break;
        case 104:                                                     // Aktion "Monat erhhen"
          xmonth ++;                                                  // Monat erhhen
          if (xmonth > 12) xmonth = 1;                                // wenn Endwert berschritten -> Monat auf 1 setzen
          okflag = 1;                                                 // OK-Bereich sichtbar machen
          break;
        case 105:                                                     // Aktion "Monat vermindern"
          xmonth --;                                                  // Monat vermindern
          if (xmonth < 1) xmonth = 12;                                // wenn Endwert unterschritten -> Monat auf 12 setzen
          okflag = 1;                                                 // OK-Bereich sichtbar machen
          break;
        case 106:                                                     // Aktion "Jahr erhhen"
          if (xyear < 99) xyear ++;                                   // wenn Jahr kleiner Maximalert -> Jahr erhhen
          okflag = 1;                                                 // OK-Bereich sichtbar machen
          break;
        case 107:                                                     // Aktion "Jahr vermindern"
          if (xyear > YEAR) xyear --;                                 // wenn Jahr ber dem Minimalwert -> Jahr vermindern
          okflag = 1;                                                 // OK-Bereich sichtbar machen
          break;
        case 108:                                                     // Aktion "Stunde erhhen"
          xhour ++;                                                   // Stunde erhhen
          if (xhour > 23) xhour = 0;                                  // wenn Endwert berschritten -> Stunde auf 0 setzen
          okflag = 1;                                                 // OK-Bereich sichtbar machen
          break;
        case 109:                                                     // Aktion "Stunde vermindern"
          if (xhour > 0) xhour --;                                    // wenn Stunde grer 0 -> Stunde vermindern
          else xhour = 23;                                            // sonst Stunde auf 23 setzen
          okflag = 1;                                                 // OK-Bereich sichtbar machen
          break;
        case 110:                                                     // Aktion "Minute erhhen"
          xminute ++;                                                 // Minute erhhen
          if (xminute > 59) xminute = 0;                              // wenn Endwert berschritten -> Minute auf 0 setzen
          okflag = 1;                                                 // OK-Bereich sichtbar machen
          break;
        case 111:                                                     // Aktion "Minute vermindern"
          if (xminute > 0) xminute --;                                // wenn Minute grer 0 -> Minute vermindern
          else xminute = 59;                                          // sonst Minute auf 59 setzen
          okflag = 1;                                                 // OK-Bereich sichtbar machen
          break;
        case 118:                                                     // Aktion "Minimale Helligkeit vermindern"
          if (brimin > 0) brimin --;                                  // wenn Minimale Helligkeit grer 0 -> Minimale Helligkeit vermindern
          eep_write_byte(e_brimin, brimin);                           // Minimale Helligkeit im EEPROM speichern
          break;
        case 119:                                                     // Aktion "Minimale Helligkeit erhhen"
          if (brimin < 9) brimin ++;                                  // wenn Minimale Helligkeit kleiner 9 -> Minimale Helligkeit erhhen
          eep_write_byte(e_brimin, brimin);                           // Minimale Helligkeit im EEPROM speichern
          break;
        case 120:                                                     // Aktion "Maximale Helligkeit vermindern"
          if (brimax > 0) brimax --;                                  // wenn Maximale Helligkeit grer 0 -> Maximale Helligkeit vermindern
          eep_write_byte(e_brimax, brimax);                           // Maximale Helligkeit im EEPROM speichern
          break;
        case 121:                                                     // Aktion "Maximale Helligkeit erhhen"
          if (brimax < 9) brimax ++;                                  // wenn Maximale Helligkeit kleiner 9 -> Maximale Helligkeit erhhen
          eep_write_byte(e_brimax, brimax);                           // Maximale Helligkeit im EEPROM speichern
          break;
        case 122:                                                     // Aktion "Kontrast vermindern
          if (lcdcon > LCDCONL) lcdcon --;                            // wenn Kontrast ber dem unteren Endwert -> Kontrast verminden
          set_lcdcon();                                               // Kontrast-Wert an LCD senden
          eep_write_byte(e_lcdcon, lcdcon);                           // Kontrast im EEPROM speichern
          break;
        case 123:                                                     // Aktion "Kontrast erhhen"
          if (lcdcon < LCDCONH) lcdcon ++;                            // wenn Kontrast unter dem oberen Endwert -> Kontrast erhhen
          set_lcdcon();                                               // Kontrast-Wert an LCD senden
          eep_write_byte(e_lcdcon, lcdcon);                           // Kontrast im EEPROM speichern
          break;
        case 128:                                                     // Aktion "DCF-Synchronisierungsstunde erhhen
          if (dcfsyn) {                                               // wenn feste DCF-Synchronisierungszeit
            dcfhou ++;                                                // DCF-Synchronisierungsstunde erhhen
            if (dcfhou > 23) dcfhou = 0;                              // wenn Endwert berschritten -> Stunde auf 0 setzen
            eep_write_byte(e_dcfhou, dcfhou);                         // DCF-Synchronisierungsstunde im EEPROM speichern
          }
          break;
        case 129:                                                     // Aktion "DCF-Synchronisierungsstunde vermindern
          if (dcfsyn) {                                               // wenn feste DCF-Synchronisierungszeit
            if (dcfhou > 0) dcfhou --;                                // wenn Stunde grer 0 -> Stunde vermindern
            else dcfhou = 23;                                         // sonst Stunde auf 23 setzen
            eep_write_byte(e_dcfhou, dcfhou);                         // DCF-Synchronisierungsstunde im EEPROM speichern
          }          
          break;
        case 132:                                                     // Aktion "Lautstrke vermindern"
          if (volume > 0) volume --;                                  // wenn Laustrke grer 0 -> Lautstrke vermindern
          eep_write_byte(e_volume, volume);                           // Lautstrke im EEPROM speichern
          break;
        case 133:                                                     // Aktion "Lautstrke erhhen"
          if (volume < 9) volume ++;                                  // wenn Lautstrke kleiner 9 -> Lautstrke erhhen
          eep_write_byte(e_volume, volume);                           // Lautstrke im EEPROM speichern
          break;
        case 173:                                                     // Aktion "Stunde erhhen"
          xalhou ++;                                                  // Stunde erhhen
          if (xalhou > 23) xalhou = 0;                                // wenn Endwert berschritten -> Stunde auf 0 setzen
          break;
        case 174:                                                     // Aktion "Stunde vermindern"
          if (xalhou > 0) xalhou --;                                  // wenn Stunde grer 0 -> Stunde vermindern
          else xalhou = 23;                                           // sonst Stunde auf 23 setzen
          break;
        case 175:                                                     // Aktion "Minute erhhen"
          xalmin ++;                                                  // Minute erhhen
          if (xalmin > 59) xalmin = 0;                                // wenn Endwert berschritten -> Minute auf 0 setzen
          break;
        case 176:                                                     // Aktion "Minute vermindern"
          if (xalmin > 0) xalmin --;                                  // wenn Minute grer 0 -> Minute vermindern
          else xalmin = 59;                                           // sonst Minute auf 59 setzen
          break;
        case 185:                                                     // Aktion "Anzahl der Tage vermindern"
          if (almprog[curprog][56] > 1) almprog[curprog][56] --;      // wenn Anzahl der Tage grer 1 -> Anzahl vermindern
          eep_write_byte(e_almpro + curprog * 57 + 56, almprog[curprog][56]); // Anzahl der Tage im EEPROM speichern
          if (almpos >= almprog[curprog][56]) {                       // wenn Weck-Programm-Position grer als Anzahl der Tage
            almpos = 0;                                               // Weck-Programm-Position auf 0 setzen
            eep_write_byte(e_almpos, almpos);                         // Weck-Programm-Position im EEPROM speichern
          }
          almreorg = 1;                                               // Weckzeit-Reorganisierung durchfhren
          break;
        case 186:                                                     // Aktion "Anzahl der Tage erhhen"
          if (almprog[curprog][56] < 28) almprog[curprog][56] ++;     // wenn Anzahl der Tage kleiner 28 -> Anzahl erhhen
          eep_write_byte(e_almpro + curprog * 57 + 56, almprog[curprog][56]); // Anzahl der Tage im EEPROM speichern
          almreorg = 1;                                               // Weckzeit-Reorganisierung durchfhren
          break;
        case 215:                                                     // Aktion "Schlummerzeit vermindern"
          if (snooze > 0) snooze --;                                  // wenn Schlummerzeit grer 0 -> Schlummerzeit vermindern
          eep_write_byte(e_snooze, snooze);                           // Schlummerzeit im EEPROM speichern
          almreorg = 1;                                               // Weckzeit-Reorganisierung durchfhren
          break;
        case 216:                                                     // Aktion "Schlummerzeit erhhen"
          if (snooze < 15) snooze ++;                                 // wenn Schlummerzeit kleiner 15 -> Schlummerzeit erhhen
          eep_write_byte(e_snooze, snooze);                           // Schlummerzeit im EEPROM speichern
          almreorg = 1;                                               // Weckzeit-Reorganisierung durchfhren
          break;
      }      
    }
                                                                      // --- Sensor/Taster bearbeiten
    if (sestat && !sequit) {                                          // wenn Sensor/Taster bedient und noch nicht quittiert wurde
      sequit = 1;                                                     // Sensor/Taster-Bedienung quittieren
      if (dcftout < 255) {                                            // wenn DCF77-Synchronisierung aktiv
        dcftout = 255;                                                // DCF77-Synchronisierungs-Timeout-Zhler stoppen
        lcd_on();                                                     // LCD einschalten
      }
      else {                                                          // wenn keine DCF77-Synchronisierung aktiv
        msgtout = MSGTOUT;                                            // Meldungs-Timeout-Zhler starten
        msgnum = 0;                                                   // zunchst Meldung auf "Keine Aktion" setzen
        if (syncstat) {                                               // wenn gltige Uhrzeit vorhanden
          if (almflag) {                                              // wenn Wecksignal aktiv
            almflag = 0;                                              // Wecksignal deaktivieren
            if (curalm[0] == 2) {                                     // wenn Schlummerzeit aktiv
              curalm[0] = 3;                                          // neuen Status setzen: auf Weckzeit warten
              msgnum = 1;                                             // Meldung "Schlummern" ausgeben
            }
            if (curalm[0] == 4) {                                     // wenn Weckzeit aktiv
              curalm[0] = 5;                                          // neuen Status setzen: Alarm-Ende
              msgnum = 2;                                             // Meldung "Alarm aus" ausgeben
            }
            if (msgnum == 0) msgnum = 2;                              // Meldung "Alarm aus" ausgeben (falls Alarm ber Testfunktion ausgelst wurde)
          }
          else {                                                      // wenn Wecksignal inaktiv
            if (((curalm[0] == 1) || (curalm[0] == 3)) && ((curalm[5] + curalm[6] * 60) <= (minute + hour * 60))) { // wenn Deaktivierungszeit erreicht
              curalm[0] = 5;                                          // neuen Status setzen: Alarm-Ende
              msgnum = 2;                                             // Meldung "Alarm aus" ausgeben
            }
          }
        }                                                             // wenn keine gltige Uhrzeit vorhanden
        if (almflag) {                                                // wenn Wecksignal aktiv
          almflag = 0;                                                // Wecksignal deaktivieren
          msgnum = 2;                                                 // Meldung "Alarm aus" ausgeben (Alarm wurde ber Testfunktion ausgelst)
        }
      }
    }
                                                                      // --- DCF77 dekodieren und Uhrzeit setzen
    uint8_t errcnt;                                                   // Fehlerzhler fr DCF77-Auswertung
    uint8_t parcnt;                                                   // Zhler fr die Parittsprfung der Daten
    uint8_t dcftmp;                                                   // Zwischenspeicher fr den DCF77-Impulszhler
    uint8_t zeroflg;                                                  // Nullbyte-Flag (wenn DCF77-Puffer nur 0-Werte enthlt)

    if (dcfflag) {                                                    // wenn ein DCF77-Minutenzyklus komplett ist
      cli();                                                          // Interrupts sperren
      dcfflag = 0;                                                    // Flag fr Minutenzyklus lschen
      dcftmp = dcfimp;                                                // DCF77-Impulszhler kopieren
      dcfimp = 0;                                                     // DCF77-Impulszhler lschen
      sei();                                                          // Interrupts wieder freigeben
      if (dcftmp == 59) {                                             // wenn 59 Impulse empfangen worden sind
        errcnt = 0;                                                   // Fehlerzhler lschen
        parcnt = 0;                                                   // Parittszhler lschen
        for (i = 4; i < 12; i ++) {                                   // Minuten-Bits (im Byte 4-11) prfen
          if (dcftab[i]) parcnt ++;                                   // wenn Bit = 1 -> Parittszhler erhhen
        }
        if (parcnt & 1) errcnt ++;                                    // wenn Minuten-Prfsumme ungeradzahlig -> Fehler
        parcnt = 0;                                                   // Parittszhler lschen
        for (i = 12; i < 19; i ++) {                                  // Stunden-Bits (im Byte 12-18) prfen
          if (dcftab[i]) parcnt ++;                                   // wenn Bit = 1 -> Parittszhler erhhen
        }
        if (parcnt & 1) errcnt ++;                                    // wenn Stunden-Prfsumme ungeradzahlig -> Fehler
        parcnt = 0;                                                   // Parittszhler lschen
        for (i = 19; i < 42; i ++) {                                  // Datum-Bits (im Byte 19-41) prfen
          if (dcftab[i]) parcnt ++;                                   // wenn Bit = 1 -> Parittszhler erhhen
        }
        if (parcnt & 1) errcnt ++;                                    // wenn Datum-Prfsumme ungeradzahlig -> Fehler

        if (errcnt == 0) {                                            // wenn alle Prfbits ok -> Zeit ermitteln
          dcfbuf1[0] = dcftab[0];                                     // Zeitzone speichern
          dcfbuf1[1] = dcftab[4] + (dcftab[5] * 2) +                  // Minuten-Bits zusammenfgen
            (dcftab[6] * 4) + (dcftab[7] * 8) + (dcftab[8] * 10) +
            (dcftab[9] * 20) + (dcftab[10] * 40);                     // und Minutenwert speichern
          dcfbuf1[2] = dcftab[12] + (dcftab[13] * 2) +                // Stunden-Bits zusammenfgen
            (dcftab[14] * 4) + (dcftab[15] * 8) +
            (dcftab[16] * 10) + (dcftab[17] * 20);                    // und Stundenwert speichern
          dcfbuf1[3] = dcftab[19] + (dcftab[20] * 2) +                // Kalendertag-Bits zusammenfgen
            (dcftab[21] * 4) + (dcftab[22] * 8) +
            (dcftab[23] * 10) + (dcftab[24] * 20);                    // und Kalendertag speichern
          dcfbuf1[4] = dcftab[25] + (dcftab[26] * 2) +                // Wochentag-Bits zusammenfgen
            (dcftab[27] * 4);                                         // und Wochentag speichern
          dcfbuf1[5] = dcftab[28] + (dcftab[29] * 2) +                // Monat-Bits zusammenfgen
            (dcftab[30] * 4) + (dcftab[31] * 8) + (dcftab[32] * 10);  // und Monat speichern
          dcfbuf1[6] = dcftab[33] + (dcftab[34] * 2) +                // Jahr-Bits zusammenfgen
            (dcftab[35] * 4) + (dcftab[36] * 8) +
            (dcftab[37] * 10) + (dcftab[38] * 20) +
            (dcftab[39] * 40) + (dcftab[40] * 80);                    // und Jahr speichern
          zeroflg = 0;                                                // Nullbyte-Flag lschen
          for (i = 0; i < 7; i ++) {                                  // 7 Puffer-Bytes vergleichen
            zeroflg = zeroflg | dcfbuf1[i];                           // Puffer 1 mit Flag verknpfen
            if (dcfbuf1[i] != dcfbuf2[i]) errcnt ++;                  // Vergleich Puffer 1 mit Puffer 2
            if (dcfbuf1[i] != dcfbuf3[i]) errcnt ++;                  // Vergleich Puffer 1 mit Puffer 3
            dcfbuf3[i] = dcfbuf2[i];                                  // Kopieren Puffer 2 -> Puffer 3
            dcfbuf2[i] = dcfbuf1[i];                                  // Kopieren Puffer 1 -> Puffer 2
          }
          if (zeroflg == 0) errcnt ++;                                // wenn alle Puffer-Bytes = 0 -> Fehler
          dcfbuf2[1] ++;                                              // Puffer 2 um eine Minute erhhen
          if (dcfbuf2[1] > 59) {                                      // wenn berlauf
            dcfbuf2[1] = 0;                                           // Minuten zurcksetzen
            dcfbuf2[2] ++;                                            // Stunde erhhen
          }
          dcfbuf3[1] ++;                                              // Puffer 3 um eine Minute erhhen
          if (dcfbuf3[1] > 59) {                                      // wenn berlauf
            dcfbuf3[1] = 0;                                           // Minuten zurcksetzen
            dcfbuf3[2] ++;                                            // Stunde erhhen
          }
        }

        if (errcnt == 0) {                                            // wenn gesamte Prozedur fehlerfrei -> Uhr stellen
          cli();                                                      // Interrupts sperren
          intcnt = 0;                                                 // Interrupt-Zhler zurcksetzen
          second = 0;                                                 // Sekunden zurcksetzen
          timezone = dcfbuf1[0];                                      // Zeitzone setzen
          minute = dcfbuf1[1];                                        // Minute setzen
          hour = dcfbuf1[2];                                          // Stunde setzen
          day = dcfbuf1[3];                                           // Kalendertag setzen
          wday = dcfbuf1[4];                                          // Wochentag setzen
          month = dcfbuf1[5];                                         // Monat setzen
          year = dcfbuf1[6];                                          // Jahr setzen
          sei();                                                      // Interrupts wieder freigeben
          if (syncstat == 0) almreorg = 1;                            // wenn bisher keine gltige Uhrzeit vorhanden war -> Weckzeit-Reorganisierung durchfhren
          syncstat = 3;                                               // Uhr ist jetzt mit DCF77 synchronisiert
          alarms[3][0] = timezone;                                    // letzte Synchronisierung speichern - Zeitzone
          alarms[3][1] = minute;                                      // letzte Synchronisierung speichern - Minute
          alarms[3][2] = hour;                                        // letzte Synchronisierung speichern - Stunde
          alarms[3][3] = day;                                         // letzte Synchronisierung speichern - Tag
          alarms[3][4] = month;                                       // letzte Synchronisierung speichern - Monat
          alarms[3][5] = year;                                        // letzte Synchronisierung speichern - Jahr
          dcftout = 255;                                              // DCF77-Synchronisierungs-Timeout-Zhler stoppen
          synctout = SYNCTOUT;                                        // Synchronisierungs-Timeout-Zhler neu starten
          lcd_on();                                                   // LCD einschalten
          autosumt = 1;                                               // Sommerzeit-Automatik einschalten
          eep_write_byte(e_autost, autosumt);                         // Sommerzeit-Automatik im EEPROM speichern
        }
      }
    }
                                                                      // --- Zeit im Men "Datum und Uhrzeit" aktualisieren
    if ((menu == 1) && (!okflag)) {                                   // wenn Menu "Datum und Uhrzeit" und noch kein Wert gendert wurde
      xminute = minute;                                               // Minute fr Einstellung kopieren
      xhour = hour;                                                   // Stunde fr Einstellung kopieren
      xday = day;                                                     // Tag fr Einstellung kopieren
      xmonth = month;                                                 // Monat fr Einstellung kopieren
      xyear = year;                                                   // Jahr fr Einstellung kopieren
      xautost = autosumt;                                             // Sommerzeit-Automatik fr Einstellung kopieren
    }
                                                                      // --- Bootloader-Anfrage erkennen und Bootloader starten
    if ((lastrsrx[0] == 'U') && (lastrsrx[1] == 'U'))                 // wenn zwei aufeinander folgende 'U' empfangen wurden
    {
      WDTCSR = _BV(WDCE) | _BV(WDE);                                  // Watchdog-Reset aktivieren, Timeout 16ms
      while(1);                                                       // Endlosschleife, Watchdog-Reset unterbricht die Schleife und startet anschlieend den Bootloader
    }
                                                                      // --- Meldungs-Timeout bearbeiten
    if (msgtout == 0) msgtout = 255;                                  // wenn Meldungs-Timeout-Zhler abgelaufen -> Timeout-Zhler stoppen

                                                                      // --- DCF77-Synchronisierung bei fester Synchronisierungszeit
    if (dcftout == 0) {                                               // wenn DCF77-Synchronisierungs-Timeout-Zhler abgelaufen
      dcftout = 255;                                                  // Timeout-Zhler stoppen
      lcd_on();                                                       // LCD einschalten
    }
    if (dcfsyn) {                                                     // wenn feste DCF-Synchronisierungszeit eingeschaltet
      if ((menu == 0) && (hour == HOUR) && (minute == 2) &&           // wenn Uhr noch nie synchronisiert, normale Anzeige und Zeit = 2 min nach Systemstart
      (second == 0) && (syncstat == 0)) {
        dcftout = DCFTOUT;                                            // DCF77-Synchronisierungs-Timeout setzen
        lcd_off();                                                    // LCD ausschalten
      }
      if ((menu == 0) && (hour == dcfhou) && (minute == 0) &&         // wenn normale Anzeige und eingestellte Synchronisierungsstunde erreicht
      (second == 0)) {
        dcftout = DCFTOUT;                                            // DCF77-Synchronisierungs-Timeout setzen
        lcd_off();                                                    // LCD ausschalten
      }
    }
                                                                      // --- Synchronisierungs-Timeout bearbeiten
    if (synctout == 0) {                                              // wenn Synchronisierungs-Timeout-Zhler abgelaufen
      synctout = 255;                                                 // Timeout-Zhler stoppen
      if (syncstat > 2) syncstat = 2;                                 // wenn Uhr synchronisiert war -> Status auf "Sync erforderlich" setzen
    }
                                                                      // --- Men-Timeout bearbeiten
    if (menutout == 0) {                                              // wenn Men-Timeout-Zhler abgelaufen
      menutout = 255;                                                 // Timeout-Zhler stoppen
      if (menu != 3) top_menu();                                      // wenn nicht im Kalibrierungs-Men -> auf normale Anzeige setzen
    }
                                                                      // --- Alarm-Timeout bearbeiten
    if (almtout == 0) {                                               // wenn Alarm-Timeout-Zhler abgelaufen
      almtout = 255;                                                  // Timeout-Zhler stoppen
      if (almflag) {                                                  // wenn Weck-Signal aktiv
        almflag = 0;                                                  // Wecksignal deaktivieren
        if (curalm[0] == 2) curalm[0] = 3;                            // wenn Schlummerzeit aktiv -> neuen Statu setzen: auf Weckzeit warten
        if (curalm[0] == 4) curalm[0] = 5;                            // wenn Weckzeit aktiv -> neuen Status setzen: Alarm-Ende
      }
    }
                                                                      // --- Weck-Ton-Steuerung
    if (almflag) {                                                    // wenn Weck-Signal aktiv
      if (snstat == 0) snstat = 1;                                    // wenn Weck-Ton-Erzeugung im Ruhezustand -> Weck-Ton-Erzeugung starten
    }
    else {                                                            // wenn Weck-Signal inaktiv
      if (snstat == 2) snstat = 3;                                    // wenn Weck-Ton-Erzeugung aktiv -> Weck-Ton-Erzeugung stoppen
    }    
                                                                      // --- Weckprogramm-Steuerung / Reorganisierung
    if (syncstat && ((lday != day) || (lmonth != month) || (lyear != year))) { // wenn gltige Uhrzeit vorhanden ist und Datum der letzten Reorganisierung in der Vergangenheit liegt
      almreorg = 1;                                                            // (bei Neustart oder Tageswechsel) -> Weckzeit-Reorganisierung durchfhren
    }
    if (almreorg) {                                                   // wenn Weckzeit-Reorganisierung durchgefhrt werden soll
      almreorg = 0;                                                   // Reorganisierungs-Flag zurcksetzen
      i = 0;                                                          // Tageszhler zurcksetzen
      while ((lday != day) || (lmonth != month) || (lyear != year)) { // Schleife solange Datum in der Vergangenheit
        tday = lday;                                                  // Tag in temporre Variable kopieren
        tmonth = lmonth;                                              // Monat in temporre Variable kopieren
        tyear = lyear;                                                // Jahr in temporre Variable kopieren
        inc_date();                                                   // nchsten Tag ermitteln
        lday = tday;                                                  // neuen Tag bernehmen
        lmonth = tmonth;                                              // neuen Monat bernehmen
        lyear = tyear;                                                // neues Jahr bernehmen
        almpos ++;                                                    // Weck-Programm-Position erhhen
        if (almpos >= almprog[curprog][56]) almpos = 0;               // wenn Anzahl der Tage im aktuellen Weck-Programm berschritten -> Position auf Anfang setzen
        for (j = 0; j < 2; j ++) {                                    // nchste Weckzeiten um einen Tag verschieben
          nxtalm[j][0] = nxtalm[j + 1][0];                            // Weckzeit-Status verschieben
          nxtalm[j][1] = nxtalm[j + 1][1];                            // Weckzeit-Minute verschieben
          nxtalm[j][2] = nxtalm[j + 1][2];                            // Weckzeit-Stunde verschieben
        }
        nxtalm[2][0] = 0;                                             // Weckzeit-Status bermorgen auf Programm-Zeit setzen
        i ++;                                                         // Tageszhler erhhen
        if (i > 28) break;                                            // wenn maximale Tagesdifferenz von 28 Tagen berschritten -> Schleife abbrechen
      }
      if (i > 28) almpos = 0;                                         // wenn maximale Tagesdifferenz berschritten -> Weck-Programm-Position auf Anfang setzen (Position verloren)
      lday = day;                                                     // Datum der letzten Reorganisierung ist jetzt das aktuelle Datum
      lmonth = month;
      lyear = year;
      eep_write_byte(e_lday, lday);                                   // Letzte Alarm-Reorganisierung Tag im EEPROM speichern
      eep_write_byte(e_lmonth, lmonth);                               // Letzte Alarm-Reorganisierung Monat im EEPROM speichern
      eep_write_byte(e_lyear, lyear);                                 // Letzte Alarm-Reorganisierung Jahr im EEPROM speichern
      eep_write_byte(e_almpos, almpos);                               // Weck-Programm-Position im EEPROM speichern

      tday = day;                                                     // aktuellen Tag in temporre Variable kopieren
      tmonth = month;                                                 // aktuellen Monat in temporre Variable kopieren
      tyear = year;                                                   // aktuelles Jahr in temporre Variable kopieren
      talmpos = almpos;                                               // aktuelle Weck-Programm-Position in temporre Variablen kopieren
      for (i = 0; i < 3; i ++) {                                      // die nchsten 3 Weckzeiten aktualisieren
        if (mode == 0) {                                              // wenn Wecker im Programm-Modus
          alarms[i][0] = 1;                                           // Weckzeit zunchst aktivieren
          alarms[i][1] = almprog[curprog][talmpos * 2];               // Minute aus Weck-Programm holen
          alarms[i][2] = almprog[curprog][talmpos * 2 + 1];           // Stunde aus Weck-Programm holen
          if (alarms[i][2] > 23) alarms[i][0] = 0;                    // wenn Stunde grer 23 (keine Weckzeit) -> Weckzeit deaktivieren
        }
        else alarms[i][0] = 0;                                        // wenn Wecker im Manuellen Modus -> Weckzeit deaktivieren
        alarms[i][3] = tday;                                          // Tag setzen
        alarms[i][4] = tmonth;                                        // Monat setzen
        alarms[i][5] = tyear;                                         // Jahr setzen
        if (nxtalm[i][0]) {                                           // wenn nchste Weckzeit manuell eingegeben wurde
          alarms[i][0] = 1;                                           // Weckzeit zunchst aktivieren
          alarms[i][1] = nxtalm[i][1];                                // manuell eingegebene Minute kopieren
          alarms[i][2] = nxtalm[i][2];                                // manuell eingegebene Stunde kopieren
          if (nxtalm[i][2] > 23) alarms[i][0] = 0;                    // wenn manuell eingegebene Weckzeit inaktiv -> Weckzeit deaktivieren
        }
        inc_date();                                                   // nchsten Tag ermitteln
        talmpos ++;                                                   // temporre Weck-Programm-Position erhhen
        if (talmpos >= almprog[curprog][56]) talmpos = 0;             // wenn Anzahl der Tage im aktuellen Weck-Programm berschritten -> Position auf Anfang setzen
        for (j = 0; j < 3; j ++) {                                    // 3 Werte im EEPROM speichern
          eep_write_byte(e_nxtalm + i * 3 + j, nxtalm[i][j]);         // Weckzeit-Status in EEPROM speichern
        }        
      }                                                               // aktuelle Schlummer- und Weckzeit setzen
      curalm[0] = alarms[0][0];                                       // Status von heutiger Weckzeit bernehmen
      curalm[1] = alarms[0][1];                                       // Weckzeit-Minute von heutiger Weckzeit bernehmen
      curalm[2] = alarms[0][2];                                       // Weckzeit-Stunde von heutiger Weckzeit bernehmen
      curalm[3] = alarms[0][1];                                       // Schlummerzeit-Minute von heutiger Weckzeit bernehmen
      curalm[4] = alarms[0][2];                                       // Schlummerzeit-Stunde von heutiger Weckzeit bernehmen
      if (snooze) {                                                   // wenn Schlummermodus aktiviert
        if ((curalm[3] + curalm[4] * 60) > snooze) {                  // wenn Schlummerzeit nach 0:00 Uhr
          if (curalm[3] < snooze) {                                   // wenn Minute < Schlummerzeit (bertrag)
            curalm[3] += 60;                                          // Minute um 60 erhhen
            curalm[4] --;                                             // Stunde vermindern
          }
          curalm[3] -= snooze;                                        // Schlummerzeit von Minute subtrahieren
        }
        else {                                                        // wenn Schlummerzeit vor 0:00 Uhr
          curalm[3] = 0;                                              // Schlummerzeit-Minute auf 0 setzen
          curalm[4] = 0;                                              // Schlummerzeit-Stunde auf 0 setzen
        }
      }
      curalm[5] = curalm[3];                                          // Deaktivierungszeit-Minute von Schlummerzeit bernehmen
      curalm[6] = curalm[4];                                          // Deaktivierungszeit-Stunde von Schlummerzeit bernehmen
      if ((curalm[5] + curalm[6] * 60) > ALMDIS) {                    // wenn Deaktivierungszeit nach 0:00 Uhr
        if (curalm[5] < ALMDIS) {                                     // wenn Minute < Deaktivierungszeit (bertrag)
          curalm[5] += 60;                                            // Minute um 60 erhhen
          curalm[6] --;                                               // Stunde vermindern
        }
        curalm[5] -= ALMDIS;                                          // Deaktivierungszeit von Minute subtrahieren
      }
      else {                                                          // wenn Deaktivierungszeit vor 0:00 Uhr
        curalm[5] = 0;                                                // Deaktivierungszeit-Minute auf 0 setzen
        curalm[6] = 0;                                                // Deaktivierungszeit-Stunde auf 0 setzen
      }
      if (curalm[0]) {                                                // wenn Weckzeit aktiviert
        curalm[0] = 0;                                                // Weckzeit zunchst deaktivieren
        if ((curalm[1] + curalm[2] * 60) > (minute + hour * 60)) {    // wenn Weckzeit nach aktueller Zeit
          if (snooze) curalm[0] = 1;                                  // wenn Schlummermodus aktiviert -> auf Schlummerzeit warten
          else curalm[0] = 3;                                         // sonst auf normale Weckzeit warten
        }
        if ((curalm[1] == 0) && (curalm[2] == 0) && (minute == 0) && (hour == 0)) curalm[0] = 3; // Sonderfall Weckzeit 0:00 und Uhrzeit 0:00 Uhr -> auf normale Weckzeit warten
      }
    }
                                                                      // --- Schlummer- und Weck-Signal auslsen
    if (syncstat) {                                                   // wenn gltige Uhrzeit vorhanden ist
      if (!almflag && (curalm[0] == 1) && 
          (curalm[3] == minute) && (curalm[4] == hour)) {             // wenn kein Wecksignal, auf Schlummerzeit gewartet wird und Schlummerzeit erreicht ist
        curalm[0] = 2;                                                // neuen Status setzen: Schlummerzeit aktiv
        flashcnt = 0;                                                 // Blinkzhler zurcksetzen
        almflag = 1;                                                  // Schlummer-Signal aktivieren
        almtout = ALMTOUT;                                            // Alarm-Timeout starten
      }
      if (!almflag && ((curalm[0] == 1) || (curalm[0] == 3)) &&
          (curalm[1] == minute) && (curalm[2] == hour)) {             // wenn kein Weck-Signal, auf Weck- oder Schlummerzeit gewartet wird und Weckzeit erreicht ist
        curalm[0] = 4;                                                // neuen Status setzen: Weckzeit aktiv
        flashcnt = 0;                                                 // Blinkzhler zurcksetzen
        almflag = 1;                                                  // Weck-Signal aktivieren
        almtout = ALMTOUT;                                            // Alarm-Timeout starten
      }
    }
  }
}
